<?php
/**
 * Metabox class.
 *
 * @since 1.0.8
 *
 * @package Envira_Audio
 * @author  Envira Team
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Metabox class.
 *
 * @since 1.0.8
 *
 * @package Envira_Audio
 * @author  Envira Team
 */
class Envira_Audio_Metaboxes {

	/**
	 * Holds the class object.
	 *
	 * @since 1.0.8
	 *
	 * @var object
	 */
	public static $instance;

	/**
	 * Path to the file.
	 *
	 * @since 1.0.8
	 *
	 * @var string
	 */
	public $file = __FILE__;

	/**
	 * The base class object.
	 *
	 * @var Envira_Audio|object
	 */
	private $base;

	/**
	 * Primary class constructor.
	 *
	 * @since 1.0.8
	 */
	public function __construct() {

		$this->base = Envira_Audio::get_instance();

		add_action( 'envira_gallery_metabox_scripts', [ $this, 'metabox_scripts' ] );
		add_action( 'envira_albums_metabox_scripts', [ $this, 'metabox_scripts' ] );

		// Gallery.
		add_filter( 'envira_gallery_tab_nav', [ $this, 'register_tabs' ] );
		add_action( 'envira_gallery_tab_audio', [ $this, 'audio_tab' ] );
		add_filter( 'envira_gallery_save_settings', [ $this, 'gallery_save' ], 10, 2 );

		// Album.
		add_filter( 'envira_albums_tab_nav', [ $this, 'register_tabs' ] );
		add_action( 'envira_albums_tab_audio', [ $this, 'audio_tab' ] );
		add_filter( 'envira_albums_save_settings', [ $this, 'album_save' ], 10, 2 );
	}

	/**
	 * Initializes scripts for the metabox admin.
	 *
	 * @since 1.0.0
	 */
	public function metabox_scripts() {
		// Conditional Fields.
		wp_register_script( $this->base->plugin_slug . '-conditional-fields-script', plugins_url( 'assets/js/min/conditional-fields-min.js', $this->base->file ), [ 'jquery', Envira_Gallery::get_instance()->plugin_slug . '-conditional-fields-script' ], $this->base->version, true );
		wp_enqueue_script( $this->base->plugin_slug . '-conditional-fields-script' );

			wp_enqueue_script( $this->base->plugin_slug . '-metabox', plugins_url( 'assets/js/metabox.js', $this->base->file ), [ 'jquery' ], $this->base->version, true );
	}

	/**
	 * Filters in a new tab for the addon.
	 *
	 * @since 1.0.8
	 *
	 * @param array $tabs  Array of default tab values.
	 * @return array $tabs Amended array of default tab values.
	 */
	public function register_tabs( $tabs ) {

		$tabs['audio'] = __( 'Audio', 'envira-audio' );
		return $tabs;
	}

	/**
	 * Callback for displaying the UI for setting gallery audio options.
	 *
	 * @since 1.0.8
	 *
	 * @param object $post The current post object.
	 */
	public function audio_tab( $post ) {

		// Get post type so we load the correct metabox instance and define the input field names
		// Input field names vary depending on whether we are editing a Gallery or Album.
		$post_type = get_post_type( $post );
		switch ( $post_type ) {

			/**
			* Gallery
			*/
			case 'envira':
				$instance = Envira_Gallery_Metaboxes::get_instance();
				$key      = '_envira_gallery';
				$type     = 'Gallery';
				break;

			/**
			* Album
			*/
			case 'envira_album':
				$instance = Envira_Albums_Metaboxes::get_instance();
				$key      = '_eg_album_data[config]';
				$type     = 'Album';
				break;
		}

		wp_nonce_field( 'envira_audio_save_settings', 'envira_audio_nonce' );

		?>
		<div id="envira-audio">
			<p class="envira-intro">
				<?php esc_html_e( 'Audio Lightbox Settings', 'envira-audio' ); ?>
				<small>
					<?php esc_html_e( 'The settings below adjust the Audio options for the Lightbox output.', 'envira-audio' ); ?>
					<br />
					<?php esc_html_e( 'Need some help?', 'envira-audio' ); ?>
					<a href="http://enviragallery.com/docs/audio-addon/" class="envira-doc" target="_blank">
						<?php esc_html_e( 'Read the Documentation', 'envira-audio' ); ?>
					</a>
					or
					<a href="https://www.youtube.com/embed/BnVFQP2_kac/?rel=0" class="envira-video" target="_blank">
						<?php esc_html_e( 'Watch a Video', 'envira-audio' ); ?>
					</a>

				</small>
			</p>
			<table class="form-table">
				<tbody>
					<tr id="envira-config-audio-box">
						<th scope="row">
							<label for="envira-config-audio">
								<?php
								// translators: Audio type.
								printf( esc_html__( 'Enable %s Audio?', 'envira-audio' ), esc_html( $type ) );
								?>
							</label>
						</th>
						<td>
							<input id="envira-config-audio" type="checkbox" name="<?php echo esc_attr( $key ); ?>[audio]" value="<?php echo esc_attr( $instance->get_config( 'audio', $instance->get_config_default( 'audio' ) ) ); ?>" <?php checked( $instance->get_config( 'audio', $instance->get_config_default( 'audio' ) ), 1 ); ?> />
							<span class="description">
								<?php
								// translators: Audio type.
								printf( esc_html__( 'Enables or disables the %s lightbox audio.', 'envira-audio' ), esc_html( $type ) );
								?>
							</span>
						</td>
					</tr>

					<tr id="envira-audio-file-box">
						<th scope="row">
							<label for="envira-audio-file"><?php esc_html_e( 'Audio File (MP3)', 'envira-audio' ); ?></label>
						</th>
						<td>
							<!-- Stores ID and file URL -->
							<input type="hidden" id="envira-audio-file-ID" name="<?php echo esc_html( $key ); ?>[audio_file_id]" value="<?php echo esc_html( $instance->get_config( 'audio_file_id', $instance->get_config_default( 'audio_file_id' ) ) ); ?>" />

							<!-- Existing File -->
							<span class="image envira-audio-file">
								<?php
								$file_id = $instance->get_config( 'audio_file_id', $instance->get_config_default( 'audio_file_id' ) );

								if ( ! empty( $file_id ) ) {
									$file          = get_post( $file_id, 'ARRAY_A' );
									$file_metadata = get_post_meta( $file_id );
									// Get file.
									if ( is_array( $file_metadata ) ) {

										// phpcs:ignore
										$audio_metadata = unserialize( $file_metadata['_wp_attachment_metadata'][0] );

										?>
										<div class="audio-file-name"><a target="_blank" href="<?php echo esc_url( $file['guid'] ); ?>"><?php echo esc_html( $file['post_title'] ); ?></a> (<?php echo esc_html( $audio_metadata['length_formatted'] ); ?>)</div>
										<?php
									}
								}
								?>
							</span>

							<span class="wp-media-buttons envira-audio-button" style="float:none;">
								<a href="#" class="button insert-media-url add_media" data-id="envira-audio-file-ID">
									<span class="wp-media-buttons-icon"></span>
									<?php

									if ( ! empty( $file_id ) ) {
										esc_html_e( 'Change Audio File', 'envira-audio' );
									} else {
										esc_html_e( 'Choose Audio File', 'envira-audio' );
									}
									?>
								</a>
							</span>

							<br/><span class="description"><?php esc_html_e( 'The file to use as the audio. Only MP3 files are supported, and the user\'s browser might prevent audio from playing from your site.', 'envira-audio' ); ?></span>
						</td>
					</tr>

					<tr id="envira-config-audio-autoplay-box">
						<th scope="row">
							<label for="envira-config-audio-autoplay"><?php esc_html_e( 'Autoplay the Audio?', 'envira-audio' ); ?></label>
						</th>
						<td>
							<input id="envira-config-audio-autoplay" type="checkbox" name="<?php echo esc_attr( $key ); ?>[audio_autoplay]" value="<?php echo esc_attr( $instance->get_config( 'audio_autoplay', $instance->get_config_default( 'autoplay' ) ) ); ?>" <?php checked( $instance->get_config( 'audio_autoplay', $instance->get_config_default( 'audio_autoplay' ) ), 1 ); ?> />
							<span class="description"><?php esc_html_e( 'Enables or disables autoplaying the audio on lightbox open.', 'envira-audio' ); ?></span>
						</td>
					</tr>

					<tr id="envira-config-audio-loop-box">
						<th scope="row">
							<label for="envira-config-audio-loop"><?php esc_html_e( 'Loop the Audio?', 'envira-audio' ); ?></label>
						</th>
						<td>
							<input id="envira-config-audio-autoplay" type="checkbox" name="<?php echo esc_attr( $key ); ?>[audio_loop]" value="<?php echo esc_attr( $instance->get_config( 'audio_loop', $instance->get_config_default( 'audio_loop' ) ) ); ?>" <?php checked( $instance->get_config( 'audio_loop', $instance->get_config_default( 'audio_loop' ) ), 1 ); ?> />
							<span class="description"><?php esc_html_e( 'Enables or disables looping of audio.', 'envira-audio' ); ?></span>
						</td>
					</tr>

					<?php do_action( 'envira_gallery_audio_box', $post ); ?>
				</tbody>
			</table>
		</div>
		<?php
	}

	/**
	 * Saves the addon setting.
	 *
	 * @since 1.0.8
	 *
	 * @param array $settings  Array of settings to be saved.
	 * @param int   $post_id     The current post ID.
	 * @return array $settings Amended array of settings to be saved.
	 */
	public function gallery_save( $settings, $post_id ) {

		if (
			! isset( $_POST['_envira_gallery'], $_POST['envira_audio_nonce'] )
			|| ! wp_verify_nonce( sanitize_key( $_POST['envira_audio_nonce'] ), 'envira_audio_save_settings' )
		) {
			return $settings;
		}

		$settings['config']['audio']          = isset( $_POST['_envira_gallery']['audio'] ) ? 1 : 0;
		$settings['config']['audio_file_id']  = ( isset( $_POST['_envira_gallery']['audio_file_id'] ) ) ? absint( $_POST['_envira_gallery']['audio_file_id'] ) : false;
		$settings['config']['audio_autoplay'] = isset( $_POST['_envira_gallery']['audio_autoplay'] ) ? 1 : 0;
		$settings['config']['audio_loop']     = isset( $_POST['_envira_gallery']['audio_loop'] ) ? 1 : 0;

		return $settings;
	}

	/**
	 * Saves the addon setting for Albms
	 *
	 * @since 1.0.8
	 *
	 * @param array $settings  Array of settings to be saved.
	 * @param int   $post_id     The current post ID.
	 * @return array $settings Amended array of settings to be saved.
	 */
	public function album_save( $settings, $post_id ) {

		if (
			! isset( $_POST['_eg_album_data']['config'], $_POST['envira_audio_nonce'] )
			|| ! wp_verify_nonce( sanitize_key( $_POST['envira_audio_nonce'] ), 'envira_audio_save_settings' )
		) {
			return $settings;
		}

		$settings['config']['audio']          = isset( $_POST['_eg_album_data']['config']['audio'] ) ? 1 : 0;
		$settings['config']['audio_autoplay'] = isset( $_POST['_eg_album_data']['config']['audio_autoplay'] ) ? 1 : 0;
		$settings['config']['audio_loop']     = isset( $_POST['_eg_album_data']['config']['audio_loop'] ) ? 1 : 0;
		$settings['config']['audio_file_id']  = ( isset( $_POST['_eg_album_data']['config']['audio_file_id'] ) ) ? absint( $_POST['_eg_album_data']['config']['audio_file_id'] ) : false;
		return $settings;
	}

	/**
	 * Returns the singleton instance of the class.
	 *
	 * @since 1.0.8
	 *
	 * @return object The Envira_Audio_Metaboxes object.
	 */
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Envira_Audio_Metaboxes ) ) {
			self::$instance = new Envira_Audio_Metaboxes();
		}

		return self::$instance;
	}
}

// Load the metabox class.
$envira_audio_metabox = Envira_Audio_Metaboxes::get_instance();
