<?php
/**
 * Plugin Name: Envira Gallery - Beaver Builder Addon
 * Plugin URI:  http://enviragallery.com
 * Description: Adds a custom Envira module to the Beaver Builder Page Builder.
 * Author:      Envira Gallery Team
 * Author URI:  http://enviragallery.com
 * Version:     1.1.2
 * Text Domain: envira-beaver-builder
 * Domain Path: languages
 *
 * @package Envira Gallery
 * @subpackage Envira Beaver Builder
 *
 * Envira Gallery is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * any later version.
 *
 * Envira Gallery is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Envira Gallery. If not, see <http://www.gnu.org/licenses/>.
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( function_exists( 'envira_license_checker' ) && false === envira_license_checker() ) {
	return false;
}

use Envira\Utils\Updater as Updater;

define( 'ENVIRA_BB_DIR', plugin_dir_path( __FILE__ ) );
define( 'ENVIRA_BB_URL', plugins_url( '/', __FILE__ ) );

/**
 * Main Beaver Builder Envira Class
 *
 * The init class that runs the Elementor Envira plugin.
 * Intended To make sure that the plugin's minimum requirements are met.
 *
 * You should only modify the constants to match your plugin's needs.
 *
 * Any custom code should go inside Plugin Class in the plugin.php file.
 *
 * @since 1.0.0
 */
class Beaver_Builder_Envira {

	/**
	 * Holds the class object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public static $instance;

	/**
	 * Plugin version, used for cache-busting of style and script file references.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	public $version = '1.1.2';

	/**
	 * The name of the plugin.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	public $plugin_name = 'Envira Beaver Builder';

	/**
	 * Unique plugin slug identifier.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	public $plugin_slug = 'envira-beaver-builder';

	/**
	 * Plugin file.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	public $file = __FILE__;

	/**
	 * Constructor
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {

		// Load the plugin textdomain.
		add_action( 'plugins_loaded', [ $this, 'load_plugin_textdomain' ] );

		// Init Plugin.
		add_action( 'plugins_loaded', [ $this, 'setup_hooks' ] );

		// Load the updater.
		add_action( 'envira_gallery_updater', [ $this, 'updater' ], 10, 1 );

	}

	/**
	 * Loads the plugin textdomain for translation.
	 *
	 * @since 1.0.0
	 */
	public function load_plugin_textdomain() {

		load_plugin_textdomain( $this->plugin_slug, false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );

	}

	/**
	 * Setup hooks if the builder is installed and activated.
	 */
	public function setup_hooks() {

		if ( ! class_exists( 'FLBuilder' ) ) {
			return;
		}

		// Load custom modules.
		add_action( 'init', [ $this, 'load_modules' ] );

		// Register custom fields.
		add_filter( 'fl_builder_custom_fields', [ $this, 'register_fields' ] );

		// Enqueue custom field assets.
		add_action( 'init', [ $this, 'enqueue_field_assets' ] );
	}

	/**
	 * Loads our custom modules.
	 */
	public function load_modules() {
		require_once ENVIRA_BB_DIR . 'modules/envira/includes/global.php';
		require_once ENVIRA_BB_DIR . 'modules/envira-basic.php';
	}

	/**
	 * Registers our custom fields.
	 *
	 * @param array $fields The fields.
	 */
	public function register_fields( $fields ) {
		$fields['checkbox-isotope']  = ENVIRA_BB_DIR . 'fields/checkbox-isotope.php';
		$fields['checkbox-lightbox'] = ENVIRA_BB_DIR . 'fields/checkbox-lightbox.php';
		return $fields;
	}

	/**
	 * Initializes the addon updater.
	 *
	 * @since 1.0.0
	 *
	 * @param string $key The user license key.
	 */
	public function updater( $key ) {

		$args = [
			'plugin_name' => $this->plugin_name,
			'plugin_slug' => $this->plugin_slug,
			'plugin_path' => plugin_basename( __FILE__ ),
			'plugin_url'  => trailingslashit( WP_PLUGIN_URL ) . $this->plugin_slug,
			'remote_url'  => 'https://enviragallery.com/',
			'version'     => $this->version,
			'key'         => $key,
		];

		$updater = new Updater( $args );

	}

	/**
	 * Enqueues our custom field assets only if the builder UI is active.
	 */
	public function enqueue_field_assets() {

		if ( ! FLBuilderModel::is_builder_active() ) {
			return;
		}

		$version = ( defined( 'ENVIRA_DEBUG' ) && 'true' === ENVIRA_DEBUG ) ? $version = time() . '-' . ENVIRA_VERSION : ENVIRA_VERSION;

		wp_enqueue_style( 'envira-bb-custom-fields', ENVIRA_BB_URL . 'assets/css/fields.css', [], $version );
	}

}

// Instantiate Beaver_Builder_Envira.
new Beaver_Builder_Envira();
