<?php
/**
 * Envira Plugin class.
 *
 * @since 1.0.0
 *
 * @package Envira_Lightroom
 * @author  Envira Team
 */

namespace ElementorEnvira;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Envira class.
 *
 * @since 1.0.0
 *
 * @package Envira_Elementor
 * @author  Envira Team
 */
class EnviraElementorPlugin {

	/**
	 * Instance
	 *
	 * @since 1.0.0
	 * @access private
	 * @static
	 *
	 * @var Plugin The single instance of the class.
	 */
	private static $_instance = null;

	/**
	 *  Plugin class constructor
	 *
	 * @since 1.2.0
	 * @access public
	 */
	public function __construct() {

		// Register widget scripts.
		add_action( 'elementor/frontend/after_register_scripts', [ $this, 'widget_scripts' ] );

		// Register widgets.
		add_action( 'elementor/widgets/register', [ $this, 'register_widgets' ] );

		add_action( 'elementor/elements/categories_registered', [ $this, 'add_elementor_widget_categories' ] );
		add_action( 'wp_ajax_envira_gallery_elementor_generate_gallery_html', [ $this, 'render_gallery' ], 10 );
		add_action( 'wp_ajax_nopriv_envira_gallery_elementor_generate_gallery_html', [ $this, 'render_gallery' ], 10 );
	}


	/**
	 *  Render the gallery.
	 *
	 * @since 1.2.0
	 * @access public
	 */
	public function render_gallery() {
		if ( ! isset( $_POST['nonce'], $_REQUEST['gallery_id'] ) || ! wp_verify_nonce( sanitize_key( $_POST['nonce'] ), 'envira-elementor-nonce' ) ) {
			return;
		}

		$gallery_id = intval( $_REQUEST['gallery_id'] );

		if ( ! $gallery_id ) {
			return;
		}

		if ( defined( 'ENVIRA_DEBUG' ) && ENVIRA_DEBUG ) {
			error_log( 'render_gallery - start' );
			error_log( print_r( $_REQUEST, true ) );
		}

		$overwrite = isset( $_REQUEST['overwrite'] ) && 'false' !== $_REQUEST['overwrite'];

		if ( false === $overwrite ) {
			$_REQUEST = false;
		}

		$layout = isset( $_REQUEST['layout'] ) && 'false' !== $_REQUEST['layout'] ? sanitize_text_field( wp_unslash( $_REQUEST['layout'] ) ) : '';
		if ( 'automatic' === $layout ) {
			$columns = '0';
		} else {
			$columns = isset( $_REQUEST['columns'] ) && 'false' !== $_REQUEST['columns'] ? sanitize_text_field( wp_unslash( $_REQUEST['columns'] ) ) : '';
		}
		$lightbox_theme   = isset( $_REQUEST['lightbox_theme'] ) && 'false' !== $_REQUEST['lightbox_theme'] ? sanitize_text_field( wp_unslash( $_REQUEST['lightbox_theme'] ) ) : '';
		$lightbox_enabled = isset( $_REQUEST['lightbox'] ) && 'false' !== $_REQUEST['lightbox'] ? sanitize_text_field( wp_unslash( $_REQUEST['lightbox'] ) ) : '';
		$margin           = isset( $_REQUEST['custom_margin'] ) && 'false' !== $_REQUEST['custom_margin'] ? sanitize_text_field( wp_unslash( $_REQUEST['custom_margin'] ) ) : '';

		ob_start();

		// Check if variables exist.
		if ( false === $gallery_id ) {
			$output = 'Select a gallery from the settings dropdown.';
		} else {

			// Render shortcode.
			$shortcode = '[envira-gallery id="' . $gallery_id . '" type="elementor" cache="false" layout="' . $layout . '" columns="' . $columns . '" lightbox_theme="' . $lightbox_theme . '" lightbox_enabled="' . $lightbox_enabled . '" margin="' . $margin . '" ]';

			echo do_shortcode( $shortcode );
			$output = ob_get_clean();

			if ( is_envira_debug_on() ) {
				error_log( $shortcode );
			}
		}

		// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped -- Intentional.
		echo $output;

		// Update gallery with provided settings.
		if ( true === $overwrite ) {
			$gallery_data = envira_get_gallery( $gallery_id, true ); // flush cache while we do this with 'true'.
			if ( ! empty( $gallery_data['config'] ) ) {
				$gallery_data['config']['layout']           = $layout;
				$gallery_data['config']['columns']          = $columns;
				$gallery_data['config']['margin']           = $margin;
				$gallery_data['config']['lightbox_enabled'] = $lightbox_enabled;
				$gallery_data['config']['lightbox_theme']   = $lightbox_theme;
			}
			update_post_meta( $gallery_id, '_eg_gallery_data', $gallery_data );
		}

		if ( $overwrite && is_envira_debug_on() ) {
			error_log( print_r( $gallery_data, true ) );
			error_log( 'render_gallery - stop' );
		}

		exit;
	}


	/**
	 * Ensures only one instance of the class is loaded or can be loaded.
	 *
	 * @since 1.2.0
	 * @access public
	 *
	 * @return Plugin An instance of the class.
	 */
	public static function instance() {

		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}

		return self::$_instance;
	}

	/**
	 * Load required plugin core files.
	 *
	 * @since 1.2.0
	 * @access public
	 */
	public function widget_scripts() {

		wp_enqueue_script( ENVIRA_ELEMENTOR_SLUG . 'ajax-script' );

		wp_localize_script( ENVIRA_ELEMENTOR_SLUG . 'ajax-script', 'envira_elementor', [ 'ajax_url' => admin_url( 'admin-ajax.php' ) ] );
	}

	/**
	 *
	 * Load widgets files
	 *
	 * @since 1.2.0
	 * @access private
	 */
	private function include_widgets_files() {

		require_once __DIR__ . '/widgets/envira.php';
	}

	/**
	 * Register new Elementor widgets.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param \Elementor\Widgets_Manager $widgets_manager Elementor widget manager.
	 */
	public function register_widgets( $widgets_manager ) {

		// Its is now safe to include Widgets files.
		$this->include_widgets_files();

		$widgets_manager->register( new \ElementorEnvira\Widgets\Envira() );

		// Register Widgets.
	}

	/**
	 * Register new Elementor widgets categories.
	 *
	 * @param array $elements_manager The elements.
	 * @since 1.0.0
	 * @access public
	 */
	public function add_elementor_widget_categories( $elements_manager ) {

		$elements_manager->add_category(
			'envira-gallery',
			[
				'title' => __( 'Envira Gallery', 'envira-elementor' ),
				'icon'  => 'fa fa-envira',
			]
		);
	}
}

// Instantiate Plugin Class.
EnviraElementorPlugin::instance();
