<?php
/**
 * Envira Widget class.
 *
 * @since 1.0.0
 *
 * @package Envira_Elementor
 * @author  Envira Team
 */

namespace ElementorEnvira\Widgets;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Plugin;
use Envira\Frontend\Shortcode;

/**
 * Envira class.
 *
 * @since 1.0.0
 *
 * @package Envira_Elementor
 * @author  Envira Team
 */
class Envira extends Widget_Base {

	/**
	 * Primary class constructor.
	 *
	 * @since 1.0.0
	 *
	 * @param   array $data Data.
	 * @param   array $args Settings.
	 */
	public function __construct( $data = [], $args = null ) {

		/* styles */

		$version = ( defined( 'ENVIRA_DEBUG' ) && 'true' === ENVIRA_DEBUG ) ? $version = time() . '-' . ENVIRA_VERSION : ENVIRA_VERSION;

		wp_enqueue_style( ENVIRA_SLUG . '-style' );
		wp_enqueue_style( ENVIRA_SLUG . '-jgallery' );

		wp_register_style( ENVIRA_ELEMENTOR_SLUG . '-style', plugins_url( 'assets/css/admin.css', ENVIRA_ELEMENTOR_FILE ), false, $version );
		wp_enqueue_style( ENVIRA_ELEMENTOR_SLUG . '-style' );

		add_filter( 'elementor/widget/print_template', [ $this, 'skin_print_template' ], 10, 2 );

		/* scripts */
		wp_localize_script(
			ENVIRA_SLUG . '-script',
			'envira_gallery',
			[
				'debug'      => false,
				'll_delay'   => 500,
				'll_initial' => 'false',
				'll'         => 'false',
				'mobile'     => false,
				'nonce'      => wp_create_nonce( 'envira-elementor-nonce' ),
			]
		);

		wp_register_script( ENVIRA_ELEMENTOR_SLUG . 'ajax-script', plugins_url( 'assets/js/min/envira-min.js', ENVIRA_ELEMENTOR_FILE ), [ 'jquery' ], $version, true );

		parent::__construct( $data, $args );
	}

	/**
	 * Retrieve the widget name.
	 *
	 * @since 1.1.0
	 *
	 * @access public
	 *
	 * @return string Widget name.
	 */
	public function get_name() {
		return 'envira';
	}

	/**
	 * Retrieve the widget title.
	 *
	 * @since 1.1.0
	 *
	 * @access public
	 *
	 * @return string Widget title.
	 */
	public function get_title() {

		return __( 'Envira', 'envira-elementor' );
	}

	/**
	 * Register Scripts Depends.
	 *
	 * @since 1.1.0
	 *
	 * @access public
	 */
	public function get_script_depends() {

		return [ 'envira-gallery-script', 'envira-gallery-style', 'envira-gallery-jgallery', 'envira-elementor' ];
	}

	/**
	 * Retrieve the widget icon.
	 *
	 * @since 1.1.0
	 *
	 * @access public
	 *
	 * @return string Widget icon.
	 */
	public function get_icon() {

		return 'fa fa-envira';
	}

	/**
	 * Retrieve the list of categories the widget belongs to.
	 *
	 * Used to determine where to display the widget in the editor.
	 *
	 * Note that currently Elementor supports only one category.
	 * When multiple categories passed, Elementor uses the first one.
	 *
	 * @since 1.1.0
	 *
	 * @access public
	 *
	 * @return array Widget categories.
	 */
	public function get_categories() {

		return [ 'envira-gallery' ];
	}

	/**
	 * Register the widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.1.0
	 *
	 * @access protected
	 */
	protected function _register_controls() { // phpcs:ignore PSR2.Methods.MethodDeclaration.Underscore

		if ( function_exists( 'envira_get_galleries' ) ) {
			$galleries        = envira_get_galleries( false, true );
			$gallery_dropdown = [];
			foreach ( $galleries as $gallery ) {
				if ( ! empty( $gallery['config']['title'] ) && ! empty( $gallery['id'] ) ) {
					$gallery_dropdown[ intval( $gallery['id'] ) ] = wp_strip_all_tags( $gallery['config']['title'] );
				}
			}
			// Clean, sort, then reverse the gallery list.
			ksort( $gallery_dropdown, SORT_NUMERIC );
			$gallery_dropdown = array_reverse( $gallery_dropdown, true );

			// Layout.
			$layout_dropdown = [
				'automatic' => 'Automatic',
				'mason'     => 'Mason',
				'grid'      => 'Grid',
				'square'    => 'Square',
				'blogroll'  => 'Blogroll',
			];

			$columns_dropdown = [
				'0'  => 'Default',
				'2'  => 'Two Columns (2)',
				'3'  => 'Three Columns (3)',
				'4'  => 'Four Columns (4)',
				'5'  => 'Five Columns (5)',
				'6'  => 'Six Columns (6)',
				'7'  => 'Seven Columns (7)',
				'8'  => 'Eight Columns (8)',
				'9'  => 'Nine Columns (9)',
				'10' => 'Ten Columns (10)',
				'11' => 'Eleven Columns (11)',
				'12' => 'Tweleve Columns (12)',
			];

			$lightbox_themes_available = envira_get_lightbox_themes();

			$lightbox_themes = [];
			foreach ( $lightbox_themes_available as $lightbox_theme_available ) {
				$lightbox_themes[ $lightbox_theme_available['value'] ] = $lightbox_theme_available['name'];
			}

			$gallery_margins = [
				'0'  => 'None',
				'1'  => 'Small',
				'5'  => 'Medium',
				'20' => 'Large',
				'-'  => 'Custom',
			];

		}

		$this->start_controls_section(
			'section_content',
			[
				'label' => __( 'Gallery', 'envira-elementor' ),
			]
		);

		$this->add_control(
			'envira_gallery_id',
			[
				'label'   => __( 'Gallery', 'envira-elementor' ),
				'type'    => \Elementor\Controls_Manager::SELECT,
				'options' => $gallery_dropdown,
				'default' => false,
			]
		);

		$this->add_control(
			'edit_gallery_link',
			[
				'label' => false,
				'type'  => \Elementor\Controls_Manager::RAW_HTML,
				'raw'   => __( '<a target="_blank" href="', 'envira-elementor' ) . admin_url( 'post.php?post=' . $gallery['id'] ) . __( '&action=edit" class="button">Edit Gallery</a>', 'envira-elementor' ),
			]
		);

		$this->add_control(
			'envira_gallery_layout',
			[
				'label'   => __( 'Select Layout', 'envira-elementor' ),
				'type'    => \Elementor\Controls_Manager::SELECT,
				'options' => $layout_dropdown,
				'default' => false,
			]
		);

		$this->add_control(
			'envira_gallery_columns',
			[
				'label'   => __( 'Select Columns', 'envira-elementor' ),
				'type'    => \Elementor\Controls_Manager::SELECT,
				'options' => $columns_dropdown,
				'default' => false,
			]
		);

		$this->add_control(
			'envira_gallery_margins',
			[
				'label'   => __( 'Gallery Margins', 'envira-elementor' ),
				'type'    => \Elementor\Controls_Manager::SELECT,
				'options' => $gallery_margins,
				'default' => false,
			]
		);

		$this->add_control(
			'envira_gallery_margin',
			[
				'label'   => __( 'Margin', 'plugin-domain' ),
				'type'    => \Elementor\Controls_Manager::NUMBER,
				'min'     => 0,
				'max'     => 50,
				'step'    => 1,
				'default' => 10,
			]
		);

		$this->add_control(
			'hr',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		);

		$this->add_control(
			'more_options',
			[
				'label'     => __( 'Lightbox', 'envira-elementor' ),
				'type'      => \Elementor\Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'envira_gallery_enable_lightbox',
			[
				'label'        => __( 'Enable Lightbox', 'plugin-domain' ),
				'type'         => \Elementor\Controls_Manager::SWITCHER,
				'label_on'     => __( 'On', 'your-plugin' ),
				'label_off'    => __( 'Off', 'your-plugin' ),
				'return_value' => '1',
				'default'      => 'yes',
			]
		);

		$this->add_control(
			'envira_gallery_lightbox_theme',
			[
				'label'   => __( 'Lightbox Theme', 'envira-elementor' ),
				'type'    => \Elementor\Controls_Manager::SELECT,
				'options' => $lightbox_themes,
				'default' => false,
			]
		);

		$this->end_controls_section();
	}

	/**
	 * Render the widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.1.0
	 *
	 * @access protected
	 */
	protected function render() {

		$setting = $this->get_settings_for_display();

		if ( defined( 'ENVIRA_DEBUG' ) && ENVIRA_DEBUG ) {
			error_log( 'setting render()' );
		}

		if ( empty( $setting ) && '' === $setting['envira_gallery_id'] || false === $setting['envira_gallery_id'] || 'default' === $setting['envira_gallery_id'] ) {
			return;
		}

		$gallery_id = intval( $setting['envira_gallery_id'] );

		$shortcode = '[envira-gallery id="' . $gallery_id . '" type="elementor" cache="false" layout="' . $setting['envira_gallery_layout'] . '" columns="' . $setting['envira_gallery_columns'] . '" lightbox_theme="' . $setting['envira_gallery_lightbox_theme'] . '" ]';
		if ( defined( 'ENVIRA_DEBUG' ) && ENVIRA_DEBUG ) {
			error_log( do_shortcode( $shortcode ) );
		}
		echo do_shortcode( $shortcode );

		$gallery_data = envira_get_gallery( $gallery_id );
		if ( defined( 'ENVIRA_DEBUG' ) && ENVIRA_DEBUG ) {
			error_log( print_r( $gallery_data, true ) );
		}
		$envira_gallery_layout = ! empty( $setting['envira_gallery_layout'] ) ? $setting['envira_gallery_layout'] : $gallery_data['config']['layout'];
		if ( 'automatic' === strtolower( $envira_gallery_layout ) ) {
			$envira_gallery_columns = '0';
		} else {
			$envira_gallery_columns = ! empty( $setting['envira_gallery_columns'] ) ? $setting['envira_gallery_columns'] : $gallery_data['config']['columns'];
		}
		$envira_gallery_margin = ! empty( $setting['envira_gallery_margin'] ) ? $setting['envira_gallery_margin'] : $gallery_data['config']['margin'];
		switch ( $envira_gallery_margin ) {
			case '0':
			case '1':
			case '5':
			case '10':
				$envira_gallery_margins = intval( $envira_gallery_margin );
				break;
			default:
				$envira_gallery_margins = '-';
				break;
		}
		$envira_gallery_margins         = ! empty( $setting['envira_gallery_margins'] ) ? $setting['envira_gallery_margins'] : $envira_gallery_margins;
		$envira_gallery_enable_lightbox = intval( $gallery_data['config']['lightbox_enabled'] );
		$envira_gallery_lightbox_theme  = ! empty( $setting['envira_gallery_lightbox_theme'] ) ? $setting['envira_gallery_lightbox_theme'] : $gallery_data['config']['lightbox_theme'];
		$envira_gallery_edit_url        = rawurlencode( admin_url( 'post.php?post=' . $gallery_id . '&action=edit' ) );
		?>
		<script type="text/javascript">
			jQuery('body').find('#envira-gallery-wrap-<?php echo intval( $setting['envira_gallery_id'] ); ?>').trigger('envira_load');
			var envira_gallery_edit_link = '<?php echo esc_url( $envira_gallery_edit_url ); ?>';
			var envira_gallery_layout = '<?php echo esc_html( $envira_gallery_layout ); ?>';
			var envira_gallery_columns = '<?php echo esc_html( $envira_gallery_columns ); ?>';
			var envira_gallery_margins = '<?php echo esc_html( $envira_gallery_margins ); ?>';
			var envira_gallery_margin = '<?php echo esc_html( $envira_gallery_margin ); ?>';
			var envira_gallery_enable_lightbox = '<?php echo esc_html( $envira_gallery_enable_lightbox ); ?>';
			var envira_gallery_lightbox_theme = '<?php echo esc_html( $envira_gallery_lightbox_theme ); ?>';
		</script>
		<?php if ( Plugin::$instance->editor->is_edit_mode() ) : ?>

			<?php

		endif;
	}

	/**
	 * Placeholder function that just returns the value.
	 *
	 * @param string $content The current post ID.
	 * @param string $button The current post object.
	 */
	public function skin_print_template( $content, $button ) {
		return $content;
	}

	/**
	 * Render the widget output in the editor.
	 *
	 * Written as a Backbone JavaScript template and used to generate the live preview.
	 *
	 * @since 1.1.0
	 *
	 * @access protected
	 */
	protected function _content_template() { // phpcs:ignore PSR2.Methods.MethodDeclaration.Underscore
		?>

	<div data-envira-gallery-id="{{{ settings.envira_gallery_id }}}" class="envira-elementor-container gallery-{{{ settings.envira_gallery_id }}}">
		<div class="envira-elementor-start">
			<div class="envira-elementor-loading-area">
				<img src="<?php echo esc_url( ENVIRA_ELEMENTOR_URL ) . 'assets/icons/envira.svg'; ?>" />
				<p id="envira-elementor-loading-message"></p>
			</div>
		</div>
	</div>

	<script type="text/javascript">

	var envira_gallery_id = '{{{ settings.envira_gallery_id }}}';

		<?php

		if ( defined( 'ENVIRA_DEBUG' ) && ENVIRA_DEBUG ) {

			?>

	<?php } ?>

	if ( envira_gallery_id === undefined || envira_gallery_id === 0 || envira_gallery_id === false || envira_gallery_id === 'false' ) {
		document.getElementById("envira-elementor-loading-message").innerHTML = "Please select a gallery.";
	} else {
		document.getElementById("envira-elementor-loading-message").innerHTML = "Standby...";
	}

	var envira_gallery_layout = '{{{ settings.envira_gallery_layout }}}',
		layout_to_post = envira_gallery_layout.length > 0 && envira_gallery_layout !== 'false' ? envira_gallery_layout : '';
	var envira_gallery_columns = '{{{ settings.envira_gallery_columns }}}',
		columns_to_post = parseInt( envira_gallery_columns ) > 0 && envira_gallery_columns !== 'false' ? envira_gallery_columns : '';
	var envira_gallery_margins = '{{{ settings.envira_gallery_margins }}}',
		margins_to_post = parseInt( envira_gallery_margins ) > 0 && envira_gallery_margins !== 'false' ? envira_gallery_margins : 'Custom';
	var envira_gallery_margin = '{{{ settings.envira_gallery_margin }}}',
		margin_to_post = parseInt( envira_gallery_margin ) >= 0 && envira_gallery_margin !== 'false' ? envira_gallery_margin : '10';
	var envira_gallery_enable_lightbox = '{{{ settings.envira_gallery_enable_lightbox }}}',
		enable_lightbox_to_post = envira_gallery_enable_lightbox !== 'false' ? envira_gallery_enable_lightbox : 0;
	var envira_gallery_lightbox_theme = '{{{ settings.envira_gallery_lightbox_theme }}}',
		lightbox_theme_to_post = envira_gallery_lightbox_theme.length > 0 && envira_gallery_lightbox_theme !== 'false' ? envira_gallery_lightbox_theme : 'base_dark';
	var panel =  ( undefined !== window.envira_elementor_panel.$el ) ? window.envira_elementor_panel.$el : false;

		<?php

		if ( defined( 'ENVIRA_DEBUG' ) && ENVIRA_DEBUG ) {

			?>

	<?php } ?>

	if ( envira_gallery_id !== false && envira_gallery_id !== 'false' ) {

		if ( parseInt( window.envira_elementor_gallery_id ) === parseInt( envira_gallery_id ) ) {
			// the gallery has not changed, so it's ok to override
			window.overwrite_to_post = true;
			layout_to_post = panel.find( '.elementor-control-envira_gallery_layout select' ).val();
			columns_to_post = panel.find( '.elementor-control-envira_gallery_columns select' ).val();
			margins_to_post = panel.find( '.elementor-control-envira_gallery_margins select' ).val();
			margin_to_post = panel.find( '.elementor-control-envira_gallery_margin input' ).val();
			enable_lightbox_to_post = panel.find( '.elementor-control-envira_gallery_enable_lightbox input.elementor-switch-input' ).checked;
			lightbox_theme_to_post = panel.find( '.elementor-control-envira_gallery_lightbox_theme select' ).val();
			// alert ('switching to true');
		} else {
			// if the gallery is different, i think/hope it's because they are changing the gallery dropdown to select a new/other gallery
			window.overwrite_to_post = false;
		}

		jQuery.ajax(envira_elementor.ajax_url, {
			type: "POST",
			data: {
				nonce: envira_gallery.nonce,
				action: 'envira_gallery_elementor_generate_gallery_html',
				gallery_id: '{{{ settings.envira_gallery_id }}}',
				layout: layout_to_post,
				columns: columns_to_post,
				margins: margins_to_post,
				custom_margin: margin_to_post,
				lightbox: enable_lightbox_to_post,
				lightbox_theme: lightbox_theme_to_post,
				overwrite: window.overwrite_to_post,
			},
			success: function(response) {
				// Tell the view we've finished successfully

				if ( response === 0 ) {
					return;
				}

				var findiv = jQuery('body').find('.envira-elementor-container.gallery-{{{ settings.envira_gallery_id }}}').find('div.envira-elementor-start');
				findiv.html(response);

				var $element = findiv.find( '.envira-gallery-wrap .envira-gallery-public' );
				var panel =  ( undefined !== window.envira_elementor_panel.$el ) ? window.envira_elementor_panel.$el : false;

				var gallery_id = $element.data('envira-id');
				<?php

				if ( defined( 'ENVIRA_DEBUG' ) && ENVIRA_DEBUG ) {

					?>

				<?php } ?>

				if ( $element !== undefined && panel !== undefined && panel !== false ) {

					if ( gallery_id ) {
						window.envira_elementor_gallery_id = gallery_id;
					}


					var gallery_images_json = $element.data('gallery-images'),
						gallery_data_json = $element.data('gallery-config');

					if ( gallery_data_json === undefined || gallery_images_json === undefined ) {
						return;
					}

					var layout    = gallery_data_json.layout.length > 0 ? gallery_data_json.layout : '',
						columns   = parseInt( gallery_data_json.columns ) > 0 && gallery_data_json.columns !== 'false' ? gallery_data_json.columns : '',
						margin    = gallery_data_json.margin ? parseInt( gallery_data_json.margin ) : '',
						lb_enable = gallery_data_json.lightbox_enabled ? parseInt( gallery_data_json.lightbox_enabled ) : '',
						lb_theme  = gallery_data_json.lightbox_theme !== 'false' && gallery_data_json.lightbox_theme.length > 0 ? gallery_data_json.lightbox_theme : '';

					var margins = '';

				<?php

				if ( defined( 'ENVIRA_DEBUG' ) && ENVIRA_DEBUG ) {

					?>

					<?php } ?>

					if ( layout === 'automattic' ) {
						columns = '';
					}

					switch( margin ) {
						case 0:
						case 1:
						case 5:
						case 20:
						margins = '' + margin;
						break;
						default:
						margins = '-';
					}

					<?php

					if ( defined( 'ENVIRA_DEBUG' ) && ENVIRA_DEBUG ) {

						?>


						<?php
					}

					$new_gallery_url = admin_url( 'post.php?post=' );

					?>

					var new_gallery_url = '<?php echo esc_url( $new_gallery_url ); ?>' + '{{{ settings.envira_gallery_id }}}' + '&action=edit';

					if ( layout !== '' ) {
						panel.find( '.elementor-control-envira_gallery_layout select' ).val( layout );
					}

					panel.find( '.elementor-control-envira_gallery_columns select' ).val( columns );

					panel.find( '.elementor-control-envira_gallery_margins select' ).val( margins );
					panel.find( '.elementor-control-envira_gallery_margin input' ).val( margin );

					panel.find( '.elementor-control-raw-html a').attr("href", new_gallery_url );

					<?php

					if ( defined( 'ENVIRA_DEBUG' ) && ENVIRA_DEBUG ) {

						?>

					<?php } ?>
					if ( lb_enable === 1 ) {
						panel.find( '.elementor-control-envira_gallery_enable_lightbox input.elementor-switch-input' ).prop( "checked", true );
					} else {
						panel.find( '.elementor-control-envira_gallery_enable_lightbox input.elementor-switch-input' ).prop( "checked", false );
					}
					panel.find( '.elementor-control-envira_gallery_lightbox_theme select' ).val( lb_theme );

				}

				findiv.trigger('envira_load');
			},
			error: function(error_message) {
				// Tell wp.media we've finished, but there was an error
				alert( "error" );
				alert( "{{{ settings.envira_gallery_id }}}" );
			},
		});

	}

	</script>

		<?php
	}

	/**
	 * Helper Method to define constants
	 *
	 * @since 1.8.8
	 *
	 * @param string $name The name.
	 * @param mixed  $value The value.
	 * @return void
	 */
	public function define( $name, $value ) {
		if ( ! defined( $name ) ) {
			define( $name, $value );
		}
	}
}
