<?php
/**
 * Plugin Name: Envira Gallery - Instagram Addon
 * Plugin URI:  http://enviragallery.com
 * Description: Populate Envira Galleries with Instagram images.
 * Author:      Envira Gallery Team
 * Author URI:  http://enviragallery.com
 * Version:     1.6.2
 * Text Domain: envira-instagram
 * Domain Path: languages
 *
 * @package Envira Gallery
 * @subpackage Envira Instagram Addon
 *
 * Envira Gallery is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * any later version.
 *
 * Envira Gallery is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Envira Gallery. If not, see <http://www.gnu.org/licenses/>.
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( function_exists( 'envira_license_checker' ) && false === envira_license_checker() ) {
	return false;
}

use Envira\Instagram\Frontend\Frontend_Container;
use Envira\Instagram\Admin\Admin_Container;
use Envira\Instagram\Utils\Data_Encryption;

use Envira\Utils\Updater;

register_uninstall_hook( __FILE__, 'Envira_Instagram::uninstall' );
register_activation_hook( __FILE__, 'Envira_Instagram::activation' );
register_deactivation_hook( __FILE__, 'Envira_Instagram::deactivation' );

if ( ! class_exists( 'Envira_Instagram' ) ) :

	/**
	 * Main plugin class.
	 *
	 * @since 1.0.0
	 *
	 * @package Envira_Instagram
	 * @author  Envira Team
	 */
	class Envira_Instagram {

		/**
		 * Holds the class object.
		 *
		 * @since 1.0.0
		 *
		 * @var object
		 */
		public static $instance;

		/**
		 * Plugin version, used for cache-busting of style and script file references.
		 *
		 * @since 1.0.0
		 *
		 * @var string
		 */
		public $version = '1.6.2';

		/**
		 * The name of the plugin.
		 *
		 * @since 1.0.0
		 *
		 * @var string
		 */
		public $plugin_name = 'Envira Instagram';

		/**
		 * Unique plugin slug identifier.
		 *
		 * @since 1.0.0
		 *
		 * @var string
		 */
		public $plugin_slug = 'envira-instagram';

		/**
		 * Plugin file.
		 *
		 * @since 1.0.0
		 *
		 * @var string
		 */
		public $file = __FILE__;

		/**
		 * Primary class constructor.
		 *
		 * @since 1.0.0
		 */
		public function __construct() {

			// Fire a hook before the class is setup.
			do_action( 'envira_instagram_pre_init' );

			// Load the plugin textdomain.
			add_action( 'plugins_loaded', [ $this, 'load_plugin_textdomain' ] );

			// Load the plugin.
			add_action( 'envira_gallery_init', [ $this, 'init' ], 99 );

			// Load the updater.
			add_action( 'envira_gallery_updater', [ $this, 'updater' ], 10, 1 );
		}

		/**
		 * Setup Constants function.
		 *
		 * @since 1.6.0
		 *
		 * @access public
		 * @return void
		 */
		public function setup_constants() {

			if ( ! defined( 'ENVIRA_INSTAGRAM_VERSION' ) ) {

				define( 'ENVIRA_INSTAGRAM_VERSION', $this->version );

			}

			if ( ! defined( 'ENVIRA_INSTAGRAM_SLUG' ) ) {

				define( 'ENVIRA_INSTAGRAM_SLUG', $this->plugin_slug );

			}

			if ( ! defined( 'ENVIRA_INSTAGRAM_FILE' ) ) {

				define( 'ENVIRA_INSTAGRAM_FILE', $this->file );

			}

			if ( ! defined( 'ENVIRA_INSTAGRAM_DIR' ) ) {

				define( 'ENVIRA_INSTAGRAM_DIR', plugin_dir_path( __FILE__ ) );

			}

			if ( ! defined( 'ENVIRA_INSTAGRAM_URL' ) ) {

				define( 'ENVIRA_INSTAGRAM_URL', plugin_dir_url( __FILE__ ) );

			}
		}

		/**
		 * Loads the plugin textdomain for translation.
		 *
		 * @since 1.0.0
		 */
		public function load_plugin_textdomain() {

			load_plugin_textdomain( $this->plugin_slug, false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
		}

		/**
		 * Loads the plugin into WordPress.
		 *
		 * @since 1.0.0
		 */
		public function init() {

			// Run hook once Envira has been initialized.
			do_action( 'envira_instagram_init' );

			// Load admin only components.
			if ( is_admin() ) {
				$admin = new \Envira\Instagram\Admin\Admin_Container();
			}

			// Set custom cron schedules.
			add_filter( 'cron_schedules', [ $this, 'cron_schedules' ] );

			$frontend = new \Envira\Instagram\Frontend\Frontend_Container();

			add_action( 'envira_instagram_token_cron_hook', [ $frontend, 'check_long_term_token' ] );

			add_action( 'remove_instagram_data_by_slot', [ $frontend, 'remove_instagram_data_by_slot_task' ], 10, 1 );

			$this->maybe_run_upgrade();
		}


		/**
		 * Initializes the addon updater.
		 *
		 * @since 1.0.0
		 *
		 * @param string $key The user license key.
		 */
		public function updater( $key ) {

			$args = [
				'plugin_name' => $this->plugin_name,
				'plugin_slug' => $this->plugin_slug,
				'plugin_path' => plugin_basename( __FILE__ ),
				'plugin_url'  => trailingslashit( WP_PLUGIN_URL ) . $this->plugin_slug,
				'remote_url'  => 'https://enviragallery.com/',
				'version'     => $this->version,
				'key'         => $key,
			];

			$updater = new Updater( $args );
		}

		/**
		 * Loads all global files into scope.
		 *
		 * @since 1.0.0
		 */
		public function require_global() {

			require_once trailingslashit( ENVIRA_INSTAGRAM_DIR ) . 'src/Functions/common.php';
		}

		/**
		 * Runs upgrade routine
		 *
		 * @since 1.6.0
		 */
		public function maybe_run_upgrade() {

			$version = get_option( 'envira_instagram_version' );

			if ( false === $version || version_compare( $version, ENVIRA_INSTAGRAM_VERSION, '<' ) ) {

				$encryption = new \Envira\Instagram\Utils\Data_Encryption();

				$total_accounts = 3;

				for ( $x = 1; $x <= $total_accounts; $x++ ) {
					$account = envira_instagram_get_instagram_auth( $x );
					if ( $account ) {
						$account['token'] = $encryption->maybe_encrypt( $account['token'] );
						if ( 1 === $x ) {
							update_option( 'envira_instagram', $account );
						} else {
							update_option( 'envira_instagram_' . $x, $account );
						}
					}
				}

				update_option( 'envira_instagram_version', ENVIRA_INSTAGRAM_VERSION );
			}
		}

		/**
		 * Autoload function.
		 *
		 * @access public
		 * @static
		 *
		 * @param string $php_class Class to autoload.
		 *
		 * @return void
		 */
		public static function autoload( $php_class ) {

			// Prepare variables.
			$prefix   = 'Envira\\Instagram\\';
			$base_dir = __DIR__ . '/src/';
			$length   = mb_strlen( $prefix );

			// If the class is not using the namespace prefix, return.
			if ( 0 !== strncmp( $prefix, $php_class, $length ) ) {
				return;
			}

			// Prepare classes to be autoloaded.
			$relative_class = mb_substr( $php_class, $length );
			$file           = $base_dir . str_replace( '\\', '/', $relative_class ) . '.php';

			// If the file exists, load it.
			if ( file_exists( $file ) ) {
				require_once $file;
			}
		}

		/**
		 * Fired when the plugin is uninstalled.
		 *
		 * @since  1.6.0
		 *
		 * @global object $wpdb The WordPress database object.
		 */
		public static function uninstall() {

			if ( ! class_exists( 'Envira_Instagram' ) ) {
				die();
			}

			self::get_instance();

			envira_instagram_clear_plaform_data( 'all' );
		}

		/**
		 * Activation hook.
		 *
		 * @return void
		 * @since  1.9.9.2
		 */
		public static function activation() {
		}

		/**
		 * Deactivation hook.
		 *
		 * @return void
		 * @since  1.9.9.2
		 */
		public static function deactivation() {

			wp_clear_scheduled_hook( 'envira_instagram_token_cron_hook' );
			wp_clear_scheduled_hook( 'remove_instagram_data_by_slot' );
		}

		/**
		 * Add cron job schedules.
		 *
		 * @since 1.9.9.2
		 * @param array $schedules Schedules.
		 *
		 * @return string Revised Where.
		 */
		public function cron_schedules( $schedules ) {

			$schedules['every_other_minute'] = [
				'interval' => 1230,
				'display'  => __( 'Every other minute' ),
			];

			$schedules['every_45_days'] = [
				'interval' => 3888000,
				'display'  => __( 'Every 45 days' ),
			];
			return $schedules;
		}

		/**
		 * Returns the singleton instance of the class.
		 *
		 * @since 1.4.1
		 *
		 * @return object The Envira_Instagram object.
		 */
		public static function get_instance() {

			if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Envira_Instagram ) ) {
				self::$instance = new self();
				self::$instance->setup_constants();
				self::$instance->require_global();
			}

			return self::$instance;
		}
	}

	spl_autoload_register( 'Envira_Instagram::autoload' );

	// Load the plugin.
	add_action(
		'envira_gallery_init',
		function () {
			Envira_Instagram::get_instance();
		}
	);

endif;
