<?php
/**
 * Settings admin class.
 *
 * @since 1.5.0
 *
 * @package Envira_Instagram
 * @author  Envira Gallery Team <support@enviragallery.com>
 */

namespace Envira\Instagram\Admin;

use Envira\Admin\Notices;
use Envira\Instagram\Utils\Data_Encryption;

/**
 * Settings class.
 *
 * @since 1.1.0
 *
 * @package Envira_Gallery
 * @author  Envira Gallery Team <support@enviragallery.com>
 */
class Settings {




	/**
	 * Allowed HTML
	 *
	 * @var mixed
	 * @access public
	 */
	public $wp_kses_allowed_html = [
		'a'      => [
			'href'                => [],
			'target'              => [],
			'class'               => [],
			'title'               => [],
			'data-status'         => [],
			'data-envira-tooltip' => [],
			'data-id'             => [],
		],
		'br'     => [],
		'img'    => [
			'src'   => [],
			'class' => [],
			'alt'   => [],
		],
		'div'    => [
			'class' => [],
		],
		'li'     => [
			'id'                              => [],
			'class'                           => [],
			'data-envira-gallery-image'       => [],
			'data-envira-gallery-image-model' => [],
		],
		'em'     => [],
		'span'   => [
			'class' => [],
		],
		'strong' => [],
	];

	/**
	 * Encryption Object
	 *
	 * @since 1.6.0
	 * @var Envira\Instagram\Utils\Data_Encryption;
	 */
	public $encryption = null;

	/**
	 * Primary class constructor.
	 *
	 * @since 1.3.0
	 */
	public function __construct() {
		$this->init();
	}

	/**
	 * Init Method
	 *
	 * @return void
	 */
	public function init() {

		$this->encryption = new \Envira\Instagram\Utils\Data_Encryption();

		// Actions.
		add_filter( 'envira_gallery_settings_tab_nav', [ $this, 'tabs' ] );
		add_action( 'envira_gallery_tab_settings_instagram', [ $this, 'settings' ] );
		add_action( 'admin_init', [ $this, 'process' ] );
	}

	/**
	 * Add a tab to the Envira Gallery Settings screen
	 *
	 * @since 1.0.0
	 *
	 * @param array $tabs Existing tabs.
	 * @return array New tabs
	 */
	public function tabs( $tabs ) {

		$tabs['instagram'] = __( 'Instagram', 'envira-instagram' );

		return $tabs;
	}

	/**
	 * Outputs settings screen for the Instagram Tab.
	 *
	 * @since 1.0.0
	 */
	public function settings() {

		// Get settings and URLs to connect/disconnect Instagram.
		$total_accounts = 3;
		$slots          = [];
		for ( $x = 1; $x <= $total_accounts; $x++ ) {
			$temp_auth = envira_instagram_get_instagram_auth( $x );
			if ( $temp_auth ) {
				$slots[] = $temp_auth;
			}
		}

		// Note: the missing 'page=envira-gallery-settings#!envira-tab-instagram' parameter is deliberate.
		// Instagram strips this URL argument in the oAuth process, and would then throw a 400 redirect_uri mismatch error.
		// Envira's API will append the 'page=envira-gallery-settings#!envira-tab-instagram' parameter on the redirect back
		// to this site, ensuring everything works correctly.
		$connect_url = envira_instagram_get_oauth_url( 'edit.php?post_type=envira' );

		wp_nonce_field( 'envira_instagram_save_settings', 'envira_instagram_nonce' );

		?>
		<div id="envira-settings-instagram">

			<h2><?php esc_html_e( 'Instagram Authorization Setup', 'envira-instagram' ); ?></h2>

			<?php

			$envira_whitelabel_name_singular = apply_filters( 'envira_whitelabel', false ) ? apply_filters( 'envira_whitelabel_name_singular', false ) : 'Envira';

			/* translators: %s */
			printf( esc_html__( 'Before you can create Instagram galleries, you need to authenticate with your %1$s Instagram accounts. Envira supports authorization with up to 3 unique Instagram accounts. You currently are using %2$s slots.', 'envira-instagram' ), esc_html( $envira_whitelabel_name_singular ), count( $slots ) );

			?>

			<p>Need assistance? <a target="_blank" href="https://enviragallery.com/docs/instagram-addon/" rel="noopener">Click here</a> to read further documentation.</p>

			<?php if ( count( $slots ) < $total_accounts ) { ?>
				<p class="description" style="margin-top: 20px;">
					<a href="<?php echo esc_url( $connect_url ); ?>" class="button-disable button-secondary-disable envira-instagram-authorize">

						<img class="hide hidden" width="250" src="<?php echo esc_url_raw( plugins_url( 'assets/images/fb_button.png', ENVIRA_INSTAGRAM_FILE ) ); ?>" />

						<?php if ( 0 === count( $slots ) ) { ?>

							<?php

							esc_html_e( 'Click here to Authenticate Your First Instagram Account With ', 'envira-instagram' );
							echo ( apply_filters( 'envira_whitelabel', false ) ? esc_html( apply_filters( 'envira_whitelabel_name_singular', false ) ) : 'Envira' );

							?>

						<?php } else { ?>

							<?php

							esc_html_e( 'Click here to Authenticate Another Instagram Account With ', 'envira-instagram' );
							echo ( apply_filters( 'envira_whitelabel', false ) ? esc_html( apply_filters( 'envira_whitelabel_name_singular', false ) ) : 'Envira' );

							?>

						<?php } ?>
					</a>
				</p>

			<?php } ?>

			<?php
			// }
			?>

			<table class="form-table" style="margin-top: 40px;">
				<tbody>

					<?php

					$account_number = 1;

					foreach ( $slots as $slot ) {
						// phpcs:disable WordPress.Arrays.MultipleStatementAlignment.DoubleArrowNotAligned, WordPress.Arrays.MultipleStatementAlignment.LongIndexSpaceBeforeDoubleArrow
						$disconnect_url = add_query_arg(
							[
								'post_type'                 => 'envira',
								'instagram_nonce'           => wp_create_nonce( 'eg_instagram_auth' ),
								'envira-instagram-remove'   => 'true',
								'envira-instagram-username' => ( ! empty( $slot['username'] ) ) ? $slot['username'] : $account_number,
								'page'                      => 'envira-gallery-settings#envira-tab-instagram',
							],
							admin_url( 'edit.php' )
						);
						// phpcs:enable WordPress.Arrays.MultipleStatementAlignment.DoubleArrowNotAligned

						// URL for clearning all Instagram platform data.
						// phpcs:disable WordPress.Arrays.MultipleStatementAlignment.DoubleArrowNotAligned
						$clear_url = add_query_arg(
							[
								'post_type'                   => 'envira',
								'instagram_nonce'             => wp_create_nonce( 'eg_instagram_auth' ),
								'envira-instagram-clear-data' => 'true',
								'envira-instagram-username'   => ( ! empty( $slot['username'] ) ) ? $slot['username'] : $account_number,
								'page'                        => 'envira-gallery-settings#envira-tab-instagram',
							],
							admin_url( 'edit.php' )
						);
						// phpcs:enable WordPress.Arrays.MultipleStatementAlignment.DoubleArrowNotAligned

						?>

						<?php if ( ! empty( $slot['token'] ) ) : ?>

							<tr id="envira-settings-standalone-enable">
								<td>

									<label for="envira-instagram-authenticate-<?php echo esc_html( $x ); ?>">

										<?php

										echo '<h3>';
										esc_html_e( 'Account #', 'envira-instagram' );
										echo ( esc_html( $account_number ) );
										?>

										<?php
										if ( ! empty( $slot['username'] ) ) {
											?>
											( <?php echo esc_html( $slot['username'] ); ?> )
														<?php
										}

										?>

											<?php
											if ( ! empty( $slot['full_name'] ) ) {
												?>
												( <small>( <?php echo esc_html( $slot['full_name'] ); ?> )</small> )
																		<?php
											}

											echo '</h3>';
											?>

												<!-- <h2><?php esc_html_e( 'Success!', 'envira-gallery' ); ?></h2> -->
									</label>
									<p>
										<?php

										echo '<em>';
										esc_html_e( 'This Instagram account was authenticated for use with ', 'envira-instagram' );
										echo ( apply_filters( 'envira_whitelabel', false ) ? esc_html( apply_filters( 'envira_whitelabel_name_singular', false ) ) : 'Envira' );
										if ( ! empty( $slot['time_taken'] ) ) {
											esc_html_e( ' on ', 'envira-instagram' );
											echo esc_html( gmdate( 'F d, Y g:ia', $slot['time_taken'] ) );
										}

										echo '.</em>';

										?>

									<p><a href="<?php echo esc_url( $disconnect_url ); ?>" class="button button-primary envira-instagram-remove envira-oauth-remove">
											<?php

											esc_html_e( 'Click Here to Remove This Instagram Authentication', 'envira-instagram' );

											?>
										</a></p>

									<br/>
									<?php
									echo '<em>';
									esc_html_e( 'Warning: This action cannot be undone. This action will clear all instagram data, images, and galleries that use Instagram as data source. ', 'envira-instagram' );
									echo '</em>';
									?>
									<p><a href="<?php echo esc_url( $clear_url ); ?>" class="button button-danger important-warning envira-instagram-delete-data" onclick="return confirm('You are about to delete Instagram authentication, images, and galleries that use Instagram as data source. This cannot be undone. Are you sure to delete? Click \'cancel\' to stop and \'OK\' to delete. ');">
											<?php

											esc_html_e( 'Click Here to Delete All Instagram Data', 'envira-instagram' );

											?>
										</a></p>



								</td>
							</tr>

							<?php

							++$account_number;

						endif;
					}
					?>


					</form>
				</tbody>
			</table>




		</div>
		<?php
	}

	/**
	 * Saves or deletes auth settings, depending on the URL query arguments
	 *
	 * @since 1.0.0
	 */
	public function process() {

		if ( wp_doing_ajax() ) {
			return;
		}

		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		if ( ! isset( $_GET['instagram_nonce'] ) || ! wp_verify_nonce( sanitize_key( $_GET['instagram_nonce'] ), 'eg_instagram_auth' ) ) {
			return;
		}

		// Obtain the data coming back from instagram.
		$envira_instagram = false;

		if ( isset( $_GET['envira-instagram'] ) ) {
			$envira_instagram = json_decode( urldecode( wp_unslash( $_GET['envira-instagram'] ) ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized -- Sanitized below

		} elseif ( isset( $_GET['page'] ) && 'envira-gallery-settings' === $_GET['page'] ) {
			if ( ! isset( $_SERVER['HTTP_HOST'], $_SERVER['REQUEST_URI'] ) ) {
				return;
			}
			$incoming_url = 'https://' . sanitize_text_field( wp_unslash( $_SERVER['HTTP_HOST'] ) ) . sanitize_text_field( wp_unslash( $_SERVER['REQUEST_URI'] ) );
			$parts        = wp_parse_url( $incoming_url );
			parse_str( $parts['query'], $query );
			if ( ! empty( $query ) && isset( $query['envira-instagram'] ) ) {
				$envira_instagram = $query['envira-instagram'];
			}
		} else {
			return;
		}

		if ( $envira_instagram ) {

			// The user has completed the oAuth process, and has come back to this site.
			$response = $envira_instagram;

			if ( isset( $response->code ) ) {
				// Error.
				add_action( $this->notice_filter, [ $this, 'notice_oauth_error' ] );
				return;
			}

			if ( isset( $response->access_token ) ) {

				// Success
				// Update the option with the Instagram access token and user ID.
				$auth               = envira_instagram_get_instagram_auth();
				$auth['token']      = $response->access_token;
				$auth['id']         = ! empty( $_GET['user_id'] ) ? sanitize_text_field( wp_unslash( $_GET['user_id'] ) ) : false;
				$auth['time_taken'] = time();

				// attempt to get user id.
				$profile_call_array = [
					'fields'       => 'id,username,media_count',
					'access_token' => $auth['token'],
				];

				$response_long_term_token = wp_remote_get(
					'https://graph.instagram.com/me',
					[
						'body' => $profile_call_array,
					]
				);
				$response_code            = wp_remote_retrieve_response_code( $response_long_term_token );
				$response_body            = wp_remote_retrieve_body( $response_long_term_token );

				if ( 200 === $response_code ) {
					$received            = json_decode( $response_body );
					$auth['username']    = $received->username;
					$auth['media_count'] = $received->media_count;
					$auth['time_taken']  = time();
				}

				// Make sure the token is Encrypted before we store it.
				$auth['token'] = $this->encryption->maybe_encrypt( $auth['token'] );

				// find an empty slot.
				$open_slot = false;
				for ( $x = 1; $x <= 3; $x++ ) {
					$test_auth = envira_instagram_get_instagram_auth( $x );
					if ( $test_auth && ! empty( $test_auth['username'] ) && ! empty( $auth['username'] ) && $test_auth['username'] === $auth['username'] ) {
						break;
					}
					if ( is_array( $test_auth ) && empty( $test_auth ) ) {
						$open_slot = $x;
						break;
					}
				}

				if ( false === $open_slot ) {
					// Output a notice, which is called if the user authenticated via the Edit Gallery screen.
					add_action( 'envira_gallery_images_tab_notice', [ $this, 'notice_slots_full' ] );
				} else {

					if ( 1 === $open_slot ) {
						update_option( 'envira_instagram', $auth );
					} else {
						update_option( 'envira_instagram_' . $open_slot, $auth );
					}

					// Output a notice, which is called if the user authenticated via the Edit Gallery screen.
					add_action( 'envira_gallery_images_tab_notice', [ $this, 'notice_oauth_success' ] );

					// Schedule cron even only after token is auth'd.
					if ( ! wp_next_scheduled( 'envira_instagram_token_cron_hook' ) ) {
						wp_schedule_event( time(), 'daily', 'envira_instagram_token_cron_hook' );
					}
				}

				return;
			}
		}

		$envira_instagram_remove = false;

		if ( isset( $_GET['envira-instagram-remove'] ) ) {
			$envira_instagram_remove   = ! empty( $_GET['envira-instagram-remove'] ) && sanitize_text_field( wp_unslash( $_GET['envira-instagram-remove'] ) );
			$envira_instagram_username = ! empty( $_GET['envira-instagram-username'] ) ? sanitize_text_field( wp_unslash( $_GET['envira-instagram-username'] ) ) : false;
		} elseif ( isset( $_GET['envira-gallery-settings'] ) ) {
			if ( ! isset( $_SERVER['HTTP_HOST'], $_SERVER['REQUEST_URI'] ) ) {
				return;
			}
			$incoming_url = 'https://' . sanitize_text_field( wp_unslash( $_SERVER['HTTP_HOST'] ) ) . sanitize_text_field( wp_unslash( $_SERVER['REQUEST_URI'] ) );

			$parts = wp_parse_url( $incoming_url );
			parse_str( $parts['query'], $query );
			if ( ! empty( $query ) && isset( $query['envira-instagram-remove'] ) ) {
				$envira_instagram_remove = $query['envira-instagram-remove'];
			}
		}

		// Disconnect.
		if ( $envira_instagram_remove ) {

			$total_accounts = 3;

			// find an empty slot.
			$open_slot = false;
			for ( $x = 1; $x <= $total_accounts; $x++ ) {
				$test_auth = envira_instagram_get_instagram_auth( $x );
				if ( ( $test_auth && $test_auth['username'] === $envira_instagram_username ) || intval( $envira_instagram_username ) === $x ) {
					if ( 1 === $x ) {
						delete_option( 'envira_instagram' );
					} else {
						delete_option( 'envira_instagram_' . $x );
					}
				}
			}
			return;
		}

		$envira_instagram_clear_data = false;

		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		if ( isset( $_GET['envira-instagram-clear-data'] ) ) {
			// phpcs:ignore WordPress.Security.NonceVerification.Recommended
			$envira_instagram_clear_data = ! empty( $_GET['envira-instagram-clear-data'] ) && sanitize_text_field( wp_unslash( $_GET['envira-instagram-clear-data'] ) );
			// phpcs:ignore WordPress.Security.NonceVerification.Recommended
			$envira_instagram_username = ! empty( $_GET['envira-instagram-username'] ) ? sanitize_text_field( wp_unslash( $_GET['envira-instagram-username'] ) ) : false;
			// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		} elseif ( isset( $_GET['envira-gallery-settings'] ) ) {
			if ( ! isset( $_SERVER['HTTP_HOST'], $_SERVER['REQUEST_URI'] ) ) {
				return;
			}
			$incoming_url = 'https://' . sanitize_text_field( wp_unslash( $_SERVER['HTTP_HOST'] ) ) . sanitize_text_field( wp_unslash( $_SERVER['REQUEST_URI'] ) );

			$parts = wp_parse_url( $incoming_url );
			parse_str( $parts['query'], $query );
			if ( ! empty( $query ) && isset( $query['envira-instagram-clear-data'] ) ) {
				$envira_instagram_clear_data = $query['envira-instagram-clear-data'];
			}
		}

		// Clear platform data.
		if ( $envira_instagram_clear_data && $envira_instagram_username ) {

			envira_instagram_clear_plaform_data( $envira_instagram_username );

			return;
		}
	}

	/**
	 * Outputs a WordPress style notification message to tell the user that the settings have been saved
	 *
	 * @since 1.0.0
	 */
	public function notice_oauth_success() {
		$notices = new Notices();
		// Define the notice classes depending on which hook is used to output the notice.
		/* translators: %s: term name */
		$text = sprintf( __( 'Your Instagram account has been authenticated for use with %s!', 'envira-instagram' ), ( apply_filters( 'envira_whitelabel', false ) ? apply_filters( 'envira_album_whitelabel_name_singular', false ) : 'Envira' ) );
		$notices->display_inline_notice(
			'envira_instagram_oauth_success',
			__( 'Success!', 'envira-instagram' ),
			esc_html( $text )
		);
	}

	/**
	 * Outputs a WordPress style notification message to tell the user that all slots are full
	 *
	 * @since 1.0.0
	 */
	public function notice_slots_full() {
		$notices = new Notices();
		// Define the notice classes depending on which hook is used to output the notice.
		/* translators: %s: term name */
		$text = sprintf( __( 'Your Instagram account has been authenticated for use with %s!', 'envira-instagram' ), ( apply_filters( 'envira_whitelabel', false ) ? apply_filters( 'envira_album_whitelabel_name_singular', false ) : 'Envira' ) );
		$notices->display_inline_notice(
			'envira_instagram_oauth_failure',
			__( 'Failed All Slots Full!', 'envira-instagram' ),
			esc_html( $text )
		);
	}

	/**
	 * Outputs a WordPress style notification message to tell the user an error occured during oAuth
	 *
	 * @since 1.0.0
	 */
	public function notice_oauth_error() {
		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		if ( ! isset( $_GET['envira-instagram'] ) ) {
			return;
		}
		// Get error.
		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		$envira_instagram = json_decode( urldecode( wp_unslash( $_GET['envira-instagram'] ) ) ); // phpcs:ignore WordPress.Security.NonceVerification.Recommended,WordPress.Security.ValidatedSanitizedInput.InputNotSanitized -- Sanitized below

		// Define the notice classes depending on which hook is used to output the notice.
		$css = 'notice error below-h2';
		?>
		<div class="<?php echo esc_html( $css ); ?>">
			<p><?php echo esc_html( $envira_instagram->code ) . ': ' . esc_html( $envira_instagram->error_type ) . ' - ' . esc_html( $envira_instagram->error_message ); ?></p>
		</div>
		<?php
	}
}
