<?php
/**
 * Shortcode class.
 *
 * @since 1.0.0
 *
 * @package Envira_Instagram
 * @author  Envira Team
 */

namespace Envira\Instagram\Frontend;

use Envira\Instagram\Utils\Data_Encryption;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Shortcode class.
 *
 * @since 1.0.0
 *
 * @package Envira_Instagram
 * @author  Envira Team
 */
class Shortcode {
	/**
	 * Class instance
	 *
	 * @var string
	 */
	private static $instance;

	/**
	 * The time before it takes to refresh long term tickets (which expire at 90 days)
	 *
	 * @since 1.5.0
	 *
	 * @var string
	 */
	public $limit_per_page = 24;

	/**
	 * Prevent uploading duplicate images from the same feed.
	 *
	 * @since 1.5.0
	 *
	 * @var string
	 */
	public $ig_image_urls = [];

	/**
	 * Primary class constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {

		// Inject Images into Albums Admin. This allows the user to choose a cover image.
		add_filter( 'envira_albums_metaboxes_get_gallery_data', [ $this, 'inject_images' ], 10, 2 );

		// Inject Images into Frontend Gallery.
		add_filter( 'envira_gallery_pre_data', [ $this, 'inject_images' ], 10, 2 );
		add_filter( 'envira_images_pre_data', [ $this, 'inject_images' ], 10, 2 );

		// Inject Images into Album Lightbox.
		add_filter( 'envira_albums_shortcode_gallery', [ $this, 'inject_images' ], 10, 2 );

		// Decide If A Link to Gallery Images Should Be Created.
		add_filter( 'envira_gallery_create_link', [ $this, 'create_link' ], 10, 6 );

		// Remove Lightbox CSS class if user wants to directly link IG image.
		add_filter( 'envira_gallery_output_before_image', [ $this, 'gallery_output_before_image' ], 10, 5 );

		// Change link href to full sized IG image if user wants.
		add_filter( 'envira_gallery_link_href', [ $this, 'gallery_link_href' ], 10, 6 );

		// Clear Envira Cache, or risk double images when images are less than limit_per_page.
		// !!! TODO Determine why we need this.
		add_filter( 'envira_gallery_should_cache', [ $this, 'should_cache' ], 10, 2 );
	}

	/**
	 * Change link href to full sized IG image if user wants
	 *
	 * @since 1.2.1
	 *
	 * @param array   $link Create link.
	 * @param array   $data Gallery data.
	 * @param string  $id The ID.
	 * @param array   $item Item Data.
	 * @param int     $i Counter.
	 * @param boolean $is_mobile Is mobile.
	 */
	public function gallery_link_href( $link, $data, $id, $item, $i, $is_mobile ) {

		if ( isset( $data['config']['instagram_link'] ) && 'instagram_image' === $data['config']['instagram_link'] && ! empty( $item['instagram_high_res'] ) ) {
			return $item['instagram_high_res'];
		} elseif ( isset( $data['config']['instagram_link'] ) && 'instagram_page' === $data['config']['instagram_link'] && ! empty( $item['instagram_link'] ) ) {
			return $item['instagram_link'];
		} elseif ( ! empty( $link ) ) {
			return $link;
		} elseif ( ! empty( $item['src'] ) ) {
			return $item['src'];
		}
	}

	/**
	 * Check on adding a create link for galleries, hooking into shortcode.php
	 *
	 * @since 1.2.1
	 *
	 * @param array  $output Output.
	 * @param string $id The ID.
	 * @param array  $item Item Data.
	 * @param int    $data Gallery data.
	 * @param int    $i Counter.
	 */
	public function gallery_output_before_image( $output, $id, $item, $data, $i ) {

		if ( isset( $data['config']['instagram_link'] ) && 'instagram_image' === $data['config']['instagram_link'] ) {

			$output = str_replace( 'envira-gallery-9160 envira-gallery-link', '', $output );

		}

		return $output;
	}

	/**
	 * Check on adding a create link for galleries, hooking into shortcode.php
	 *
	 * @since 1.2.1
	 *
	 * @param array   $create_link Create link.
	 * @param array   $data Gallery data.
	 * @param string  $id The ID.
	 * @param array   $item Item Data.
	 * @param int     $i Counter.
	 * @param boolean $is_mobile Is mobile.
	 */
	public function create_link( $create_link, $data, $id, $item, $i, $is_mobile ) {

		if ( isset( $data['config']['type'] ) && 'instagram' === $data['config']['type'] && ( ! $data['config']['instagram_link'] && 0 === intval( $data['config']['lightbox_enabled'] ) ) ) {
			$create_link = false;
		}

		return $create_link;
	}

	/**
	 * Injects gallery images into the given $data array, using the $data settings
	 *
	 * @since 1.0.0
	 *
	 * @param array $data  Gallery Config.
	 * @param int   $gallery_id The gallery ID.
	 * @return array $data Amended array of gallery config, with images.
	 */
	public function inject_images( $data, $gallery_id ) {

		// Return early if not an Instagram gallery.
		if ( 'instagram' !== envira_get_config( 'type', $data ) ) {
			return $data;
		}

		// Return early if pagination is activated on this instagram gallery.
		if ( class_exists( 'Envira_Pagination' ) && 1 === intval( envira_get_config( 'pagination', $data ) ) ) {
			return $data;
		}

		// Grab the Instagram data from cache or live.
		$instagram_images = ( envira_get_config( 'instagram_cache', $data ) ? $this->get_instagram_data( $gallery_id, $data ) : $this->query_instagram_data( $gallery_id, $data ) );

		if ( ! $instagram_images ) {
			return $data;
		}

		// Insert data into gallery.
		$data['gallery'] = $instagram_images;

		return $data;
	}

	/**
	 * Attempts to get Instagram image data from transient/cache
	 *
	 * If transient does not exist, performs a live query and caches the results
	 *
	 * @since 1.0.0
	 *
	 * @param int   $gallery_id Gallery ID.
	 * @param array $data Gallery Data.
	 * @return array Instagram Images
	 */
	public function get_instagram_data( $gallery_id, $data = false ) {

		// Attempt to return the transient first, otherwise generate the new query to retrieve the data.
		$instagram_images = get_transient( '_envira_instagram_' . $gallery_id );
		if ( false === $instagram_images && ! empty( $data ) ) {
			$instagram_images    = $this->query_instagram_data( $gallery_id, $data );
			$instagram_transient = apply_filters( 'envira_instagram_allow_transient', true, $gallery_id );
			if ( $instagram_images && true === $instagram_transient ) {
				$expiration = envira_get_transient_expiration_time( 'envira-instagram' );
				set_transient( '_envira_instagram_' . $gallery_id, maybe_serialize( $instagram_images ), $expiration );
			}
		}

		// Return the slider data.
		return maybe_unserialize( $instagram_images );
	}

	/**
	 * Queries Instagram for image data
	 *
	 * @since 1.0.0
	 *
	 * @param int   $gallery_id Gallery ID.
	 * @param array $data Gallery Data.
	 * @return mixed false|Image Array
	 */
	public function query_instagram_data( $gallery_id, $data ) {

		if ( is_admin() ) {
			return false;
		}

		// With addition of multiple instagram accounts, make sure to get the slot.
		$slot        = envira_get_config( 'instagram_account', $data );
		$slot_number = ( false !== $slot ) ? $slot : 1;

		// Grab the Instagram auth data.
		$auth       = envira_instagram_get_instagram_auth( $slot_number );
		$encryption = new \Envira\Instagram\Utils\Data_Encryption();

		if ( empty( $auth['token'] ) || empty( $auth['id'] ) ) {
			return false;
		}

		// Setup variables that help define how many images and pages we need based on gallery settings.
		$next                = false;
		$instagram_data      = false;
		$current_image_total = 0;
		$max_images          = ! empty( envira_get_config( 'instagram_number', $data ) ) ? envira_get_config( 'instagram_number', $data ) : 10;

		// How many times to we need to paginate.
		$number_of_pages = ceil( $max_images / $this->limit_per_page );
		if ( $number_of_pages < 0 ) {
			$number_of_pages = 1;
		}

		// get initial page of results and go from there.

		for ( $current_page = 1; $current_page <= $number_of_pages; $current_page++ ) {
			// if this is the first page we will need to ping the API just to get the next.

			$media_call_array = [
				'fields'       => 'id,caption,media_type,media_url,permalink,thumbnail_url,timestamp,username,children{media_url,thumbnail_url}',
				'access_token' => $encryption->maybe_decrypt( $auth['token'] ),
			];

			$api_call_url = ( $current_page > 1 && ! empty( $next ) ) ? $next : 'https://graph.instagram.com/' . $auth['id'] . '/media';

			$media_call = wp_safe_remote_get(
				$api_call_url,
				[
					'body' => $media_call_array,
				]
			);

			// If there is an error with the request, return false.
			if ( is_wp_error( $media_call ) ) {
				return false;
			}

			$response_code = wp_remote_retrieve_response_code( $media_call );
			$response_body = wp_remote_retrieve_body( $media_call );

			$body = json_decode( $response_body, true );

			$next = ! empty( $body['paging']['next'] ) ? $body['paging']['next'] : false;

			$instagram_data = $this->process_instagram_page( $body, $instagram_data, $max_images, $current_image_total, $next, $data );

		}

		return apply_filters( 'envira_instagram_get_instagram_data', $instagram_data, $body['data'], $gallery_id, $data );
	}

	/**
	 * Queries Instagram for image data
	 *
	 * @since 1.0.0
	 *
	 * @param array $body body data.
	 * @param array $instagram_data Array of IG images.
	 * @param int   $max_images Max number of images.
	 * @param int   $current_image_total image total.
	 * @param int   $next_link next link info.
	 * @param array $data Gallery Data.
	 * @return mixed false|Image Array
	 */
	public function process_instagram_page( $body, $instagram_data = false, $max_images = false, $current_image_total = 0, $next_link = false, $data = false ) {

		if ( is_null( $body ) || empty( $body['data'] ) ) {
			return $instagram_data;
		}

		if ( false === $data ) {
			return $instagram_data;
		}

		// Loop through the response data and remove any emoticons that can't be stored in the DB.
		$image_ids = [];
		if ( false === $max_images ) {
			$max_images = ! empty( envira_get_config( 'instagram_number', $data ) ) ? envira_get_config( 'instagram_number', $data ) : 10;
		}

		// Update the current_image_total to make sure we don't grab more images than we need.
		if ( ! empty( $instagram_data ) ) {
			$current_image_total = count( $instagram_data );
		}

		// With addition of multiple instagram accounts, make sure to get the slot.
		$slot        = envira_get_config( 'instagram_account', $data );
		$slot_number = ( false !== $slot ) ? $slot : 1;

		// Grab the Instagram auth data.
		$auth = envira_instagram_get_instagram_auth( $slot_number );

		foreach ( $body['data'] as $i => $image ) {

			// Make sure this image isn't already in the gallery.
			if ( is_array( $image_ids ) && in_array( $image['id'], $image_ids, true ) ) {
				continue;
			}

			if ( strtolower( $image['media_type'] ) !== 'image' && strtolower( $image['media_type'] ) !== 'carousel_album' ) {
				continue;
			}

			if ( $current_image_total >= $max_images ) {
				// we have reached the max set by the user in the gallery settings.
				continue;

			}

			// Determine link.
			if ( envira_get_config( 'instagram_link', $data ) === 'instagram_page' && ! empty( $image['permalink'] ) ) {
				// Add a new value to pass to $instagram_data linking to Instagram page containing image.
				$link           = ( ! empty( $image['permalink'] ) ? esc_url( $image['permalink'] ) : '' );
				$instagram_link = $link;
			} elseif ( envira_get_config( 'instagram_link', $data ) === 'instagram_image' ) {
				// Link to large Instagram image.
				$link           = ( ! empty( $image['media_url'] ) ? esc_url( $image['media_url'] ) : '' );
				$instagram_link = $link;
			} else {
				$link           = false;
				$instagram_link = false;
			}

			// Determine target.
			if ( envira_get_config( 'instagram_link_target', $data ) && ! empty( $image['permalink'] ) ) {
				// Determine if the link to Instagram page opens in a new window.
				$target = '_blank';
			} else {
				$target = false;
			}

			// Attempt to get caption.
			$caption = ( envira_get_config( 'instagram_caption', $data ) && ! empty( $image['caption'] ) ? esc_attr( $image['caption'] ) : '' );

			// Limit caption length, if required.
			$caption_length = envira_get_config( 'instagram_caption_length', $data );
			if ( ! empty( $caption ) && $caption_length > 0 ) {
				$caption_words       = explode( ' ', $caption );
				$caption_words_limit = array_slice( $caption_words, 0, $caption_length );
				$caption             = implode( ' ', $caption_words_limit );
				$caption             = htmlspecialchars_decode( $caption ); // fixes a 'cannot use in in operator' JS error when attempting to open gallery in an album.
			}

			// attempt to grab images, carousels included.
			if ( strtolower( $image['media_type'] ) === 'carousel_album' ) {

				// the children in the array are the key to grabbing each image.
				if ( isset( $image['children'] ) && ! empty( $image['children'] ) ) {
					foreach ( $image['children']['data'] as $child ) {
						if ( isset( $child['media_url'] ) && ! empty( $child['media_url'] ) ) {
							$image['id'] = $child['id'];

							$result_array = $this->process_instagram_image( esc_url( $child['media_url'] ), $instagram_data, $image_ids, $caption, $image, $current_image_total, $i, $data, $instagram_link, $target );

							// $result_array should return an array if successful, but false (or a non-array at least) if not.
							if ( ! is_array( $result_array ) ) {
								continue;
							}

							if ( ! isset( $result_array['current_image_total'] ) || empty( $result_array['current_image_total'] ) ) {
								$result_array['current_image_total'] = isset( $result_array['instagram_data'] ) && ! empty( $result_array['instagram_data'] ) ? count( $result_array['instagram_data'] ) : 0;
							}

							if ( $current_image_total + intval( $result_array['current_image_total'] ) >= $max_images ) {
								continue;
							}

							$instagram_data      = ( isset( $result_array['instagram_data'] ) && false !== $result_array['instagram_data'] ) ? $result_array['instagram_data'] : false;
							$current_image_total = ( isset( $result_array['current_image_total'] ) && false !== $result_array['current_image_total'] ) ? $result_array['current_image_total'] : false;
							$image_ids           = ( isset( $result_array['image_ids'] ) && false !== $result_array['image_ids'] ) ? $result_array['image_ids'] : false;

						}
					}
				}
			} elseif ( $current_image_total < $max_images ) {

					$result_array = $this->process_instagram_image( $image['media_url'], $instagram_data, $image_ids, $caption, $image, $current_image_total, $i, $data, $instagram_link, $target );

					// $result_array should return an array if successful, but false (or a non-array at least) if not.
				if ( ! is_array( $result_array ) ) {
					continue;
				}

				if ( isset( $result_array['instagram_data'] ) && isset( $result_array['current_image_total'] ) && isset( $result_array['image_ids'] ) ) {
					$instagram_data      = ( isset( $result_array['instagram_data'] ) && false !== $result_array['instagram_data'] ) ? $result_array['instagram_data'] : false;
					$current_image_total = ( isset( $result_array['current_image_total'] ) && false !== $result_array['current_image_total'] ) ? $result_array['current_image_total'] : false;
					$image_ids           = ( isset( $result_array['image_ids'] ) && false !== $result_array['image_ids'] ) ? $result_array['image_ids'] : false;
				}
			}
		}

		// ensure there are no duplicate instagram images.
		$attachment_id_array = [];
		foreach ( $instagram_data as $index => $item ) {
			if ( ! empty( $attachment_id_array ) && in_array( $item['attachment_id'], $attachment_id_array, true ) ) {
				unset( $instagram_data[ $index ] );
			} else {
				$attachment_id_array[] = $item['attachment_id'];
			}
		}

		// Return the Instagram data, compatible for the Envira Gallery.
		return $instagram_data;
	}

	/**
	 * Processes an image from an Instagram feed.
	 *
	 * @since 1.0.0
	 *
	 * @param string  $media_url url of image on instagram.
	 * @param array   $instagram_data Array of IG images.
	 * @param int     $image_ids Max number of images.
	 * @param string  $caption Caption.
	 * @param int     $image image info.
	 * @param integer $current_image_total Total.
	 * @param integer $i Counter.
	 * @param array   $data Gallery Data.
	 * @param string  $instagram_link Instagram Link.
	 * @param string  $target Target.
	 * @return mixed  false|Image Array
	 */
	public function process_instagram_image( $media_url = false, $instagram_data = [], $image_ids = [], $caption = false, $image = false, $current_image_total = false, $i = false, $data = false, $instagram_link = false, $target = false ) {

		// grab the "hacked" way to get the full/non-cropped image.
		$starting_image           = $media_url;
		$starting_image           = preg_replace( '/\?.*/', '', $starting_image );
		$starting_image_array     = explode( '/', $starting_image );
		$instagram_image_filename = $starting_image_array[ count( $starting_image_array ) - 1 ];
		$instagram_data           = empty( $instagram_data ) ? [] : $instagram_data;
		$check_file               = wp_check_filetype( $instagram_image_filename );

		if ( ! isset( $check_file ) || false === $check_file['ext'] || false === $check_file['type'] ) {
			return;
		}

		global $wp_filesystem;

		$image_url       = ( $media_url );
		$post_id         = 0;
		$upload_dir_info = wp_upload_dir();
		$upload_dir      = trailingslashit( $upload_dir_info['path'] );
		$new_file_path   = $upload_dir . $instagram_image_filename;

		// if the file exists, we need the attachment id so we don't upload/import this image from Instagram again.
		$args  = [
			'post_type'   => 'attachment',
			'post_status' => 'inherit', // <<-- IMPORTANT
			'meta_query'  => [
				[
					'key'   => 'envira_instagram_id',
					'value' => $image['id'],
				],
			],
		];
		$query = new \WP_Query( $args );

		if ( file_exists( $new_file_path ) && $query->have_posts() ) {

			$attachment_id  = $query->post->ID;
			$image_data     = wp_get_attachment_image_src( $attachment_id, 'full' );
			$src            = $image_data[0];
			$width          = $image_data[1];
			$height         = $image_data[2];
			$instagram_link = get_post_meta( $attachment_id, 'envira_instagram_permalink', true );

			switch ( $data['config']['instagram_image_size'] ) {
				case 'default':
					// don't know yet.
					$thumbnail_size = 'large'; // this used to be array( 640, 640 ).
					$image_data     = wp_get_attachment_image_src( $attachment_id, $thumbnail_size );
					$thumbnail_url  = $image_data[0];
					$lightbox_url   = $image_data[0];
					break;

				default:
					$image_data    = wp_get_attachment_image_src( $attachment_id, $data['config']['instagram_image_size'] );
					$thumbnail_url = $image_data[0];
					$lightbox_url  = $image_data[0];
					break;
			}

			// Build array of instagram data for this image.
			$instagram_data[ $attachment_id ] = [
				'status'             => 'published',
				'src'                => $lightbox_url,
				'title'              => $caption,
				'width'              => $width,
				'height'             => $height,
				'link'               => $src,
				'instagram_link'     => empty( $data['config']['instagram_link'] ) ? $lightbox_url : $instagram_link,
				'instagram_high_res' => $src,
				'alt'                => apply_filters( 'envira_instagram_image_attribute_alt', $caption, $src, $image ),
				'target'             => $target,
				'caption'            => false,
				'thumb'              => $thumbnail_url,
				'link_new_window'    => 0,
				'attachment_id'      => $attachment_id,
			];

			if ( false === $image_ids ) {
				$image_ids = [];
			}
			$image_ids[] = $image['id'];
			++$current_image_total;

			return (
				[
					'instagram_data'      => $instagram_data,
					'image_ids'           => $image_ids,
					'current_image_total' => $current_image_total,
				]
			);

		} else {

			require_once ABSPATH . '/wp-admin/includes/media.php';
			require_once ABSPATH . '/wp-admin/includes/file.php';
			require_once ABSPATH . '/wp-admin/includes/image.php';
			WP_Filesystem();

			$response = wp_safe_remote_get( html_entity_decode( $image_url ) );

			if ( is_wp_error( $response ) ) {
				return $response->get_error_message();
			}

			$status_code = wp_remote_retrieve_response_code( $response );

			if ( 200 !== intval( $status_code ) ) {
				return false;
			}

			$image_binary = wp_remote_retrieve_body( $response );

			if ( ! $wp_filesystem->put_contents( $upload_dir . $instagram_image_filename, $image_binary ) ) {

				return (
					[
						'instagram_data'      => $instagram_data,
						'image_ids'           => $image_ids,
						'current_image_total' => $current_image_total,
					]
				);
			}

			if ( function_exists( 'mime_content_type' ) ) {
				$new_file_mime = mime_content_type( $new_file_path );
			} else {
				$filetype      = wp_check_filetype( basename( $new_file_path ), null );
				$new_file_mime = ! empty( $filetype['type'] ) ? $filetype['type'] : false;
			}

			$attachment_id = wp_insert_attachment(
				[
					'guid'           => $new_file_path,
					'post_mime_type' => $new_file_mime,
					'post_title'     => $caption,
					'post_content'   => '',
					'post_status'    => 'inherit',
				],
				$new_file_path
			);

			wp_update_attachment_metadata( $attachment_id, wp_generate_attachment_metadata( $attachment_id, $new_file_path ) );

			// add the instagram ID as metadata so we can check this image down the road.
			add_post_meta( $attachment_id, 'envira_instagram_id', $image['id'] );
			add_post_meta( $attachment_id, 'envira_instagram_permalink', $image['permalink'] );
			add_post_meta( $attachment_id, 'envira_instagram_username', $image['username'] );

			// set username in the gallery meta. Refactor it to avoid repetitive checks for every image.
			$gallery_id   = $data['id'];
			$gallery_data = get_post_meta( $gallery_id, '_eg_gallery_data', true );
			$username     = ! empty( envira_get_config( 'instagram_username', $gallery_data ) ) ? envira_get_config( 'instagram_username', $gallery_data ) : false;
			if ( ! $username ) {
				$gallery_data['config']['instagram_username'] = $image['username'];
				update_post_meta( $gallery_id, '_eg_gallery_data', $gallery_data );
			}

			$image_data = wp_get_attachment_image_src( $attachment_id, 'full' );
			$src        = $image_data[0];
			$width      = $image_data[1];
			$height     = $image_data[2];

			switch ( $data['config']['instagram_image_size'] ) {
				case 'default':
					// don't know yet.
					$thumbnail_size = 'large';
					$image_data     = wp_get_attachment_image_src( $attachment_id, $thumbnail_size );
					$thumbnail_url  = $image_data[0];
					$lightbox_url   = $image_data[0];
					break;

				default:
					$image_data    = wp_get_attachment_image_src( $attachment_id, $data['config']['instagram_image_size'] );
					$thumbnail_url = $image_data[0];
					$lightbox_url  = $image_data[0];
					break;
			}

			$image_data = wp_get_attachment_image_src( $attachment_id, 'full' );

			// Build array of instagram data for this image.
			$instagram_data[ $attachment_id ] = [
				'status'             => 'published',
				'src'                => $lightbox_url,
				'title'              => $caption,
				'width'              => $width,
				'height'             => $height,
				'link'               => $src,
				'instagram_link'     => $instagram_link,
				'instagram_high_res' => $src,
				'alt'                => apply_filters( 'envira_instagram_image_attribute_alt', $caption, $src, $image ),
				'target'             => $target,
				'caption'            => false,
				'thumb'              => $thumbnail_url, // is used to be the following expression instead which we have commented out and is likely no longer needed: ( ! empty( $image['images']['thumbnail']['url'] ) ? esc_url( $image['images']['thumbnail']['url'] ) : '' ).
				'link_new_window'    => 0,
				'attachment_id'      => $attachment_id,
			];

		}

		$image_ids[] = $image['id'];

		++$current_image_total;

		return (
			[
				'instagram_data'      => $instagram_data,
				'image_ids'           => $image_ids,
				'current_image_total' => $current_image_total,
			]
		);
	}

	/**
	 * Should we cache this instagram... Envira Cache.
	 *
	 * @since 1.0.0
	 *
	 * @param boolean $should_cache Should Cache.
	 * @param array   $data Gallery Data.
	 * @return mixed false|Image Array
	 */
	public function should_cache( $should_cache, $data ) {

		if ( ! isset( $data['config']['instagram_number'] ) || empty( $data['config']['instagram_number'] ) ) {
			return $should_cache;
		}

		if ( intval( $data['config']['instagram_number'] > $this->limit_per_page ) ) {
			return false;
		}

		return $should_cache;
	}

	/**
	 * Returns the singleton instance of the class.
	 *
	 * @since 1.0.0
	 *
	 * @return self The Envira_Instagram_Shortcode object.
	 */
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}
}
