<?php
/**
 * Common
 *
 * @since 1.5.0
 *
 * @package Envira_Instagram
 * @author  Envira Gallery Team <support@enviragallery.com>
 */

/**
 * Adds the default settings for this addon.
 *
 * @since 1.0.0
 *
 * @param array $defaults  Array of default config values.
 * @param int   $post_id     The current post ID.
 * @return array $defaults Amended array of default config values.
 */
function envira_instagram_defaults( $defaults, $post_id ) {

	$defaults['instagram_type']           = 'users_self_media_recent';
	$defaults['instagram_number']         = 5;
	$defaults['instagram_res']            = 'standard_resolution';
	$defaults['instagram_link']           = '';
	$defaults['instagram_image_size']     = 'default';
	$defaults['instagram_link_target']    = 0;
	$defaults['instagram_caption']        = 1;
	$defaults['instagram_caption_length'] = 999;
	$defaults['instagram_random']         = 0;
	$defaults['instagram_cache']          = 1;

	return $defaults;
}
add_filter( 'envira_gallery_defaults', 'envira_instagram_defaults', 10, 2 );

/**
 * Returns the URL to begin the Instagram oAuth process with
 *
 * @since 1.0.5
 *
 * @param   string $return_to URL to return to.
 * @return  string  Instagram oAuth URL
 */
function envira_instagram_get_oauth_url( $return_to ) {

	$return = add_query_arg(
		[
			'instagram_nonce' => wp_create_nonce( 'eg_instagram_auth' ),
		],
		admin_url( $return_to )
	);

	$url = add_query_arg(
		[
			'client_id'     => '197680837962436',
			'response_type' => 'code',
			'scope'         => 'user_profile,user_media',
			'redirect_uri'  => 'https://us-central1-envira-gallery-instagram.cloudfunctions.net/instagram_auth',
			'state'         => rawurlencode( $return ),
		],
		'https://api.instagram.com/oauth/authorize/'
	);

	return $url;
}

/**
 * Returns Instagram auth data.
 *
 * @since 1.4.2
 * @param int $slot Slot.
 * @return array Access token on success, false on failure.
 */
function envira_instagram_get_instagram_auth( $slot = 1 ) {

	if ( intval( $slot ) <= 1 ) {
		$option = get_option( 'envira_instagram' );
	} else {
		$option = get_option( 'envira_instagram_' . $slot );
	}
	return empty( $option ) ? [] : $option;
}

/**
 * Returns the available Instagram query types.
 *
 * @since 1.0.0
 *
 * @return array Array of Instagram query types.
 */
function envira_instagram_instagram_types() {

	$types = [
		[
			'value' => 'users_self_media_recent',
			'name'  => __( 'My Instagram Photos', 'envira-instagram' ),
		],
	];

	return apply_filters( 'envira_instagram_types', $types );
}

/**
 * Returns the available accounts.
 *
 * @since 1.4.2
 *
 * @return array Array of Instagram query types.
 */
function envira_instagram_instagram_accounts() {

	$total_accounts = 3;

	for ( $x = 1; $x <= $total_accounts; $x++ ) {
		$temp_auth = envira_instagram_get_instagram_auth( $x );
		if ( $temp_auth ) {
			$accounts[] = [
				'value' => $x,
				'name'  => ( ! empty( $temp_auth['username'] ) ) ? $temp_auth['username'] : 'Instagram Account #' . $x,
			];
		}
	}

	return apply_filters( 'envira_instagram_accounts', $accounts );
}

/**
 * Returns the available Instagram image resolutions.
 *
 * @since 1.0.0
 *
 * @return array Array of Instagram image resolutions.
 */
function envira_instagram_instagram_resolutions() {

	$resolutions = [
		[
			'value' => 'thumbnail',
			'name'  => __( 'Thumbnail (150x150)', 'envira-instagram' ),
		],
		[
			'value' => 'low_resolution',
			'name'  => __( 'Low Resolution (306x306)', 'envira-instagram' ),
		],
		[
			'value' => 'standard_resolution',
			'name'  => __( 'Standard Resolution (640x640)', 'envira-instagram' ),
		],
	];

	return apply_filters( 'envira_instagram_resolutions', $resolutions );
}

/**
 * Returns the available Instagram link options.
 *
 * @since 1.0.0
 *
 * @return array Array of Instagram image resolutions.
 */
function envira_instagram_get_instagram_link_options() {

	$link_options = [
		[
			'value' => '',
			'name'  => __( 'No link (Use Lightbox If Enabled)', 'envira-instagram' ),
		],
		[
			'value' => 'instagram_page',
			'name'  => __( 'Original Page at Instagram', 'envira-instagram' ),
		],
		[
			'value' => 'instagram_image',
			'name'  => __( 'Direct Full Sized Image On Site', 'envira-instagram' ),
		],
	];

	return apply_filters( 'envira_instagram_link_options', $link_options );
}

/**
 * Check if OpenSSL extension loaded
 *
 * @since 1.0.0
 *
 * @return bool
 */
function envira_instagram_doing_openssl() {
	return extension_loaded( 'openssl' );
}

/**
 * Clear all instagram images.
 *
 * @param string $username Instagram username or 'all' to delete all Instagram images.
 * @return void
 * @since  1.9.9.2
 */
function envira_instagram_remove_images( $username ) {

	if ( '' === $username || empty( $username ) ) {
		return;
	} elseif ( 'all' === $username ) {
		$meta_query = [
			[
				'key'     => 'envira_instagram_id',
				'compare' => 'EXISTS',
			],
		];
	} else {
		$meta_query = [
			[
				'key'     => 'envira_instagram_username',
				'value'   => $username,
				'compare' => 'EXISTS',
			],
		];
	}

	$args = [
		'post_type'      => 'attachment',
		'post_status'    => 'any',
		'posts_per_page' => -1,
		'no_found_rows'  => true,
		'fields'         => 'ids',
		'meta_query'     => $meta_query, // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
	];

	$images = new WP_Query( $args );

	if ( ! isset( $images->posts ) || empty( $images->posts ) ) {
		return;
	}

	// Now loop through all the images found and only use Instagram images.
	foreach ( $images->posts as $image_id ) {
		// Deletes the file and the post meta of the attachment.
		wp_delete_attachment( $image_id, true );
	}
}

/**
 * Clear galleries that use Instagram as a source.
 *
 * @param string $username Instagram username.
 * @return void
 * @since  1.9.9.2
 */
function envira_instagram_remove_galleries( $username = '' ) {

	if ( '' === $username || empty( $username ) ) {
		return;
	} elseif ( 'all' === $username ) {

		$args = [
			'post_type'      => 'envira',
			'post_status'    => 'any',
			'posts_per_page' => -1,
			'no_found_rows'  => true,
			'fields'         => 'ids',
			'meta_query'     => [ // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
				'relation' => 'AND',
				[
					'key'     => '_eg_gallery_data',
					'value'   => 'instagram',
					'compare' => 'LIKE',
				],
				[
					'key'     => '_eg_gallery_data',
					'value'   => 'instagram_account',
					'compare' => 'LIKE',
				],
			],
		];

		$galleries = new WP_Query( $args );

		if ( ! isset( $galleries->posts ) || empty( $galleries->posts ) ) {
			return;
		}

		// Now loop through all the galleries found and only use Instagram galleries.
		foreach ( $galleries->posts as $gallery_id ) {
			$data = get_post_meta( $gallery_id, '_eg_gallery_data', true );

			if ( ! empty( $data['gallery'] ) ) {
				continue;
			}

			$type = envira_get_config( 'type', $data );

			if ( 'instagram' === $type ) {
				delete_post_meta( $gallery_id, true );
				wp_delete_post( $gallery_id, true );
				envira_instagram_remove_shortcodes( $gallery_id );
			}
		}
	} else {
		$gallery_ids = envira_instagram_get_gallery_by_username( $username );
		if ( $gallery_ids && count( $gallery_ids ) > 0 ) {
			foreach ( $gallery_ids as $gallery_id ) {
				delete_post_meta( $gallery_id, true );
				wp_delete_post( $gallery_id, true );
				envira_instagram_remove_shortcodes( $gallery_id );
			}
		}

		// Check the slot with the given username and delete that option.
		for ( $x = 1; $x <= 3; $x++ ) {
			if ( $x > 1 ) {
				$key = "envira_instagram_{$x}";
			} else {
				$key = 'envira_instagram';
			}
			$option = get_option( $key );
			if ( isset( $option ) && isset( $option['username'] ) ) {
				if ( $username === $option['username'] ) {
					delete_option( $key );
				}
			}
		}
	}
}

/**
 * Replace Envira Instagram gallery shortcodes.
 *
 * @param int $gallery_id gallery ID.
 * @return void
 * @since 1.9.9.2
 */
function envira_instagram_remove_shortcodes( $gallery_id ) {
	global $wpdb;

	$shortcode_double_quotes = '%[envira-gallery id="' . $gallery_id . '"]%';
	$shortcode_single_quotes = "%[envira-gallery id='{$gallery_id}']%";

	$query = $wpdb->remove_placeholder_escape(
		$wpdb->prepare(
			"SELECT ID from $wpdb->posts WHERE post_content LIKE %s OR post_content like %s ",
			[ $shortcode_double_quotes, $shortcode_single_quotes ]
		)
	);

	// phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching,WordPress.DB.PreparedSQL.NotPrepared
	$object_ids = $wpdb->get_results( $query );

	if ( ! isset( $object_ids ) || empty( $object_ids ) ) {
		return;
	}

	if ( $object_ids ) {

		foreach ( $object_ids as $object_id ) {

			if ( isset( $object_id->ID ) ) {
				$object = get_post( $object_id->ID );

				// Check if any Gutenberg block exists.
				if ( function_exists( 'register_block_type' ) && has_blocks( $object ) ) {
					envira_instagram_remove_gallery_blocks( $object_id->ID, $gallery_id, $shortcode_double_quotes );
				} else {
					// Replace Envira gallery shortcodes in the content.
					$query = $wpdb->remove_placeholder_escape(
						$wpdb->prepare(
							"UPDATE $wpdb->posts SET post_content = REPLACE(post_content,%s,%s) WHERE ID = %d",
							[ trim( $shortcode_double_quotes, '%' ), '', $object_id->ID ]
						)
					);
					// phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching,WordPress.DB.PreparedSQL.NotPrepared
					$wpdb->query( $query );
				}
			}
		}
	}
}

/**
 * Remove Envira Gallery blocks that use Instagram as source.
 *
 * @param int    $object_id ID of the post object.
 * @param int    $gallery_id ID of the Envira gallery.
 * @param string $shortcode Shortcode of the gallery.
 * @return void
 * @since 1.9.9.2
 */
function envira_instagram_remove_gallery_blocks( $object_id, $gallery_id, $shortcode ) {
	$object = get_post( $object_id );
	$blocks = parse_blocks( $object->post_content );
	foreach ( $blocks as $key => $block ) {
		if ( 'envira/envira-gallery' === $block['blockName'] && $gallery_id === $block['attrs']['galleryId'] ) {
			unset( $blocks[ $key ] );
		} elseif ( 'core/shortcode' === $block['blockName'] && trim( $shortcode, '%' === $block['innerHTML'] ) ) {
			unset( $blocks[ $key ] );
		}
	}
	$object->post_content = serialize_blocks( $blocks );
	wp_update_post( $object );
}

/**
 * Remove the transients set by Envira Instagram add-on.
 *
 * @param string $username Instagram username or 'all' to delete all transients.
 * @return void
 * @since  1.9.9.2
 */
function envira_instagram_remove_transients( $username = 'all' ) {

	if ( '' === $username || empty( $username ) ) {
		return;
	} elseif ( 'all' === $username ) {

		global $wpdb;

		$prefix = $wpdb->esc_like( '_transient__envira_instagram_' );
		$sql    = "SELECT `option_name` FROM $wpdb->options WHERE `option_name` LIKE '%s'";
		// phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery,WordPress.DB.DirectDatabaseQuery.NoCaching,WordPress.DB.PreparedSQL.NotPrepared
		$keys = $wpdb->get_results( $wpdb->prepare( $sql, $prefix . '%' ), ARRAY_A );

		if ( is_wp_error( $keys ) ) {
			return;
		}

		foreach ( $keys as $key ) {
			// Remove '_transient_' from the option name.
			$option_name = substr( $key['option_name'], strlen( '_transient_' ) );
			delete_transient( $option_name );
		}
	} else {
		$gallery_ids = envira_instagram_get_gallery_by_username( $username );
		if ( $gallery_ids && count( $gallery_ids ) > 0 ) {
			foreach ( $gallery_ids as $gallery_id ) {
				delete_transient( "_envira_instagram_{$gallery_id}" );
			}
		}
	}
}

/**
 * Clears all platform data.
 *
 * @param string $username Instagram username.
 * @return void
 * @since  1.9.9.2
 */
function envira_instagram_clear_plaform_data( $username = '' ) {

	/**
	 * Retain this order of these methods, as they all depend on Envria gallery meta data,
	 * delete the galleries at the last step.
	 */

	envira_instagram_remove_transients( $username );
	envira_instagram_remove_images( $username );
	envira_instagram_remove_galleries( $username );

	if ( 'all' === $username ) {
		// Remove options data.
		delete_option( 'envira_instagram' );
		delete_option( 'envira_instagram_2' );
		delete_option( 'envira_instagram_3' );
	}
}

/**
 * Get Instagram gallery IDs by username.
 *
 * @param  @param string $username Instagram username.
 * @return array array of IDs on success or empty array when no galleries found.
 * @since  1.9.9.2
 */
function envira_instagram_get_gallery_by_username( $username ) {
	$args  = [
		'post_type'      => 'envira',
		'fields'         => 'ids',
		'posts_per_page' => -1,
		'meta_query'     => [
			'relation' => 'AND',
			[
				'key'     => '_eg_gallery_data',
				'value'   => 'instagram_username',
				'compare' => 'LIKE',
			],
			[
				'key'     => '_eg_gallery_data',
				'value'   => $username,
				'compare' => 'LIKE',
			],
		],
	];
	$query = new WP_Query( $args );
	$ids   = [];
	if ( $query->posts ) {
		foreach ( $query->posts as $key => $post_id ) {
			$ids[] = $post_id;
		}
	}
	return $ids;
}

/**
 * Update old galleries and set username to gallery meta.
 *
 * @return void
 * @since  1.9.9.2
 */
function envira_instagram_add_username() {
	$args = [
		'post_type'      => 'envira',
		'post_status'    => 'any',
		'posts_per_page' => -1,
		'no_found_rows'  => true,
		'fields'         => 'ids',
		'meta_query'     => [ // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
			'relation' => 'AND',
			[
				'key'     => '_eg_gallery_data',
				'value'   => 'instagram',
				'compare' => 'LIKE',
			],
			[
				'key'     => '_eg_gallery_data',
				'value'   => 'instagram_account',
				'compare' => 'LIKE',
			],
		],
	];

	$galleries = new WP_Query( $args );

	if ( ! isset( $galleries->posts ) || empty( $galleries->posts ) ) {
		return;
	}

	// Now loop through all the galleries found and only use Instagram galleries.
	foreach ( $galleries->posts as $gallery_id ) {
		$data = get_post_meta( $gallery_id, '_eg_gallery_data', true );

		if ( ! empty( $data['gallery'] ) ) {
			continue;
		}

		// check if instagram_username is not already set.
		if ( 'instagram' === $data['config']['type'] && '' === $data['config']['instagram_username'] ) {
			$slot = $data['config']['instagram_account'];
			if ( 1 === $slot ) {
				$key = 'envira_instagram';
			} else {
				$key = "envira_instagram_{$slot}";
			}
			$option = get_option( $key );
			if ( isset( $option ) && isset( $option['username'] ) ) {
				$data['config']['instagram_username'] = $option['username'];
				update_post_meta( $gallery_id, '_eg_gallery_data', $data );
				// TODO: Add Instagram username to all gallery images query_instagram_data.
			}
		}
	}
}
