<?php
/**
 * Plugin Name: Envira Gallery - Proofing Addon
 * Plugin URI:  http://enviragallery.com
 * Description: Allows clients to login to Galleries and choose photos for ordering.
 * Author:      Envira Gallery Team
 * Author URI:  http://enviragallery.com
 * Version:     2.0.8
 * Text Domain: envira-proofing
 * Domain Path: languages
 *
 * @package Envira Gallery
 * @subpackage Envira Proofing
 *
 * Envira Gallery is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * any later version.
 *
 * Envira Gallery is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Envira Gallery. If not, see <http://www.gnu.org/licenses/>.
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( function_exists( 'envira_license_checker' ) && false === envira_license_checker() ) {
	return false;
}

use Envira\Proofing\Admin\Admin_Container;
use Envira\Proofing\Frontend\Frontend_Container;
use Envira\Proofing\Frontend\Order;

// Register the installation/uninstall hooks.
register_activation_hook( __FILE__, 'Envira_Proofing::activate' );

if ( ! class_exists( 'Envira_Proofing' ) ) :

	/**
	 * Main plugin class.
	 *
	 * @since 1.6.0
	 *
	 * @package Envira_Proofing
	 * @author  Envira Team
	 */
	class Envira_Proofing {

		/**
		 * Holds the class object.
		 *
		 * @since 1.6.0
		 *
		 * @var object
		 */
		public static $instance;

		/**
		 * Plugin version, used for cache-busting of style and script file references.
		 *
		 * @since 1.6.0
		 *
		 * @var string
		 */
		public $version = '2.0.8';

		/**
		 * The name of the plugin.
		 *
		 * @since 1.6.0
		 *
		 * @var string
		 */
		public $plugin_name = 'Envira Proofing';

		/**
		 * Unique plugin slug identifier.
		 *
		 * @since 1.6.0
		 *
		 * @var string
		 */
		public $plugin_slug = 'envira-proofing';

		/**
		 * Plugin file.
		 *
		 * @since 1.6.0
		 *
		 * @var string
		 */
		public $file = __FILE__;

		/**
		 * Primary class constructor.
		 *
		 * @since 1.6.0
		 */
		public function __construct() {

			// Fire a hook before the class is setup.
			do_action( 'envira_proofing_pre_init' );

			// Load the plugin textdomain.
			add_action( 'plugins_loaded', [ $this, 'load_plugin_textdomain' ] );

			// Load the plugin.
			add_action( 'envira_gallery_loaded', [ $this, 'init' ], 98 );
		}

		/**
		 * Setup Plugin Constants.
		 *
		 * @since 1.6.0
		 *
		 * @access public
		 * @return void
		 */
		public function setup_constants() {

			if ( ! defined( 'ENVIRA_PROOFING_VERSION' ) ) {

				define( 'ENVIRA_PROOFING_VERSION', $this->version );

			}

			if ( ! defined( 'ENVIRA_PROOFING_SLUG' ) ) {

				define( 'ENVIRA_PROOFING_SLUG', $this->plugin_slug );

			}

			if ( ! defined( 'ENVIRA_PROOFING_FILE' ) ) {

				define( 'ENVIRA_PROOFING_FILE', $this->file );

			}

			if ( ! defined( 'ENVIRA_PROOFING_DIR' ) ) {

				define( 'ENVIRA_PROOFING_DIR', plugin_dir_path( __FILE__ ) );

			}

			if ( ! defined( 'ENVIRA_PROOFING_URL' ) ) {

				define( 'ENVIRA_PROOFING_URL', plugin_dir_url( __FILE__ ) );

			}
		}

		/**
		 * Loads the plugin textdomain for translation.
		 *
		 * @since 1.6.0
		 */
		public function load_plugin_textdomain() {

			load_plugin_textdomain( $this->plugin_slug, false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
		}

		/**
		 * Registers the Envira Proofing widget.
		 *
		 * @since 1.6.0
		 */
		public function widget() {
		}

		/**
		 * Loads the plugin into WordPress.
		 *
		 * @since 1.6.0
		 */
		public function init() {

			// Check if we need to run an update routine.
			$this->maybe_run_update();

			// Run hook once Envira has been initialized.
			do_action( 'envira_proofing_init' );

			new Frontend_Container();

			// Load admin only components.
			if ( is_admin() ) {
				new Admin_Container();
			}

			// Load the updater.
			add_action( 'envira_gallery_updater', [ $this, 'updater' ] );

			// Fire a hook after.
			do_action( 'envira_proofing_post_init' );
		}

		/**
		 * Undocumented function
		 *
		 * @return void
		 */
		public function maybe_run_update() {

			$version = get_option( 'envira_proofing_version' );

			if ( version_compare( $version, ENVIRA_VERSION, '<' ) ) {

				envira_flush_all_cache();

				update_option( 'envira_proofing_version', ENVIRA_VERSION, 'no' );

			}
		}

		/**
		 * Initializes the addon updater.
		 *
		 * @since 1.6.0
		 *
		 * @param string $key The user license key.
		 */
		public function updater( $key ) {

			$args = [
				'plugin_name' => $this->plugin_name,
				'plugin_slug' => $this->plugin_slug,
				'plugin_path' => plugin_basename( __FILE__ ),
				'plugin_url'  => trailingslashit( WP_PLUGIN_URL ) . $this->plugin_slug,
				'remote_url'  => 'https://enviragallery.com/',
				'version'     => $this->version,
				'key'         => $key,
			];

			if ( class_exists( 'Updater' ) ) {
				$updater = new Updater( $args );
			}
		}

		/**
		 * Loads all global files into scope.
		 *
		 * @since 1.7.0
		 */
		public function require_global() {

			require_once trailingslashit( ENVIRA_PROOFING_DIR ) . 'src/Functions/common.php';
		}

		/**
		 * Returns an album based on ID.
		 *
		 * @since 1.6.0
		 *
		 * @param int $id     The album ID used to retrieve an album.
		 * @return array|bool Array of album data or false if none found.
		 */
		public function get_order( $id ) {

			return envira_get_proofing_order( $id );
		}

		// Legacy Code.

		/**
		 * Internal method that returns an album based on ID.
		 *
		 * @since 1.6.0
		 *
		 * @param int $id     The album ID used to retrieve an album.
		 * @return array|bool Array of album data or false if none found.
		 */
		public function _get_order( $id ) { // phpcs:ignore PSR2.Methods.MethodDeclaration.Underscore

			return _envira_get_proofing_order( $id );
		}


		/**
		 * Returns an album based on slug.
		 *
		 * @since 1.6.0
		 *
		 * @param string $slug The album slug used to retrieve an album.
		 * @return array|bool  Array of album data or false if none found.
		 */
		public function get_order_by_slug( $slug ) {

			return envira_get_proofing_order_by_slug( $slug );
		}

		// Legacy Code.

		/**
		 * Internal method that returns an album based on slug.
		 *
		 * @since 1.6.0
		 *
		 * @param string $slug The album slug used to retrieve an album.
		 * @return array|bool  Array of album data or false if none found.
		 */
		public function _get_order_by_slug( $slug ) { // phpcs:ignore PSR2.Methods.MethodDeclaration.Underscore

			return _envira_get_proofing_order_by_slug( $slug );
		}


		/**
		 * Returns all orders created on the site.
		 *
		 * @since 1.6.0
		 *
		 * @param bool   $skip_empty   Skip empty proofing.
		 * @param bool   $ignore_cache Ignore Transient cache.
		 * @param string $search_terms Search for specified Proofing by Title.
		 * @return array|bool Array of album data or false if none found.
		 */
		public function get_orders( $skip_empty = true, $ignore_cache = false, $search_terms = '' ) {

			return envira_get_proofing_orders( $skip_empty, $ignore_cache, $search_terms );
		}

		// Legacy Code.

		/**
		 * Internal method that returns all proofing created on the site.
		 *
		 * @since 1.6.0
		 *
		 * @param bool   $skip_empty     Skip Empty Proofing.
		 * @param string $search_terms   Search for specified Proofing by Title.
		 * @return mixed                 Array of proofing data or false if none found.
		 */
		public function _get_orders( $skip_empty = true, $search_terms = '' ) { // phpcs:ignore PSR2.Methods.MethodDeclaration.Underscore

			return _envira_get_proofing_orders( $skip_empty, $search_terms );
		}


		/**
		 * Helper Method fires when Envira Proofing Activated.
		 *
		 * @access public
		 * @static
		 * @return void
		 */
		public static function activate() {

			/**
			 * We cannot get to the class here, but registering this and then flushing permalinks should be enough
			 * posttype class will re-register with args
			 */

			register_post_type( 'envira_proof_order' );

			flush_rewrite_rules();
		}

		/**
		 * Autoload function.
		 *
		 * @access public
		 * @static
		 *
		 * @param string $php_class Class to Autoload.
		 *
		 * @return void
		 */
		public static function autoload( $php_class ) {

			// Prepare variables.
			$prefix   = 'Envira\\Proofing\\';
			$base_dir = __DIR__ . '/src/';
			$length   = mb_strlen( $prefix );

			// If the class is not using the namespace prefix, return.
			if ( 0 !== strncmp( $prefix, $php_class, $length ) ) {
				return;
			}

			// Prepare classes to be autoloaded.
			$relative_class = mb_substr( $php_class, $length );
			$file           = $base_dir . str_replace( '\\', '/', $relative_class ) . '.php';

			// If the file exists, load it.
			if ( file_exists( $file ) ) {
				require_once $file;
			}
		}

		/**
		 * Returns the singleton instance of the class.
		 *
		 * @since 1.6.0
		 *
		 * @return object The Envira_Proofing object.
		 */
		public static function get_instance() {

			if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Envira_Proofing ) ) {
				self::$instance = new self();
				self::$instance->setup_constants();
				self::$instance->require_global();

			}

			return self::$instance;
		}
	}

	spl_autoload_register( 'Envira_Proofing::autoload' );

	// Load the plugin.
	add_action(
		'envira_gallery_init',
		function () {
			Envira_Proofing::get_instance();
		}
	);

endif;
