<?php
/**
 * Proofing Admin Container Class
 *
 * @package Envira Proofing
 */

namespace Envira\Proofing\Admin;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use Envira\Proofing\Frontend\Order;
use Envira\Admin\Notices;
use Envira\Proofing\Utils\Capabilities;

/**
 * Albums Admin Container Class
 */
class Admin_Container {

	/**
	 * Holds the number of active orders to add to WP menus.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public $active_orders_count;

	/**
	 * __construct function.
	 *
	 * @access public
	 * @return void
	 */
	public function __construct() {

		$posttype = new Posttype();
		$metabox  = new Metaboxes();
		$settings = new Settings();
		$table    = new Table();
		$cap      = new Capabilities();

		// Load admin assets.
		add_action( 'admin_enqueue_scripts', [ $this, 'admin_styles' ], 20 );

		// Deal with showing active orders, badges, and altering menus.
		add_action( 'admin_init', [ $this, 'calculate_active_orders' ] );
		add_action( 'admin_notices', [ $this, 'notices_check_smtp' ], 9999 );
		add_action( 'admin_menu', [ $this, 'change_menus' ], 9999 );

		// Flush Album Caches on Post/Page/CPT deletion / restore.
		add_action( 'wp_trash_post', [ $this, 'trash_untrash_orders' ] );
		add_action( 'untrash_post', [ $this, 'trash_untrash_orders' ] );
	}

	/**
	 * Loads styles for our admin tables.
	 *
	 * @since 1.0.0
	 *
	 * @return null Return early if not on the proper screen.
	 */
	public function admin_styles() {

		if ( 'envira_proof_order' !== get_current_screen()->post_type ) {
			return;
		}

		wp_register_style( ENVIRA_SLUG . '-admin-style', plugins_url( 'assets/css/admin.css', ENVIRA_FILE ), [], ENVIRA_VERSION );
		wp_enqueue_style( ENVIRA_SLUG . '-admin-style' );
		wp_register_style( ENVIRA_SLUG . '-admin-modal-style', plugins_url( 'assets/css/admin-modal.css', ENVIRA_FILE ), [], ENVIRA_VERSION );
		wp_enqueue_style( ENVIRA_SLUG . '-admin-modal-style' );
		wp_register_style( ENVIRA_PROOFING_SLUG . '-metabox-style', plugins_url( 'assets/css/proofing-admin.css', ENVIRA_PROOFING_FILE ), [], ENVIRA_PROOFING_VERSION );
		wp_enqueue_style( ENVIRA_PROOFING_SLUG . '-metabox-style' );

		// Fire a hook to load in custom admin styles.
		do_action( 'envira_proofing_orders_admin_styles' );
	}

	/**
	 * Calculates active orders
	 *
	 * @since 2.0.0
	 */
	public function calculate_active_orders() {

		$order = new Order();

		$envira_gallery_proofing_active_total = $order->get_active_orders_total();

		$this->active_orders_count = ( $envira_gallery_proofing_active_total ) ? ' <span class="update-plugins count-' . $envira_gallery_proofing_active_total . '"><span class="plugin-count"> ' . $envira_gallery_proofing_active_total . ' </span></span>' : false;
	}

	/**
	 * Calculates active orders
	 *
	 * @since 2.0.0
	 */
	public function change_menus() {
		global $menu;
		global $submenu;

		$cap = new Capabilities();

		if ( current_user_can( 'read_envira_proof', false ) ) {

			$this->calculate_active_orders();

			foreach ( $menu as $menu_order => $menu_item ) {
				if ( isset( $menu_item[5] ) && 'menu-posts-envira_proof_order' === $menu_item[5] ) {
					$menu[ $menu_order ][0] = __( 'Orders', 'envira-proofing' ) . $this->active_orders_count;
				}
			}

			$submenu['edit.php?post_type=envira_proof_order'][5][0] = __( 'Orders', 'envira-proofing' ) . $this->active_orders_count;

		}
	}

	/**
	 * Flush gallery cache when a order is deleted
	 *
	 * @since 2.0.0
	 *
	 * @param int $id The post ID being trashed.
	 * @return null Return early if no album is found.
	 */
	public function trash_untrash_orders( $id ) {

		$album = get_post( $id );

		// Flush necessary gallery caches to ensure trashed orders are not showing.
		envira_flush_gallery_caches( $id );

		// Return early if not an Envira proofing screen.
		if ( 'envira_proof_order' !== $album->post_type ) {
			return;
		}
	}

	/**
	 * Outputs notices related to SMTP
	 *
	 * @since 2.0.0
	 */
	public function notices_check_smtp() {

		$notices = new Notices();

		$warning_message_disabled = envira_proofing_get_setting( 'warning_message_disabled' );

		if ( ! empty( $warning_message_disabled ) ) {
			return;
		}

		do_action( 'envira_proofing_check_smtp_plugins' );

		$functions_to_check = apply_filters( 'envira_proofing_smtp_functions_check', [ 'wmp_mail_smtp' ] );
		$classes_to_check   = apply_filters( 'envira_proofing_smtp_classes_check', [ 'EasyWPSMTP' ] );
		$function_found     = false;
		$class_found        = false;
		if ( ! empty( $functions_to_check ) ) {
			foreach ( $functions_to_check as $function ) {
				if ( function_exists( $function ) ) {
					$function_found = true;
				}
			}
		}
		if ( ! empty( $classes_to_check ) ) {
			foreach ( $classes_to_check as $class ) {
				if ( class_exists( $function ) ) {
					$class_found = true;
				}
			}
		}

		if ( false !== $function_found || false !== $class_found ) {
			return;
		}

		$smtp_plugin_present = apply_filters( 'envira_gallery_proofing_smtp_plugin_activated', false );

		// If there is no license key, output nag about ensuring key is set for automatic updates.
		if ( false === $smtp_plugin_present ) :

			$message = sprintf(
				/* translators: %s: URL */
				__( '<strong>Envira Proofing</strong>: You do not appear to have a SMTP plugin activated. Envira Gallery highly recommends using SMTP (and not default PHP Mail) to ensure reliable email delivery of proofing emails. <a href="%s" target="_blank" rel="noopener">Please see our documentation for more information</a>.', 'envira-gallery' ),
				esc_url(
					add_query_arg(
						[
							'post_type' => 'envira',
							'page'      => 'envira-gallery-settings',
						],
						admin_url( 'edit.php' )
					)
				)
			);

			$notices->display_inline_notice( 'warning-no-smtp', false, $message, 'error', $button_text = '', $button_url = '', true, DAY_IN_SECONDS );

		endif;
	}
}
