<?php
/**
 * Metabox class.
 *
 * @since 1.0.0
 *
 * @package Envira_Proofing
 * @author  Envira Team
 */

namespace Envira\Proofing\Admin;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use Envira\Proofing\Frontend\Order;
use Envira\Admin\Notices;

/**
 * Albums Metabox class
 */
class Metaboxes {

	/**
	 * Holds the class object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public static $instance;

	/**
	 * Path to the file.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	public $file = __FILE__;

	/**
	 * Holds the base class object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public $base;

	/**
	 * Allowed HTML
	 *
	 * @var mixed
	 * @access public
	 */
	public $wp_kses_allowed_html = [
		'a'      => [
			'href'                => [],
			'target'              => [],
			'class'               => [],
			'title'               => [],
			'data-status'         => [],
			'data-envira-tooltip' => [],
			'data-id'             => [],
		],
		'br'     => [],
		'img'    => [
			'src'   => [],
			'class' => [],
			'alt'   => [],
		],
		'div'    => [
			'class' => [],
		],
		'li'     => [
			'id'                              => [],
			'class'                           => [],
			'data-envira-gallery-image'       => [],
			'data-envira-gallery-image-model' => [],
		],
		'em'     => [],
		'span'   => [
			'class' => [],
		],
		'strong' => [],
	];

	/**
	 * Primary class constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {

		// Load the base class object.

		// Scripts and styles.
		add_action( 'envira_gallery_metabox_styles', [ $this, 'meta_box_styles' ] );
		add_action( 'envira_gallery_metabox_scripts', [ $this, 'meta_box_scripts' ] );

		// Tab and Metabox.
		add_filter( 'envira_gallery_tab_nav', [ $this, 'tabs' ] );
		add_action( 'envira_gallery_lightbox_box', [ $this, 'lightbox_box' ] );
		add_action( 'envira_gallery_tab_proofing', [ $this, 'proofing_box' ] );

		// Save Settings.
		add_filter( 'envira_gallery_save_settings', [ $this, 'save' ], 10, 2 );

		// Upgrade Orders.
		add_filter( 'admin_init', [ $this, 'upgrade_legacy_orders' ], 10 );

		// Misc.
		add_filter( 'admin_body_class', [ $this, 'body_css' ], 10, 2 );
	}

	/**
	 * Adds a tab for this Addon
	 *
	 * @since 1.0.0
	 *
	 * @param string $classes CSS Classes.
	 * @return array Tabs
	 */
	public function body_css( $classes ) {

		$query      = [
			'post_type'   => 'envira_proof_order',
			'post_status' => [ 'publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit', 'proofing-completed', 'proofing-pending', 'proofing-active' ],
		];
		$post_count = new \WP_Query( $query ); // find a better way.

		$classes .= ' envira-' . $post_count->post_count . '-orders ';

		return $classes;
	}

	/**
	 * Enqueues CSS for the metabox
	 *
	 * @since 1.0.0
	 */
	public function meta_box_styles() {

		wp_enqueue_style( ENVIRA_PROOFING_SLUG . '-metabox-style', plugins_url( 'assets/css/proofing-admin.css', ENVIRA_PROOFING_FILE ), [], ENVIRA_PROOFING_VERSION );
	}

	/**
	 * Enqueues JS for the metabox
	 *
	 * @since 1.0.0
	 */
	public function meta_box_scripts() {

		wp_enqueue_script( ENVIRA_PROOFING_SLUG . '-metabox-script', envira_script( 'assets/js/min/metabox-min.js', ENVIRA_PROOFING_FILE ), [ 'jquery', 'jquery-ui-sortable' ], ENVIRA_PROOFING_VERSION, true );
		wp_localize_script(
			ENVIRA_PROOFING_SLUG . '-metabox-script',
			'envira_proofing_metabox',
			[
				'delete_size'  => __( 'Are you sure you want to delete this size?', 'envira-proofing' ),
				'clear_order'  => __( 'Are you sure you want to clear this order? This action cannot be reversed, and the user will need to re-order again.', 'envira-proofing' ),
				'unlock_order' => __( 'Are you sure you want to unlock this order? This action cannot be reversed, and the user will need to re-submit again.', 'envira-proofing' ),
				'nonce'        => wp_create_nonce( 'envira-proofing-clear-order' ),
			]
		);

		// Conditional Fields.
		wp_register_script( ENVIRA_PROOFING_SLUG . '-conditional-fields-script', envira_script( 'assets/js/min/conditional-fields-min.js', ENVIRA_PROOFING_FILE ), [ 'jquery', ENVIRA_SLUG . '-conditional-fields-script' ], ENVIRA_PROOFING_VERSION, true );
		wp_enqueue_script( ENVIRA_PROOFING_SLUG . '-conditional-fields-script' );
	}

	/**
	 * Adds a tab for this Addon
	 *
	 * @since 1.0.0
	 *
	 * @param array $tabs Tabs.
	 * @return array Tabs
	 */
	public function tabs( $tabs ) {

		$tabs['proofing'] = __( 'Proofing', 'envira-proofing' );
		return $tabs;
	}

	/**
	 * Adds addon settings UI to the Lightbox tab
	 *
	 * @since 1.0.0
	 *
	 * @param object $post The current post object.
	 */
	public function lightbox_box( $post ) {

		// Setup instance and meta key.

		$data = ! empty( $post->ID ) ? get_post_meta( $post->ID, '_eg_gallery_data', true ) : false;

		?>
		<tr id="envira-config-lightbox-proofing-box">
			<th scope="row">
				<label for="envira-config-lightbox-proofing"><?php esc_html_e( 'Display Proofing Options?', 'envira-proofing' ); ?></label>
			</th>
			<td>
				<input id="envira-config-lightbox-proofing" type="checkbox" name="_envira_gallery[proofing_lightbox]" value="<?php echo esc_html( envira_get_config( 'proofing_lightbox', $data, envira_proofing_get_global_config_defaults( 'global_proofing_lightbox' ) ) ); ?>" <?php checked( envira_get_config( 'proofing_lightbox', $data, envira_proofing_get_global_config_defaults( 'global_proofing_lightbox' ) ), 1 ); ?> />
				<span class="description"><?php esc_html_e( 'If enabled, and the Proofing Addon is configured, displays Proofing options for each image in the Lightbox view.', 'envira-proofing' ); ?></span>
			</td>
		</tr>
		<?php
	}

	/**
	 * Adds addon settings UI to the Proofing tab
	 *
	 * @since 1.0.0
	 *
	 * @param object $post The current post object.
	 */
	public function proofing_box( $post ) {

		// Setup instance and meta key.
		$proofing_order = new Order();
		$orders         = ! empty( $post->ID ) ? $proofing_order->get_orders( $post->ID ) : false;
		$data           = ! empty( $post->ID ) ? get_post_meta( $post->ID, '_eg_gallery_data', true ) : false;
		$key            = '_envira_gallery';
		$gallery_id     = ! empty( $post->ID ) ? ( $post->ID ) : false;

		// Get orientations.
		$orientations = envira_proofing_get_size_orientations();

		// If the Defaults Addon isn't enabled, show a notice that this can all be preconfigured to save time
		// if the user installs our Defaults Addon.
		if ( ! is_plugin_active( 'envira-defaults/envira-defaults.php' ) ) {
			if ( class_exists( 'Envira_Gallery_Notice_Admin' ) ) {
				// Display notice.
				Envira_Gallery_Notice_Admin::get_instance()->display_inline_notice(
					'proofing_defaults',
					__( 'Want to preconfigure Proofing options and sizes?', 'envira-proofing' ),
					__( 'Install and activate the <strong>Envira Defaults Addon</strong> to set Proofing options for all future Envira Galleries.', 'envira-proofing' ),
					'success',
					__( 'Click here to manage your Envira Addons', 'envira-proofing' ),
					'edit.php?post_type=envira&amp;page=envira-gallery-settings#!envira-tab-addons',
					true
				);
			}
		}
		?>
		<div id="envira-proofing">
			<p class="envira-intro">
				<?php esc_html_e( 'Proofing Gallery Settings', 'envira-proofing' ); ?>
				<small>
					<?php esc_html_e( 'The settings below adjust the Proofing options for the Gallery.', 'envira-proofing' ); ?>
					<br />
					<?php esc_html_e( 'Need some help?', 'envira-proofing' ); ?>
					<a href="http://enviragallery.com/docs/proofing-addon/" class="envira-doc" target="_blank" rel="noopener">
						<?php esc_html_e( 'Read the Documentation', 'envira-proofing' ); ?>
					</a>
					or
					<a href="https://www.youtube.com/embed/Hg12iisPEdw/?rel=0" class="envira-video" target="_blank" rel="noopener">
						<?php esc_html_e( 'Watch a Video', 'envira-proofing' ); ?>
					</a>
				</small>
			</p>
			<table class="form-table">
				<tbody>
					<tr id="envira-proofing-enabled-box">
						<th scope="row">
							<label for="envira-proofing-enabled"><?php esc_html_e( 'Enabled?', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-enabled" type="checkbox" name="<?php echo esc_html( $key ); ?>[proofing]" value="1" <?php checked( envira_get_config( 'proofing', $data, envira_proofing_get_global_config_defaults( 'global_proofing' ) ), 1 ); ?> />
							<span class="description"><?php esc_html_e( 'Enables or disables the Proofing selection process.', 'envira-proofing' ); ?></span>
						</td>
					</tr>
					<tr id="envira-proofing-delete-box">
						<th scope="row">
							<label for="envira-proofing-delete"><?php esc_html_e( 'Allow User to Delete Order?', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-delete" type="checkbox" name="<?php echo esc_html( $key ); ?>[proofing_delete]" value="1" <?php checked( envira_get_config( 'proofing_delete', $data, envira_proofing_get_global_config_defaults( 'global_proofing_delete' ) ), 1 ); ?> />
							<span class="description"><?php esc_html_e( 'Allow Users to delete a Proofing Order they have previously saved.', 'envira-proofing' ); ?></span>
						</td>
					</tr>
					<tr id="envira-proofing-multiple-enabled-box">
						<th scope="row">
							<label for="envira-proofing-multiple-enable"><?php esc_html_e( 'Require Email Address', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-multiple-enabled" type="checkbox" name="<?php echo esc_html( $key ); ?>[proofing_multiple_enabled]" value="1" <?php checked( envira_get_config( 'proofing_multiple_enabled', $data, envira_proofing_get_global_config_defaults( 'global_proofing_multiple_enabled' ) ), 1 ); ?> />
							<span class="description"><?php esc_html_e( 'If enabled, each visitor will need to supply their email address to make an order. This is useful if you have more than one person submitting orders and/or if the gallery is public. Orders will also be displayed by each individual email address below.', 'envira-proofing' ); ?></span>
						</td>
					</tr>
					<tr id="envira-proofing-email-box-position-box">
						<th scope="row">
							<label for="envira-proofing-multiple-enable"><?php esc_html_e( 'Display Email Form?', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<select id="envira-config-email-box-position" name="_envira_gallery[proofing_email_box_position]">
								<?php foreach ( (array) $this->get_proofing_email_box_positions() as $i => $loop_data ) : ?>
									<option value="<?php echo esc_html( $loop_data['value'] ); ?>"<?php selected( $loop_data['value'], envira_get_config( 'proofing_email_box_position', $data, envira_proofing_get_global_config_defaults( 'global_proofing_email_box_position' ) ) ); ?>><?php echo esc_html( $loop_data['name'] ); ?></option>
								<?php endforeach; ?>
							</select>
							<p class="description"><?php esc_html_e( 'Display the email form above or below gallery images.', 'envira-proofing' ); ?></p>
						</td>
					</tr>
					<tr id="envira-proofing-name-enabled-box">
						<th scope="row">
							<label for="envira-proofing-name-enable"><?php esc_html_e( 'Add Name Field?', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-name-enabled" type="checkbox" name="<?php echo esc_html( $key ); ?>[proofing_name_enabled]" value="1" <?php checked( envira_get_config( 'proofing_name_enabled', $data, envira_proofing_get_global_config_defaults( 'global_proofing_name_enabled' ) ), 1 ); ?> />
							<span class="description"><?php esc_html_e( 'If enabled, each visitor will be asked for their name as well as their email address.', 'envira-proofing' ); ?></span>
						</td>
					</tr>

					<tr id="envira-proofing-hide-until-email">
						<th scope="row">
							<label for="envira-proofing-quantity-enabled"><?php esc_html_e( 'Hide Gallery until Email Address entered?', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-quantity-enabled" type="checkbox" name="<?php echo esc_html( $key ); ?>[proofing_hide_gallery]" value="1" <?php checked( envira_get_config( 'proofing_hide_gallery', $data, envira_proofing_get_global_config_defaults( 'global_proofing_hide_gallery' ) ), 1 ); ?> />
							<span class="description"><?php esc_html_e( 'Check this option if you do not want to display the Gallery until the user has entered their email address.', 'envira-proofing' ); ?></span>
						</td>
					</tr>

					<tr id="envira-proofing-edit-box">
						<th scope="row">
							<label for="envira-proofing-edit"><?php esc_html_e( 'Allow User to Edit Submitted Order?', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-edit" type="checkbox" name="<?php echo esc_html( $key ); ?>[proofing_edit]" value="1" <?php checked( envira_get_config( 'proofing_edit', $data, envira_proofing_get_global_config_defaults( 'global_proofing_edit' ) ), 1 ); ?> />
							<span class="description"><?php esc_html_e( 'This option allows users to edit submitted orders.', 'envira-proofing' ); ?></span>
						</td>
					</tr>

					<tr id="envira-proofing-quantity-enabled-box">
						<th scope="row">
							<label for="envira-proofing-quantity-enabled"><?php esc_html_e( 'Enable Quantity Field?', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-quantity-enabled" type="checkbox" name="<?php echo esc_html( $key ); ?>[proofing_quantity_enabled]" value="1" <?php checked( envira_get_config( 'proofing_quantity_enabled', $data, envira_proofing_get_global_config_defaults( 'global_proofing_quantity_enabled' ) ), 1 ); ?> />
							<span class="description"><?php esc_html_e( 'Enables or disables the option for the user to specify a quantity to order for each image.', 'envira-proofing' ); ?></span>
						</td>
					</tr>

					<tr id="envira-proofing-size-enabled-box">
						<th scope="row">
							<label for="envira-proofing-size-enabled"><?php esc_html_e( 'Enable Size Options?', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-size-enabled" type="checkbox" name="<?php echo esc_html( $key ); ?>[proofing_size_enabled]" value="1" <?php checked( envira_get_config( 'proofing_size_enabled', $data, envira_proofing_get_global_config_defaults( 'global_proofing_size_enabled' ) ), 1 ); ?> />
							<span class="description"><?php esc_html_e( 'Enables or disables the option for the user to order different size(s) for each image.', 'envira-proofing' ); ?></span>
						</td>
					</tr>

					<!-- Sizes -->
					<tr id="envira-proofing-sizes-box">
						<th scope="row">
							<label for="envira-proofing-sizes"><?php esc_html_e( 'Sizes', 'envira-proofing' ); ?></label><br />
							<small><a href="#" title="<?php esc_html_e( 'Add Size', 'envira-proofing' ); ?>" class="add"><?php esc_html_e( 'Add Size', 'envira-proofing' ); ?></a></small>
						</th>
						<td>
							<table id="envira-proofing-sizes">
								<tbody>
									<?php
									// Get available sizes.
									$sizes = envira_get_config( 'proofing_sizes', $data, envira_proofing_get_global_config_defaults( 'global_proofing_sizes' ) );
									if ( is_array( $sizes ) && count( $sizes ) > 0 ) {
										foreach ( $sizes as $size ) {
											?>
											<tr class="sortable">
												<td><input type="text" name="<?php echo esc_html( $key ); ?>[proofing_sizes][name][]" value="<?php echo esc_attr( $size['name'] ); ?>" placeholder="<?php esc_html_e( '4x6, 8x10 etc', 'envira-proofing' ); ?>"/></td>
												<td>
													<select name="<?php echo esc_html( $key ); ?>[proofing_sizes][orientation][]" size="1">
														<?php
														foreach ( (array) $orientations as $i => $loop_data ) {
															// Not using selected() as we introduced orientation in 1.0.2, so it's an undefined key for existing
															// Proofing Galleries.
															?>
															<option value="<?php echo esc_html( $loop_data['value'] ); ?>"<?php echo ( ( isset( $size['orientation'] ) && $loop_data['value'] === $size['orientation'] ) ? ' selected' : '' ); ?>><?php echo esc_html( $loop_data['name'] ); ?></option>
															<?php
														}
														?>
													</select>
												</td>
												<td>
													<a href="#" class="move" title="<?php esc_html_e( 'Move', 'envira-proofing' ); ?>"></a>
													<a href="#" class="dashicons dashicons-trash delete" title="<?php esc_html_e( 'Delete', 'envira-proofing' ); ?>"></a>
												</td>
											</tr>
											<?php
										}
									}
									?>

									<tr class="hidden">
										<td><input type="text" name="<?php echo esc_html( $key ); ?>[proofing_sizes][name][]" value="" /></td>
										<td>
											<select name="<?php echo esc_html( $key ); ?>[proofing_sizes][orientation][]" size="1">
												<?php
												foreach ( (array) $orientations as $i => $loop_data ) {
													?>
													<option value="<?php echo esc_html( $loop_data['value'] ); ?>"><?php echo esc_html( $loop_data['name'] ); ?></option>
													<?php
												}
												?>
											</select>
										</td>
										<td>
											<a href="#" class="move" title="<?php esc_html_e( 'Move', 'envira-proofing' ); ?>"></a>
											<a href="#" class="dashicons dashicons-trash delete" title="<?php esc_html_e( 'Delete', 'envira-proofing' ); ?>"></a>
										</td>
									</tr>
								</tbody>
							</table>
						</td>
					</tr>
					<?php

							// defaults for these gets complicated, so we first determine if Envira Default is being used.
					// phpcs:ignore WordPress.Security.NonceVerification.Recommended
					if ( ! isset( $_GET['envira_defaults_config_id'] ) ) {

						$proofing_add_to_order_label = esc_html( empty( $data['id'] ) ) ? esc_attr( envira_proofing_get_global_config_defaults( 'global_proofing_add_to_order_label' ) ) : esc_attr( envira_get_config( 'proofing_add_to_order_label', $data, envira_proofing_get_global_config_defaults( 'proofing_add_to_order_label' ) ) );

						$proofing_save_button_label = esc_html( empty( $data ) ) ? esc_attr( envira_proofing_get_global_config_defaults( 'global_proofing_save_button_label' ) ) : esc_attr( envira_get_config( 'proofing_save_button_label', $data, envira_proofing_get_global_config_defaults( 'proofing_save_button_label' ) ) );

						$proofing_edit_button_label = esc_html( empty( $data ) ) ? esc_attr( envira_proofing_get_global_config_defaults( 'global_proofing_edit_button_label' ) ) : esc_attr( envira_get_config( 'proofing_edit_button_label', $data, envira_proofing_get_global_config_defaults( 'proofing_edit_button_label' ) ) );

						$proofing_delete_button_label = esc_html( empty( $data ) ) ? esc_attr( envira_proofing_get_global_config_defaults( 'global_proofing_delete_button_label' ) ) : esc_attr( envira_get_config( 'proofing_delete_button_label', $data, envira_proofing_get_global_config_defaults( 'proofing_delete_button_label' ) ) );

						$proofing_submit_button_label = esc_html( empty( $data ) ) ? esc_attr( envira_proofing_get_global_config_defaults( 'global_proofing_submit_button_label' ) ) : esc_attr( envira_get_config( 'proofing_submit_button_label', $data, envira_proofing_get_global_config_defaults( 'proofing_submit_button_label' ) ) );

						$proofing_notes_placeholder_text = esc_html( empty( $data ) ) ? esc_attr( envira_proofing_get_global_config_defaults( 'global_proofing_notes_placeholder_text' ) ) : esc_attr( envira_get_config( 'proofing_notes_placeholder_text', $data, envira_proofing_get_global_config_defaults( 'proofing_notes_placeholder_text' ) ) );

						$proofing_save_notes_button_label = esc_html( empty( $data ) ) ? esc_attr( envira_proofing_get_global_config_defaults( 'global_proofing_save_notes_button_label' ) ) : esc_attr( envira_get_config( 'proofing_save_notes_button_label', $data, envira_proofing_get_global_config_defaults( 'proofing_save_notes_button_label' ) ) );

					} else {

								$proofing_add_to_order_label      = envira_get_config( 'proofing_add_to_order_label', $data, envira_proofing_get_global_config_defaults( 'proofing_add_to_order_label' ) );
								$proofing_save_button_label       = envira_get_config( 'proofing_save_button_label', $data, envira_proofing_get_global_config_defaults( 'proofing_save_button_label' ) );
								$proofing_edit_button_label       = envira_get_config( 'proofing_edit_button_label', $data, envira_proofing_get_global_config_defaults( 'proofing_edit_button_label' ) );
								$proofing_delete_button_label     = envira_get_config( 'proofing_delete_button_label', $data, envira_proofing_get_global_config_defaults( 'proofing_delete_button_label' ) );
								$proofing_submit_button_label     = envira_get_config( 'proofing_submit_button_label', $data, envira_proofing_get_global_config_defaults( 'proofing_submit_button_label' ) );
								$proofing_notes_placeholder_text  = envira_get_config( 'proofing_notes_placeholder_text', $data, envira_proofing_get_global_config_defaults( 'proofing_notes_placeholder_text' ) );
								$proofing_save_notes_button_label = envira_get_config( 'proofing_save_notes_button_label', $data, envira_proofing_get_global_config_defaults( 'proofing_save_notes_button_label' ) );

					}

					?>
					<tr id="envira-proofing-add-to-order-label-box">
						<th scope="row">
							<label for="envira-proofing-add-to-order-label"><?php esc_html_e( 'Add to Order Label', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-add-to-order-label" type="text" name="<?php echo esc_html( $key ); ?>[proofing_add_to_order_label]" value="<?php echo ( esc_html( $proofing_add_to_order_label ) ); ?>" /><br />
							<span class="description"><?php esc_html_e( 'The text to display on the Add to Order label in the Lightbox view.', 'envira-proofing' ); ?></span>
						</td>
					</tr>
					<tr id="envira-proofing-save-button-label-box">
						<th scope="row">
							<label for="envira-proofing-save-button-label"><?php esc_html_e( 'Save Button Label', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-save-button-label" type="text" name="<?php echo esc_html( $key ); ?>[proofing_save_button_label]" value="<?php echo ( esc_html( $proofing_save_button_label ) ); ?>" /><br />
							<span class="description"><?php esc_html_e( 'The text to display on the save button, which allows the user to save the order.', 'envira-proofing' ); ?></span>
						</td>
					</tr>
					<tr id="envira-proofing-edit-button-label-box">
						<th scope="row">
							<label for="envira-proofing-edit-button-label"><?php esc_html_e( 'Edit Order Button Label', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-edit-button-label" type="text" name="<?php echo esc_html( $key ); ?>[proofing_edit_button_label]" value="<?php echo ( esc_html( $proofing_edit_button_label ) ); ?>" /><br />
							<span class="description"><?php esc_html_e( 'The text to display on the edit button, which allows the user to edit the order.', 'envira-proofing' ); ?></span>
						</td>
					</tr>
					<tr id="envira-proofing-delete-button-label-box">
						<th scope="row">
							<label for="envira-proofing-delete-button-label"><?php esc_html_e( 'Delete Button Label', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-delete-button-label" type="text" name="<?php echo esc_html( $key ); ?>[proofing_delete_button_label]" value="<?php echo ( esc_html( $proofing_delete_button_label ) ); ?>" /><br />
							<span class="description"><?php esc_html_e( 'The text to display on the delete button, which allows the user to delete the order.', 'envira-proofing' ); ?></span>
						</td>
					</tr>
					<tr id="envira-proofing-submit-button-label-box">
						<th scope="row">
							<label for="envira-proofing-submit-button-label"><?php esc_html_e( 'Submit Button Label', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-submit-button-label" type="text" name="<?php echo esc_html( $key ); ?>[proofing_submit_button_label]" value="<?php echo ( esc_html( $proofing_submit_button_label ) ); ?>" /><br />
							<span class="description"><?php esc_html_e( 'The text to display on the submit button, which allows the user to submit/finalise the order.', 'envira-proofing' ); ?></span>
						</td>
					</tr>
					<tr id="envira-proofing-save-notes-button-label-box">
						<th scope="row">
							<label for="envira-proofing-order-notes-placeholder-text"><?php esc_html_e( 'Order Notes Placeholder Text', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-order-notes-placeholder-text" type="text" name="<?php echo esc_html( $key ); ?>[proofing_notes_placeholder_text]" value="<?php echo ( esc_html( $proofing_notes_placeholder_text ) ); ?>" /><br />
							<span class="description"><?php esc_html_e( 'The placeholder text to display in the Order Notes field.', 'envira-proofing' ); ?></span>
						</td>
					</tr>
					<tr id="envira-proofing-save-notes-button-label-box">
						<th scope="row">
							<label for="envira-proofing-save-notes-button-label"><?php esc_html_e( 'Save Notes Button Label', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-save-notes-button-label" type="text" name="<?php echo esc_html( $key ); ?>[proofing_save_notes_button_label]" value="<?php echo ( esc_html( $proofing_save_notes_button_label ) ); ?>" /><br />
							<span class="description"><?php esc_html_e( 'The text to display on the save button, which is displayed once an order has been submitted.', 'envira-proofing' ); ?></span>
						</td>
					</tr>
					<tr id="envira-proofing-override-global-settings">
						<th scope="row">
							<label for="envira-proofing-override-global-settings"><?php esc_html_e( 'Override Global Settings', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-verride-global-settings" type="checkbox" name="<?php echo esc_html( $key ); ?>[proofing_override_globals]" value="1" <?php checked( envira_get_config( 'proofing_override_globals', $data, envira_proofing_get_global_config_defaults( 'proofing_override_globals' ) ), 1 ); ?> />
							<span class="description"><?php esc_html_e( 'Override global settings and provide specific email settings ONLY for this gallery.', 'envira-proofing' ); ?></span>
						</td>
					</tr>
					<tr id="envira-proofing-submitted-message-box">
						<th scope="row">
							<label for="envira-proofing-submitted-message"><?php esc_html_e( 'Custom Submitted Message to Customer', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<?php
							$message = envira_get_config( 'proofing_submitted_message', $data, envira_proofing_get_global_config_defaults( 'global_proofing_submitted_message' ) );

							wp_editor(
								$message,
								'envira-proofing-submitted-message',
								[
									'media_buttons' => false,
									'wpautop'       => true,
									'tinymce'       => true,
									'textarea_name' => $key . '[proofing_submitted_message]',
								]
							);
							?>
							<p class="description"><?php esc_html_e( 'Custom text (for this gallery only) to display once the user has submitted their order. Leave this empty if you want to use the global email message.', 'envira-proofing' ); ?></p>
						</td>
					</tr>
					<?php
					/*
					<tr id="envira-proofing-email-box">
						<th scope="row">
							<label for="envira-proofing-email-label"><?php esc_html_e( 'Email Admin Order Notifications To', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-email-label" type="text" name="<?php echo esc_html( $key ); ?>[proofing_email]" value="<?php echo esc_html( envira_get_config( 'proofing_email', $data, envira_proofing_get_global_config_defaults( 'global_proofing_email' ) ) ); ?>" /><br />
							<span class="description"><?php esc_html_e( 'Enter an email address to receive notifications when a new order is submitted. Leave blank if you want to use the global admin notification setting.', 'envira-proofing' ); ?></span>
						</td>
					</tr>
					*/
					?>
					<tr id="envira-proofing-email-subject-box">
						<th scope="row">
							<label for="envira-proofing-email-subject"><?php esc_html_e( 'Customer Submit Email Subject', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-email-subject" type="text" name="<?php echo esc_html( $key ); ?>[proofing_email_subject]" value="<?php echo esc_attr( envira_get_config( 'proofing_email_subject', $data, envira_proofing_get_global_config_defaults( 'global_proofing_email_subject' ) ) ); ?>" /><br />
							<span class="description"><?php esc_html_e( 'The email message subject. Leave this empty if you want to use the global submit email subject.', 'envira-proofing' ); ?></span>
						</td>
					</tr>
					<tr id="envira-proofing-email-message-box">
						<th scope="row">
							<label for="envira-proofing-email-message"><?php esc_html_e( 'Customer Submit Email Message', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<?php
							$message = envira_get_config( 'proofing_email_message', $data, envira_proofing_get_global_config_defaults( 'global_proofing_email_message' ) );
							wp_editor(
								$message,
								'envira-proofing-email-message',
								[
									'media_buttons' => false,
									'wpautop'       => true,
									'tinymce'       => true,
									'textarea_name' => $key . '[proofing_email_message]',
								]
							);
							?>
							<p class="description">
								<?php esc_html_e( 'The email message sent once the user has submitted their order. Leave this empty if you want to use the global submit email message.', 'envira-proofing' ); ?><br /><br/>
								<?php esc_html_e( 'Supported Tags:', 'envira-proofing' ); ?><br />
								<strong>{title}: </strong><?php esc_html_e( 'The Gallery Title', 'envira-proofing' ); ?><br />
								<strong>{url}: </strong><?php esc_html_e( 'The Gallery Edit URL', 'envira-proofing' ); ?><br />
								<strong>{order_email}: </strong><?php esc_html_e( 'The customer\'s email address', 'envira-proofing' ); ?>
							</p>
						</td>
					</tr>
				</tbody>
			</table>
		</div>



		<!-- Order -->
		<div id="envira-proofing-orders">
			<p class="envira-proofing-orders-header">
				<?php esc_html_e( 'Orders', 'envira-proofing' ); ?>
				<?php
					// phpcs:ignore WordPress.Security.NonceVerification.Recommended
					$post = ( isset( $_GET['post'] ) ? intval( $_GET['post'] ) : false );

				?>
				<small>
					Orders specific to this gallery submitted by customers. <a href="<?php echo esc_url( admin_url( '/edit.php?post_status=all&post_type=envira_proof_order&&envira_gallery_id=' . $post . '&filter_action=Filter&paged=1' ) ); ?>">Click here</a> to see a more detailed list of this gallery's orders. <a href="<?php echo esc_url( admin_url( '/edit.php?post_type=envira_proof_order' ) ); ?>">Click here</a> to see all site orders.
				</small>
			</p>
			<table id="envira-proofing-order-list" class="form-table">
				<thead>
					<th>Order #</th>
					<th>Customer</th>
					<th>Status</th>
					<th>Submitted</th>
				</thead>
				<tbody>
		<?php
		$orders = new \WP_Query(
			[
				'post_type'      => 'envira_proof_order',
				'post_status'    => [ 'proofing-completed', 'proofing-active', 'proofing-pending' ],
				'posts_per_page' => -1,
				'order_by'       => 'date',
				'meta_query'     => [
					[
						'key'   => 'gallery_id',
						'value' => $gallery_id,
					],
				],
			]
		);
		if ( 0 !== absint( $gallery_id ) && ! empty( $orders ) && ! empty( $orders->posts ) ) {
			foreach ( $orders->posts as $order ) {

				$proofing_name  = get_post_meta( $order->ID, 'proofing_name', true );
				$name           = ( isset( $proofing_name ) ? esc_html( $proofing_name ) : false );
				$proofing_email = get_post_meta( $order->ID, 'proofing_email', true );
				$name_and_email = ( isset( $proofing_email ) && is_email( $proofing_email ) && false !== $name ? $name . '<br/><a href="mailto:' . esc_html( $proofing_email ) . '">' . esc_html( $proofing_email ) . '</a>' : 'Unknown' );
				?>
				<tr>
					<td><a href="<?php echo esc_url( get_edit_post_link( $order->ID ) ); ?>"><?php echo esc_html( $order->post_title ); ?></a></td>
					<td><?php echo wp_kses( $name_and_email, $this->wp_kses_allowed_html ); ?></td>
					<td><?php echo esc_html( ucfirst( str_replace( 'proofing-', '', $order->post_status ) ) ); ?></td>
					<td><?php echo esc_html( human_time_diff( time(), strtotime( $order->post_date ) ) ) . ' ago'; ?>
					</td>
				</tr>
				<?php
			}
		} else {
			echo '<tr><td colspan="4"><p>No orders for this gallery.</p></td></tr>';
		}
		?>

				</tbody>
			</table>

		</div>

		<?php

		$legacy_orders = get_post_meta( $gallery_id, '_envira_proofing_order', true );

		// Filter.
		$legacy_orders = apply_filters( 'envira_proofing_order_get_orders', $legacy_orders, $gallery_id );

		if ( $legacy_orders ) {

			?>
			<!-- Legacy Orders -->
			<div id="envira-legacy-proofing-orders">
				<p class="envira-intro">
					<?php esc_html_e( 'Legacy Orders', 'envira-proofing' ); ?>
					<small>
						<?php esc_html_e( 'Orders that were placed before upgrading to version 2.0 of the Envira Proofing Addon. You can update legacy orders, but please ', 'envira-proofing' ); ?>
						<a href="#"><?php esc_html_e( 'read this documentation first', 'envira-proofing' ); ?></a>.
					</small>
				</p>
				<p>
					<a class="button button-primary" href="<?php echo esc_url_raw( add_query_arg( '_wponce', wp_create_nonce( 'convert_orders' ), admin_url( '/post.php?post=' . $gallery_id . '&action=edit&orders=convert' ) ) ); ?>">Convert Legacy Orders</a>
				</p>
			</div>

			<?php

			/* legacy orders */
			if ( is_array( $legacy_orders ) ) {
				foreach ( $legacy_orders as $email => $order ) {
					// Skip if the order has no images selected.
					if ( ! isset( $order['images'] ) || ( is_array( $order['images'] ) && count( $order['images'] ) ) === 0 ) {
						continue;
					}

					// Display order details.
					?>
					<hr />
					<div class="envira-proofing-order">
						<div class="envira-alert envira-clear">
							<div style="float:left;">
								<?php if ( ! empty( $order['name'] ) ) { ?>
									<p>
										<strong><?php esc_html_e( 'Name: ', 'envira-proofing' ); ?></strong>
										<?php echo wp_kses( esc_html( $order['name'] ), $this->wp_kses_allowed_html ); ?>
									</p>
								<?php } ?>

								<?php if ( ! empty( $order['email'] ) ) { ?>
									<p>
										<strong><?php esc_html_e( 'Email: ', 'envira-proofing' ); ?></strong>
										<?php echo wp_kses( esc_html( $order['email'] ), $this->wp_kses_allowed_html ); ?>
									</p>
								<?php } ?>

								<?php if ( ! empty( $order['notes'] ) ) { ?>
									<p>
										<strong><?php esc_html_e( 'Notes: ', 'envira-proofing' ); ?></strong>
										<?php echo wp_kses( esc_html( $order['notes'] ), $this->wp_kses_allowed_html ); ?>
									</p>
								<?php } ?>

							</div>
						</div>

						<table class="form-table">
							<tbody>
								<?php if ( ! empty( $order['images'] ) ) : ?>
									<?php
									foreach ( $order['images'] as $image_id => $image_order ) {
										?>
										<tr>
											<th scope="row">
												<?php
												$thumbnail = wp_get_attachment_image_src( $image_id, 'thumbnail' );
												?>
												<img src="<?php echo esc_url( $thumbnail[0] ); ?>"  /><br />
												<?php echo esc_html( basename( $thumbnail[0] ) ); ?>
											</th>
											<td>
												<?php
												foreach ( $image_order as $size => $quantity ) {
													?>
													<strong><?php echo esc_html( $size ); ?></strong>: <?php echo esc_attr( $quantity ); ?><br />
													<?php
												}
												?>
											</td>
										</tr>
										<?php
									}
									?>
								<?php endif; ?>
							</tbody>
						</table>
					</div>
					<?php
				}
			}
		}
	}



	/**
	 * Helper method for retrieving lightbox transition effects.
	 *
	 * @since 1.0.0
	 *
	 * @return array Array of transition effect data.
	 */
	public function get_proofing_email_box_positions() {

		return envira_proofing_get_proofing_email_box_positions();
	}

	/**
	 * Saves the addon's settings for Galleries.
	 *
	 * @since 1.0.0
	 *
	 * @param array $settings  Array of settings to be saved.
	 * @param int   $post_id     The current post ID.
	 * @return array $settings Amended array of settings to be saved.
	 */
	public function save( $settings, $post_id ) {

		// Bail out if we fail a security check.
		if ( ! isset( $_POST['envira-gallery'] ) || ! wp_verify_nonce( sanitize_key( $_POST['envira-gallery'] ), 'envira-gallery' ) || ! isset( $_POST['_envira_gallery'] ) ) {
			return;
		}

		// Lightbox.
		$settings['config']['proofing_lightbox'] = ( isset( $_POST['_envira_gallery']['proofing_lightbox'] ) ? 1 : 0 );

		// Proofing Settings.
		$settings['config']['proofing']        = ( isset( $_POST['_envira_gallery']['proofing'] ) ? 1 : 0 );
		$settings['config']['proofing_edit']   = ( isset( $_POST['_envira_gallery']['proofing_edit'] ) ? 1 : 0 );
		$settings['config']['proofing_delete'] = ( isset( $_POST['_envira_gallery']['proofing_delete'] ) ? 1 : 0 );

		// Envira Proofing Integration Settings.
		$settings['config']['proofing_multiple_enabled']        = ( isset( $_POST['_envira_gallery']['proofing_multiple_enabled'] ) ? 1 : 0 );
		$settings['config']['proofing_name_enabled']            = ( isset( $_POST['_envira_gallery']['proofing_name_enabled'] ) ? 1 : 0 );
		$settings['config']['proofing_hide_gallery']            = ( isset( $_POST['_envira_gallery']['proofing_hide_gallery'] ) ? 1 : 0 );
		$settings['config']['proofing_quantity_enabled']        = ( isset( $_POST['_envira_gallery']['proofing_quantity_enabled'] ) ? 1 : 0 );
		$settings['config']['proofing_size_enabled']            = ( isset( $_POST['_envira_gallery']['proofing_size_enabled'] ) ? 1 : 0 );
		$settings['config']['proofing_add_to_order_label']      = isset( $_POST['_envira_gallery']['proofing_add_to_order_label'] ) ? esc_attr( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_add_to_order_label'] ) ) ) : false;
		$settings['config']['proofing_save_button_label']       = isset( $_POST['_envira_gallery']['proofing_save_button_label'] ) ? esc_attr( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_save_button_label'] ) ) ) : false;
		$settings['config']['proofing_edit_button_label']       = isset( $_POST['_envira_gallery']['proofing_edit_button_label'] ) ? esc_attr( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_edit_button_label'] ) ) ) : false;
		$settings['config']['proofing_delete_button_label']     = isset( $_POST['_envira_gallery']['proofing_delete_button_label'] ) ? esc_attr( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_delete_button_label'] ) ) ) : false;
		$settings['config']['proofing_notes_placeholder_text']  = isset( $_POST['_envira_gallery']['proofing_notes_placeholder_text'] ) ? esc_attr( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_notes_placeholder_text'] ) ) ) : false;
		$settings['config']['proofing_save_notes_button_label'] = isset( $_POST['_envira_gallery']['proofing_save_notes_button_label'] ) ? esc_attr( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_save_notes_button_label'] ) ) ) : false;
		$settings['config']['proofing_submit_button_label']     = isset( $_POST['_envira_gallery']['proofing_submit_button_label'] ) ? esc_attr( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_submit_button_label'] ) ) ) : false;

		// Overrides.
		$settings['config']['proofing_override_globals']  = ( isset( $_POST['_envira_gallery']['proofing_override_globals'] ) ? 1 : 0 );
		$settings['config']['proofing_submitted_message'] = isset( $_POST['_envira_gallery']['proofing_submitted_message'] ) ? trim( sanitize_textarea_field( wp_unslash( $_POST['_envira_gallery']['proofing_submitted_message'] ) ) ) : false;
		$settings['config']['proofing_email']             = isset( $_POST['_envira_gallery']['proofing_email'] ) ? sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_email'] ) ) : false;
		if ( isset( $_POST['_envira_gallery']['proofing_email_box_position'] ) ) {
			$settings['config']['proofing_email_box_position'] = isset( $_POST['_envira_gallery']['proofing_email_box_position'] ) ? sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_email_box_position'] ) ) : false;
		}
		$settings['config']['proofing_email_subject'] = isset( $_POST['_envira_gallery']['proofing_email_subject'] ) ? sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_email_subject'] ) ) : false;
		$settings['config']['proofing_email_message'] = isset( $_POST['_envira_gallery']['proofing_email_message'] ) ? trim( sanitize_textarea_field( wp_unslash( $_POST['_envira_gallery']['proofing_email_message'] ) ) ) : false;

		// Sizes.
		$sizes = [];
		if ( isset( $_POST['_envira_gallery']['proofing_sizes'] ) && ! empty( $_POST['_envira_gallery']['proofing_sizes']['name'] ) ) {
			// Build array of sizes.
			$names_sanitized = array_map( 'sanitize_text_field', wp_unslash( $_POST['_envira_gallery']['proofing_sizes']['name'] ) );
			foreach ( $names_sanitized as $index => $name ) {
				// Skip empty sizes.
				if ( empty( $_POST['_envira_gallery']['proofing_sizes']['name'][ $index ] ) ) {
					continue;
				}

				// Generate slug.
				$slug = str_replace( ' ', '_', esc_attr( strtolower( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_sizes']['name'][ $index ] ) ) ) ) );

				// Add to sizes array.
				$sizes[ $slug ] = [
					'slug'        => $slug,
					'name'        => isset( $_POST['_envira_gallery']['proofing_sizes']['name'][ $index ] ) ? esc_attr( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_sizes']['name'][ $index ] ) ) ) : false,
					'orientation' => isset( $_POST['_envira_gallery']['proofing_sizes']['orientation'][ $index ] ) ? esc_attr( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_sizes']['orientation'][ $index ] ) ) ) : false,
				];
			}
		}

		$settings['config']['proofing_sizes'] = $sizes;

		return $settings;
	}

	/**
	 * Upgrade legacy orders.
	 *
	 * @since 2.0.0
	 */
	public function upgrade_legacy_orders() {

		// Bail out if we fail a security check.
		if ( ! isset( $_GET['orders'], $_GET['_wponce'] ) || ! wp_verify_nonce( sanitize_key( $_GET['_wponce'] ), 'convert_orders' ) ) {
			return;
		}

		$post_id = isset( $_GET['post'] ) ? intval( $_GET['post'] ) : false;

		if ( false === $post_id ) {
			return;
		}

		$legacy_orders = get_post_meta( $post_id, '_envira_proofing_order', true );

		// Filter.
		$legacy_orders = apply_filters( 'envira_proofing_order_get_orders', $legacy_orders, $post_id );

		if ( empty( $legacy_orders ) ) {
			return;
		}

		$converted_count = 0;

		foreach ( $legacy_orders as $legacy_order ) {

			$name   = ! empty( $legacy_order['name'] ) ? esc_html( $legacy_order['name'] ) : false;
			$email  = ! empty( $legacy_order['email'] ) ? esc_html( $legacy_order['email'] ) : false;
			$notes  = ! empty( $legacy_order['notes'] ) ? esc_html( $legacy_order['notes'] ) : false;
			$images = ! empty( $legacy_order['images'] ) ? ( $legacy_order['images'] ) : [];

			$proofing_order_id = wp_insert_post(
				[
					'post_title'   => 'order from gallery ' . $post_id,
					'post_type'    => 'envira_proof_order',
					'post_status'  => 'proofing-pending',
					'post_content' => $notes,
				]
			);

			if ( $proofing_order_id > 0 ) {

				// add custom meta.
				add_post_meta( $proofing_order_id, 'proofing_name', $name );
				add_post_meta( $proofing_order_id, 'proofing_email', $email );
				add_post_meta( $proofing_order_id, 'proofing_notes', $notes );
				add_post_meta( $proofing_order_id, 'proofing_images', $images );
				add_post_meta( $proofing_order_id, 'gallery_id', $post_id );

				// update the title to reflect order number?
				wp_update_post(
					[
						'ID'         => $proofing_order_id,
						'post_title' => 'Order #' . $proofing_order_id,
					]
				);

				++$converted_count;

			}
		}

		if ( $converted_count > 0 ) :

			// Finally rename the metadata for older oreders for now, don't delete.
			update_post_meta( $post_id, '_envira_proofing_order_converted', $legacy_orders );
			delete_post_meta( $post_id, '_envira_proofing_order' );

			$notices = new Notices();

			$message = sprintf(
				/* translators: %s: URL */
				__( '<strong>Envira Proofing</strong>: Legacy orders converted.', 'envira-proofing' )
			);

			$notices->display_inline_notice( 'proofing-converted-orders', false, $message, 'warning', false, false, true, false );

		endif;

		// Do a redirect.
		$redirect_url = admin_url( '/post.php?post=' . $post_id . '&action=edit&converted=' . $converted_count . '#!envira-tab-proofing' );
		wp_safe_redirect( $redirect_url );

		exit;
	}


	/**
	 * Returns the singleton instance of the class.
	 *
	 * @since 1.0.0
	 *
	 * @return object The Envira_Proofing_Metaboxes object.
	 */
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Envira_Proofing_Metaboxes ) ) {
			self::$instance = new Envira_Proofing_Metaboxes();
		}

		return self::$instance;
	}
}

