<?php
/**
 * Settings class.
 *
 * @since 1.7.0
 *
 * @package Envira_Gallery
 * @author  Envira Gallery Team <support@enviragallery.com>
 */

namespace Envira\Proofing\Admin;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use Envira\Admin\Metaboxes;

/**
 * Settings class.
 *
 * @since 1.7.0
 *
 * @package Envira_Gallery
 * @author  Envira Gallery Team <support@enviragallery.com>
 */
class Settings {

	/**
	 * Holds the submenu pagehook.
	 *
	 * @since 1.7.0
	 *
	 * @var string
	 */
	public $hook;

	/**
	 * Holds the key.
	 *
	 * @since 2.0.0
	 *
	 * @var string
	 */
	public $key = '_envira_gallery_proofing';

	/**
	 * Primary class constructor.
	 *
	 * @since 1.7.0
	 */
	public function __construct() {

		// Add custom settings submenu.
		add_action( 'admin_menu', [ &$this, 'admin_menu' ], 11 );

		// Add callbacks for settings tabs.
		add_action( 'envira_gallery_proofing_tab_settings_general', [ $this, 'settings_general_tab' ] );
		add_action( 'envira_gallery_proofing_tab_settings_customer_emails', [ $this, 'settings_customer_emails_tab' ] );
		add_action( 'envira_gallery_proofing_tab_settings_admin_emails', [ $this, 'settings_admin_emails_tab' ] );

		// Add the settings menu item to the Plugins table.
		add_filter( 'plugin_action_links_' . plugin_basename( plugin_dir_path( dirname( __DIR__ ) ) . 'envira-proofing.php' ), [ $this, 'settings_link' ] );
	}

	/**
	 * Register the Settings submenu item for Envira.
	 *
	 * @since 1.7.0
	 */
	public function admin_menu() {

		$label = apply_filters( 'envira_whitelabel_name', 'Envira' );

		// Register the submenu.
		$this->hook = add_submenu_page(
			'edit.php?post_type=envira_proof_order',
			$label . __( ' Proofing Settings', 'envira-gallery' ),
			__( 'Settings', 'envira-gallery' ),
			apply_filters( 'envira_gallery_menu_cap', 'manage_options' ),
			ENVIRA_SLUG . '-settings-proofing',
			[ &$this, 'settings_page' ]
		);

		// If successful, load admin assets only on that page and check for addons refresh.
		if ( ! $this->hook ) {
			return;
		}

		// Add all of our settings hooks.
		add_action( 'load-' . $this->hook, [ $this, 'update_general_settings' ] );
		add_action( 'load-' . $this->hook, [ $this, 'update_customer_email_settings' ] );
		add_action( 'load-' . $this->hook, [ $this, 'update_admin_email_settings' ] );
		add_action( 'load-' . $this->hook, [ $this, 'enqueue_admin_settings_styles' ] );

		// Add admin Scripts and Styles.
		add_action( 'admin_enqueue_scripts', [ $this, 'enqueue_admin_styles' ] );
		add_action( 'admin_enqueue_scripts', [ $this, 'enqueue_admin_scripts' ] );
	}


	/**
	 * Saves General Settings
	 *
	 * @since 1.7.0
	 *
	 * @return null
	 */
	public function update_general_settings() {

		// Check if user pressed the 'Update' button and nonce is valid.
		if ( ! isset( $_POST['envira-gallery-proofing-settings-general-submit'] ) ) {
			return;
		}

		if ( isset( $_POST['envira-gallery-proofing-settings-general-nonce'] ) && ! wp_verify_nonce( sanitize_key( $_POST['envira-gallery-proofing-settings-general-nonce'] ), 'envira-gallery-proofing-settings-general-nonce' ) ) {
			return;
		}

		$global_proofing_saved_message     = isset( $_POST[ $this->key ]['global_proofing_saved_message'] ) ? sanitize_text_field( wp_unslash( $_POST[ $this->key ]['global_proofing_saved_message'] ) ) : '';
		$global_proofing_submitted_message = isset( $_POST[ $this->key ]['global_proofing_submitted_message'] ) ? sanitize_text_field( wp_unslash( $_POST[ $this->key ]['global_proofing_submitted_message'] ) ) : '';
		$proofing_email_mailing_address    = isset( $_POST[ $this->key ]['proofing_email_mailing_address'] ) ? ( sanitize_text_field( wp_unslash( $_POST[ $this->key ]['proofing_email_mailing_address'] ) ) ) : '';
		$proofing_email_logo               = isset( $_POST[ $this->key ]['proofing_email_logo'] ) ? esc_url( sanitize_text_field( wp_unslash( $_POST[ $this->key ]['proofing_email_logo'] ) ) ) : '';
		$warning_message_disabled          = isset( $_POST[ $this->key ]['smtp_warning_message'] ) ? intval( $_POST[ $this->key ]['smtp_warning_message'] ) : 0;

		// Update settings.
		envira_update_setting( 'global_proofing_saved_message', $global_proofing_saved_message );
		envira_update_setting( 'global_proofing_submitted_message', $global_proofing_submitted_message );
		envira_update_setting( 'proofing_email_mailing_address', $proofing_email_mailing_address );
		envira_update_setting( 'warning_message_disabled', $warning_message_disabled );
		envira_update_setting( 'proofing_email_logo', $proofing_email_logo );

		// Output an admin notice so the user knows what happened.
		add_action( 'envira_gallery_proofing_settings_general_tab_notice', [ $this, 'updated_settings' ] );
	}

	/**
	 * Saves images Settings:
	 * - Add New Images
	 * - Delete Images on Gallery Deletion
	 *
	 * @since 1.7.0
	 *
	 * @return null Return early if not fixing the broken migration
	 */
	public function update_customer_email_settings() {

		// Check if user pressed the 'Update' button and nonce is valid.
		if ( ! isset( $_POST['envira-gallery-proofing-settings-customer-submit'] ) ) {
			return;
		}
		if ( isset( $_POST['envira-gallery-proofing-settings-customer-nonce'] ) && ! wp_verify_nonce( sanitize_key( $_POST['envira-gallery-proofing-settings-customer-nonce'] ), 'envira-gallery-proofing-settings-customer-nonce' ) ) {
			return;
		}

		$global_proofing_customer_email_subject_submit = isset( $_POST[ $this->key ]['global_proofing_customer_email_subject_submit'] ) ? sanitize_text_field( wp_unslash( $_POST[ $this->key ]['global_proofing_customer_email_subject_submit'] ) ) : '';
		$global_proofing_customer_email_message_submit = isset( $_POST[ $this->key ]['global_proofing_customer_email_message_submit'] ) ? sanitize_text_field( wp_unslash( $_POST[ $this->key ]['global_proofing_customer_email_message_submit'] ) ) : '';

		$global_proofing_customer_email_subject_save = isset( $_POST[ $this->key ]['global_proofing_customer_email_subject_save'] ) ? sanitize_text_field( wp_unslash( $_POST[ $this->key ]['global_proofing_customer_email_subject_save'] ) ) : '';
		$global_proofing_customer_email_message_save = isset( $_POST[ $this->key ]['global_proofing_customer_email_message_save'] ) ? sanitize_text_field( wp_unslash( $_POST[ $this->key ]['global_proofing_customer_email_message_save'] ) ) : '';

		// Update settings.
		envira_update_setting( 'global_proofing_customer_email_subject_save', $global_proofing_customer_email_subject_save );
		envira_update_setting( 'global_proofing_customer_email_message_save', $global_proofing_customer_email_message_save );
		envira_update_setting( 'global_proofing_customer_email_subject_submit', $global_proofing_customer_email_subject_submit );
		envira_update_setting( 'global_proofing_customer_email_message_submit', $global_proofing_customer_email_message_submit );

		// Output an admin notice so the user knows what happened.
		add_action( 'envira_gallery_proofing_settings_customer_tab_notice', [ $this, 'updated_settings' ] );
	}

	/**
	 * Saves images Settings:
	 * - Add New Images
	 * - Delete Images on Gallery Deletion
	 *
	 * @since 1.7.0
	 *
	 * @return null Return early if not fixing the broken migration
	 */
	public function update_admin_email_settings() {

		// Check if user pressed the 'Update' button and nonce is valid.
		if ( ! isset( $_POST['envira-gallery-proofing-settings-admin-submit'] ) ) {
			return;
		}
		if ( isset( $_POST['envira-gallery-proofing-settings-admin-nonce'] ) && ! wp_verify_nonce( sanitize_key( $_POST['envira-gallery-proofing-settings-admin-nonce'] ), 'envira-gallery-proofing-settings-admin-nonce' ) ) {
			return;
		}

		$global_proofing_admin_email_notification = isset( $_POST[ $this->key ]['global_proofing_admin_email_notification'] ) ? 1 : 0;
		$global_proofing_admin_email              = isset( $_POST[ $this->key ]['global_proofing_admin_email'] ) ? sanitize_text_field( wp_unslash( $_POST[ $this->key ]['global_proofing_admin_email'] ) ) : '';
		$global_proofing_admin_email_subject      = isset( $_POST[ $this->key ]['global_proofing_admin_email_subject'] ) ? sanitize_text_field( wp_unslash( $_POST[ $this->key ]['global_proofing_admin_email_subject'] ) ) : '';
		$global_proofing_email_message            = isset( $_POST[ $this->key ]['global_proofing_email_message'] ) ? sanitize_text_field( wp_unslash( $_POST[ $this->key ]['global_proofing_email_message'] ) ) : '';

		// Update settings.
		envira_update_setting( 'global_proofing_admin_email_notification', $global_proofing_admin_email_notification );
		envira_update_setting( 'global_proofing_admin_email', $global_proofing_admin_email );
		envira_update_setting( 'global_proofing_admin_email_subject', $global_proofing_admin_email_subject );
		envira_update_setting( 'global_proofing_email_message', $global_proofing_email_message );

		// Output an admin notice so the user knows what happened.
		add_action( 'envira_gallery_proofing_settings_admin_tab_notice', [ $this, 'updated_settings' ] );
	}

	/**
	 * Outputs a WordPress style notification to tell the user settings were saved
	 *
	 * @since 1.7.0
	 */
	public function updated_settings() {

		?>
		<div class="notice updated below-h2">
			<p><strong><?php esc_html_e( 'Settings saved successfully.', 'envira-gallery' ); ?></strong></p>
		</div>
		<?php
	}


	/**
	 * Register and enqueue settings page specific CSS.
	 *
	 * @since 1.7.0
	 */
	public function enqueue_admin_settings_styles() {

		wp_register_style( ENVIRA_SLUG . '-settings-style', plugins_url( 'assets/css/settings.css', ENVIRA_FILE ), [], ENVIRA_VERSION );
		wp_enqueue_style( ENVIRA_SLUG . '-settings-style' );

		wp_register_style( ENVIRA_PROOFING_SLUG . '-metabox-style', plugins_url( 'assets/css/proofing-admin.css', ENVIRA_PROOFING_FILE ), [], ENVIRA_PROOFING_VERSION );
		wp_enqueue_style( ENVIRA_PROOFING_SLUG . '-metabox-style' );

		// Run a hook to load in custom styles.
		do_action( 'envira_gallery_proofing_settings_styles' );
	}


	/**
	 * Register and enqueue settings page specific CSS.
	 *
	 * @since 1.7.0
	 */
	public function enqueue_admin_styles() {

		// Run a hook to load in custom styles.
		do_action( 'envira_gallery_proofing_admin_styles' );
	}

	/**
	 * Register and enqueue settings page specific JS.
	 *
	 * @since 1.7.0
	 */
	public function enqueue_admin_scripts() {

		// Tabs.
		wp_register_script( ENVIRA_SLUG . '-tabs-script', plugins_url( 'assets/js/tabs.js', ENVIRA_FILE ), [ 'jquery' ], ENVIRA_VERSION, true );
		wp_enqueue_script( ENVIRA_SLUG . '-tabs-script' );

		// Settings.
		wp_register_script( ENVIRA_SLUG . '-settings-script', plugins_url( 'assets/js/settings.js', ENVIRA_FILE ), [ 'jquery', 'jquery-ui-tabs' ], ENVIRA_VERSION, true );
		wp_enqueue_script( ENVIRA_SLUG . '-settings-script' );
		wp_localize_script(
			ENVIRA_SLUG . '-settings-script',
			'envira_gallery_settings',
			[
				'active'           => __( 'Status: Active', 'envira-gallery' ),
				'activate'         => __( 'Activate', 'envira-gallery' ),
				'activate_nonce'   => wp_create_nonce( 'envira-gallery-activate' ),
				'activating'       => __( 'Activating...', 'envira-gallery' ),
				'ajax'             => admin_url( 'admin-ajax.php' ),
				'deactivate'       => __( 'Deactivate', 'envira-gallery' ),
				'deactivate_nonce' => wp_create_nonce( 'envira-gallery-deactivate' ),
				'deactivating'     => __( 'Deactivating...', 'envira-gallery' ),
				'inactive'         => __( 'Status: Inactive', 'envira-gallery' ),
				'install'          => __( 'Install', 'envira-gallery' ),
				'install_nonce'    => wp_create_nonce( 'envira-gallery-install' ),
				'installing'       => __( 'Installing...', 'envira-gallery' ),
				'proceed'          => __( 'Proceed', 'envira-gallery' ),
			]
		);

		// Run a hook to load in custom scripts.
		do_action( 'envira_gallery_settings_scripts' );
	}

	/**
	 * Callback to output the Envira settings page.
	 *
	 * @since 1.7.0
	 */
	public function settings_page() {

		do_action( 'envira_head' );

		?>

		<!-- Tabs -->
		<h2 id="envira-tabs-nav" class="envira-tabs-nav" data-container="#envira-gallery-proofing-settings" data-update-hashbang="1">
			<?php
			$i = 0;
			foreach ( (array) $this->get_settings_proofing_tab_nav() as $id => $title ) {
				$class = ( 0 === $i ? 'envira-active' : '' );
				?>
				<a class="nav-tab <?php echo esc_html( $class ); ?>" href="#envira-tab-<?php echo esc_attr( $id ); ?>" title="<?php echo esc_attr( $title ); ?>"><?php echo esc_html( $title ); ?></a>
				<?php
				++$i;
			}
			?>
		</h2>

		<!-- Tab Panels -->
		<div id="envira-gallery-proofing-settings" class="wrap">
			<h1 class="envira-hideme"></h1>
			<div class="envira-gallery envira-clear">
				<div id="envira-tabs" class="envira-clear" data-navigation="#envira-tabs-nav">
					<?php
					$i = 0;
					foreach ( (array) $this->get_settings_proofing_tab_nav() as $id => $title ) {
						$class = ( 0 === $i ? 'envira-active' : '' );
						?>
						<div id="envira-tab-<?php echo esc_attr( $id ); ?>" class="envira-tab envira-clear <?php echo esc_attr( $class ); ?>">
							<?php do_action( 'envira_gallery_proofing_tab_settings_' . $id ); ?>
						</div>
						<?php
						++$i;
					}
					?>
				</div>
			</div>
		</div>

		<?php
	}

	/**
	 * Callback for getting all of the settings tabs for Envira.
	 *
	 * @since 1.7.0
	 *
	 * @return array Array of tab information.
	 */
	public function get_settings_proofing_tab_nav() {

		$tabs = [
			'general'         => __( 'General', 'envira-gallery' ), // This tab is required. DO NOT REMOVE VIA FILTERING.
			'customer_emails' => __( 'Customer Emails', 'envira-gallery' ),
			'admin_emails'    => __( 'Admin Emails', 'envira-gallery' ),

		];
		$tabs = apply_filters( 'envira_gallery_proofing_settings_tab_nav', $tabs );

		// Make sure debug is always last
		// $tabs['debug'] = __( 'System Info', 'envira-gallery' );.
		return $tabs;
	}

	/**
	 * Callback for displaying the UI for general settings tab.
	 *
	 * @since 1.7.0
	 */
	public function settings_general_tab() {

		// Setup instance and meta key.
		$envira_metaboxes = new Metaboxes();

		// Get settings.
		$proofing_submitted_message     = envira_proofing_get_setting( 'global_proofing_submitted_message' );
		$proofing_saved_message         = envira_proofing_get_setting( 'global_proofing_saved_message' );
		$proofing_email                 = envira_proofing_get_setting( 'global_proofing_email' );
		$proofing_email_subject         = envira_proofing_get_setting( 'proofing_email_subject' );
		$proofing_email_message         = envira_proofing_get_setting( 'proofing_email_message' );
		$warning_message_disabled       = envira_proofing_get_setting( 'warning_message_disabled' );
		$proofing_email_mailing_address = envira_proofing_get_setting( 'proofing_email_mailing_address' );
		$proofing_email_logo            = envira_proofing_get_setting( 'proofing_email_logo' );

		?>
		<div id="envira-settings-general">
			<?php
			// Output any notices now.
			do_action( 'envira_gallery_proofing_settings_general_tab_notice' );
			?>

			<h2>General</h2>
			<hr />

			<!-- Settings Form -->
			<form id="envira-proofing-global-settings" method="post">
				<table class="form-table">
					<tbody>
					<tr id="envira-proofing-saved-message-box">
						<th scope="row">
							<label for="envira-proofing-saved-message"><?php esc_html_e( 'Saved Message', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<?php
							if ( empty( $proofing_saved_message ) ) {
								$message = envira_proofing_get_global_config_defaults( 'global_proofing_saved_message' );
							}
							wp_editor(
								$proofing_saved_message,
								'envira-proofing-saved-message',
								[
									'media_buttons' => false,
									'wpautop'       => true,
									'textarea_rows' => 5,
									'tinymce'       => true,
									'textarea_name' => $this->key . '[global_proofing_saved_message]',
								]
							);
							?>
							<p class="description"><?php esc_html_e( 'The text to display once the user has saved an order for any gallery.', 'envira-proofing' ); ?></p>
						</td>
					</tr>
					<tr id="envira-proofing-submitted-message-box">
						<th scope="row">
							<label for="envira-proofing-submitted-message"><?php esc_html_e( 'Submitted Message', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<?php
							if ( empty( $proofing_submitted_message ) ) {
								$message = envira_proofing_get_global_config_defaults( 'global_proofing_submitted_message' );
							}
							wp_editor(
								$proofing_submitted_message,
								'envira-proofing-submitted-message',
								[
									'media_buttons' => false,
									'wpautop'       => true,
									'textarea_rows' => 5,
									'tinymce'       => true,
									'textarea_name' => $this->key . '[global_proofing_submitted_message]',
								]
							);
							?>
							<p class="description"><?php esc_html_e( 'The text to display once the user has submitted their order for any gallery.', 'envira-proofing' ); ?></p>
						</td>
					</tr>
					<tr id="envira-proofing-email-logo">
						<th scope="row">
							<label for="envira-proofing-email-logo"><?php esc_html_e( 'Logo URL', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-email-subject-submit" type="text" name="<?php echo esc_html( $this->key ); ?>[proofing_email_logo]" value="<?php echo esc_attr( $proofing_email_logo ); ?>" /><br />
							<span class="description"><?php esc_html_e( 'Logo URL to be used on emails, which replaces the {envira_proofing_order_logo} text placeholder in the email templates. Leave blank for no logo.', 'envira-proofing' ); ?><br />
							</span>
						</td>
					</tr>
					<tr id="envira-proofing-email-mailing-address">
						<th scope="row">
							<label for="envira-proofing-email-mailing-address"><?php esc_html_e( 'Mailing Address', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<?php
							if ( empty( $proofing_submitted_message ) ) {
								$message = envira_proofing_get_global_config_defaults( 'proofing_email_mailing_address' );
							}
							wp_editor(
								$proofing_email_mailing_address,
								'envira-proofing-submitted-message',
								[
									'media_buttons' => false,
									'wpautop'       => true,
									'textarea_rows' => 5,
									'tinymce'       => false,
									'textarea_name' => $this->key . '[proofing_email_mailing_address]',
								]
							);
							?>
							<p class="description"><?php esc_html_e( 'The text to display by default when a mailing address is included in your emails, replaces the {envira_proofing_order_from_address} text placeholder in the email templates.', 'envira-proofing' ); ?></p>
						</td>
					</tr>
					<tr id="envira-proofing-smtp-warning-message">
						<th scope="row">
							<label for="envira-proofing-smtp-warning-message"><?php esc_html_e( 'SMTP Warning Message', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input name="<?php echo esc_html( $this->key ); ?>[smtp_warning_message]" label="envira-proofing-smtp-warning-message" type="checkbox" value="1" <?php checked( true, $warning_message_disabled ); ?> ><label><?php esc_html_e( 'Disable SMTP Warning Message', 'envira-gallery' ); ?></label>
							<p class="description"><?php esc_html_e( 'Recommended when you are using SMTP but not a plugin Envira yet recognizes.', 'envira-gallery' ); ?></p>
						</td>
					</tr>
					<?php do_action( 'envira_gallery_proofing_settings_general_box' ); ?>
					</tbody>
				</table>

				<?php wp_nonce_field( 'envira-gallery-proofing-settings-general-nonce', 'envira-gallery-proofing-settings-general-nonce' ); ?>
				<?php submit_button( __( 'Save Settings', 'envira-gallery' ), 'primary', 'envira-gallery-proofing-settings-general-submit', false ); ?>
			</form>
		</div>
		<?php
	}

	/**
	 * Callback for displaying the UI for general settings tab.
	 *
	 * @since 1.7.0
	 */
	public function settings_customer_emails_tab() {

		// Setup instance and meta key.
		$envira_metaboxes = new Metaboxes();

		// Get settings.
		$global_proofing_customer_email_subject_submit = ( ! empty( envira_proofing_get_setting( 'global_proofing_customer_email_subject_submit' ) ) ) ? envira_proofing_get_setting( 'global_proofing_customer_email_subject_submit' ) : envira_proofing_get_global_config_defaults( 'global_proofing_customer_email_subject_submit' );

		$global_proofing_customer_email_message_submit = ( ! empty( envira_proofing_get_setting( 'global_proofing_customer_email_message_submit' ) ) ) ? envira_proofing_get_setting( 'global_proofing_customer_email_message_submit' ) : envira_proofing_get_global_config_defaults( 'global_proofing_customer_email_message_submit' );

		$global_proofing_customer_email_subject_save = ( ! empty( envira_proofing_get_setting( 'global_proofing_customer_email_subject_save' ) ) ) ? envira_proofing_get_setting( 'global_proofing_customer_email_subject_save' ) : envira_proofing_get_global_config_defaults( 'global_proofing_customer_email_subject_save' );

		$global_proofing_customer_email_message_save = ( ! empty( envira_proofing_get_setting( 'global_proofing_customer_email_message_save' ) ) ) ? envira_proofing_get_setting( 'global_proofing_customer_email_message_save' ) : envira_proofing_get_global_config_defaults( 'global_proofing_customer_email_message_save' );

		?>
		<div id="envira-settings-customer-emails">
			<?php
			// Output any notices now.
			do_action( 'envira_gallery_proofing_settings_general_tab_notice' );
			?>

			<h2>Customer Emails</h2>
			<hr />

			<!-- Settings Form -->
			<form id="envira-proofing-customer-emails-settings" method="post">
				<table class="form-table">
					<tbody>


					<tr id="envira-proofing-email-subject-box">
						<th scope="row">
							<label for="envira-proofing-email-subject-save"><?php esc_html_e( 'Email Subject (Saved)', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-email-subject-save" type="text" name="<?php echo esc_html( $this->key ); ?>[global_proofing_customer_email_subject_save]" value="<?php echo esc_attr( $global_proofing_customer_email_subject_save ); ?>" /><br />
							<span class="description"><?php esc_html_e( 'The email message subject.', 'envira-proofing' ); ?><br />
							<?php esc_html_e( 'Supported Tags:', 'envira-proofing' ); ?><br />
							<strong>{title}: </strong><?php esc_html_e( 'The Gallery Title', 'envira-proofing' ); ?><br />
							</span>
						</td>
					</tr>
					<tr id="envira-proofing-email-message-box">
						<th scope="row">
							<label for="envira-proofing-email-message"><?php esc_html_e( 'Email Message (Saved)', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<?php
							if ( empty( $global_proofing_customer_email_message_save ) ) {
								$global_proofing_customer_email_message_save = envira_proofing_get_global_config_defaults( 'global_proofing_customer_email_message_save' );
							}
							wp_editor(
								$global_proofing_customer_email_message_save,
								'envira-proofing-email-message-save',
								[
									'media_buttons' => false,
									'wpautop'       => true,
									'tinymce'       => true,
									'textarea_name' => $this->key . '[global_proofing_customer_email_message_save]',
								]
							);
							?>
							<p class="description">
								<?php esc_html_e( 'The email message sent to the customer once they have submitted their order.', 'envira-proofing' ); ?><br />
								<?php esc_html_e( 'Supported Tags:', 'envira-proofing' ); ?><br />
								<strong>{title}: </strong><?php esc_html_e( 'The Gallery Title', 'envira-proofing' ); ?><br />
								<strong>{url}: </strong><?php esc_html_e( 'The Gallery Edit URL', 'envira-proofing' ); ?><br />
								<strong>{order_email}:</strong><?php esc_html_e( 'The customer\'s email address', 'envira-proofing' ); ?>
							</p>
						</td>
					</tr>



					<tr id="envira-proofing-email-subject-box">
						<th scope="row">
							<label for="envira-proofing-email-subject-submit"><?php esc_html_e( 'Email Subject (Submit)', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-email-subject-submit" type="text" name="<?php echo esc_html( $this->key ); ?>[global_proofing_customer_email_subject_submit]" value="<?php echo esc_attr( $global_proofing_customer_email_subject_submit ); ?>" /><br />
							<span class="description"><?php esc_html_e( 'The email message subject.', 'envira-proofing' ); ?><br />
							<?php esc_html_e( 'Supported Tags:', 'envira-proofing' ); ?><br />
							<strong>{title}: </strong><?php esc_html_e( 'The Gallery Title', 'envira-proofing' ); ?><br />
							</span>
						</td>
					</tr>
					<tr id="envira-proofing-email-message-box">
						<th scope="row">
							<label for="envira-proofing-email-message"><?php esc_html_e( 'Email Message (Submit)', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<?php
							if ( empty( $global_proofing_customer_email_message_submit ) ) {
								$global_proofing_customer_email_message_submit = envira_proofing_get_global_config_defaults( 'global_proofing_customer_email_message_submit' );
							}
							wp_editor(
								$global_proofing_customer_email_message_submit,
								'envira-proofing-email-message-submit',
								[
									'media_buttons' => false,
									'wpautop'       => true,
									'tinymce'       => true,
									'textarea_name' => $this->key . '[global_proofing_customer_email_message_submit]',
								]
							);
							?>
							<p class="description">
								<?php esc_html_e( 'The email message sent to the customer once they have submitted their order.', 'envira-proofing' ); ?><br />
								<?php esc_html_e( 'Supported Tags:', 'envira-proofing' ); ?><br />
								<strong>{title}: </strong><?php esc_html_e( 'The Gallery Title', 'envira-proofing' ); ?><br />
								<strong>{url}: </strong><?php esc_html_e( 'The Gallery Edit URL', 'envira-proofing' ); ?><br />
								<strong>{order_email}:</strong><?php esc_html_e( 'The customer\'s email address', 'envira-proofing' ); ?>
							</p>
						</td>
					</tr>

						<?php do_action( 'envira_gallery_proofing_settings_general_box' ); ?>
					</tbody>
				</table>

				<?php wp_nonce_field( 'envira-gallery-proofing-settings-nonce', 'envira-gallery-proofing-settings-nonce' ); ?>
				<?php submit_button( __( 'Save Settings', 'envira-gallery' ), 'primary', 'envira-gallery-proofing-settings-customer-submit', false ); ?>
			</form>
		</div>
		<?php
	}


	/**
	 * Callback for displaying the UI for general settings tab.
	 *
	 * @since 1.7.0
	 */
	public function settings_admin_emails_tab() {

		// Setup instance and meta key.
		$envira_metaboxes = new Metaboxes();

		// Get settings.
		$global_proofing_admin_email_notification = ( envira_proofing_get_setting( 'global_proofing_admin_email_notification' ) !== '' ) ? envira_proofing_get_setting( 'global_proofing_admin_email_notification' ) : envira_proofing_get_global_config_defaults( 'global_proofing_admin_email_notification' );
		$global_proofing_admin_email              = envira_proofing_get_setting( 'global_proofing_admin_email' );
		$global_proofing_email_subject            = ( envira_proofing_get_setting( 'global_proofing_admin_email_subject' ) ) ? envira_proofing_get_setting( 'global_proofing_admin_email_subject' ) : envira_proofing_get_global_config_defaults( 'global_proofing_admin_email_subject' );
		$global_proofing_email_message            = envira_proofing_get_setting( 'global_proofing_admin_email_message' );

		?>
		<div id="envira-settings-admin-emails">
			<?php
			// Output any notices now.
			do_action( 'envira_gallery_proofing_settings_general_tab_notice' );
			?>

			<h2>Admin Emails</h2>
			<hr />

			<!-- Settings Form -->
			<form id="envira-proofing-admin-emails-settings" method="post">
				<table class="form-table">
					<tbody>
					<tr id="envira-proofing-admin-send-notification-box">
						<th scope="row">
							<label for="envira-proofing-admin-send-notification-label"><?php esc_html_e( 'Send Admin Email?', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-admin-send-notification-label" type="checkbox" name="<?php echo esc_html( $this->key ); ?>[global_proofing_admin_email_notification]" value="1" <?php checked( $global_proofing_admin_email_notification, 1 ); ?> />
							<span class="description"><?php esc_html_e( 'Enables or disables global admin notifications on orders. Note that individual galleries may override this setting.', 'envira-gallery' ); ?></span>
						</td>

					</tr>
					<tr id="envira-proofing-email-box">
						<th scope="row">
							<label for="envira-proofing-email-label"><?php esc_html_e( 'Email Order Notifications To', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-email-label" type="text" name="<?php echo esc_html( $this->key ); ?>[global_proofing_admin_email]" value="<?php echo esc_html( $global_proofing_admin_email ); ?>" /><br />
							<span class="description"><?php esc_html_e( 'Enter a global email address that receives notifications when a new order is submitted. Leave blank to use the admin email in WordPress general settings. Note that individual galleries may override this setting.', 'envira-proofing' ); ?></span>
						</td>
					</tr>
					<tr id="envira-proofing-email-subject-box">
						<th scope="row">
							<label for="envira-proofing-email-subject"><?php esc_html_e( 'Email Subject', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<input id="envira-proofing-email-subject" type="text" style="width: 90%;" name="<?php echo esc_html( $this->key ); ?>[global_proofing_admin_email_subject]" value="<?php echo esc_attr( $global_proofing_email_subject ); ?>" /><br />
							<span class="description"><?php esc_html_e( 'The email message subject.', 'envira-proofing' ); ?></span>
						</td>
					</tr>
					<tr id="envira-proofing-email-message-box">
						<th scope="row">
							<label for="envira-proofing-email-message"><?php esc_html_e( 'Email Message', 'envira-proofing' ); ?></label>
						</th>
						<td>
							<?php
							if ( empty( $global_proofing_email_message ) ) {
								$global_proofing_email_message = envira_proofing_get_global_config_defaults( 'global_proofing_admin_email_message' );
							}
							wp_editor(
								$global_proofing_email_message,
								'envira-proofing-email-message',
								[
									'media_buttons' => false,
									'wpautop'       => true,
									'tinymce'       => true,
									'textarea_name' => $this->key . '[global_proofing_admin_email_message]',
								]
							);
							?>
							<p class="description">
								<?php esc_html_e( 'The email message sent once the user has submitted their order.', 'envira-proofing' ); ?><br />
								<?php esc_html_e( 'Supported Tags:', 'envira-proofing' ); ?><br />
								<strong>{title}: </strong><?php esc_html_e( 'The Gallery Title', 'envira-proofing' ); ?><br />
								<strong>{url}: </strong><?php esc_html_e( 'The Gallery Edit URL', 'envira-proofing' ); ?><br />
								<strong>{order_email}:</strong><?php esc_html_e( 'The customer\'s email address', 'envira-proofing' ); ?>
							</p>
						</td>
					</tr>

						<?php do_action( 'envira_gallery_proofing_settings_general_box' ); ?>
					</tbody>
				</table>

				<?php wp_nonce_field( 'envira-gallery-proofing-settings-nonce', 'envira-gallery-proofing-settings-nonce' ); ?>
				<?php submit_button( __( 'Save Settings', 'envira-gallery' ), 'primary', 'envira-gallery-proofing-settings-admin-submit', false ); ?>
			</form>
		</div>
		<?php
	}

	/**
	 * Saves the addon's settings for Galleries.
	 *
	 * @since 1.0.0
	 *
	 * @param array $settings  Array of settings to be saved.
	 * @param int   $post_id     The current post ID.
	 * @return array $settings Amended array of settings to be saved.
	 */
	public function save( $settings, $post_id ) {

		// Bail out if we fail a security check.
		if ( ! isset( $_POST['envira-gallery'] ) || ! wp_verify_nonce( sanitize_key( $_POST['envira-gallery'] ), 'envira-gallery' ) || ! isset( $_POST['_envira_gallery'] ) ) {
			return;
		}

		// Lightbox.
		$settings['config']['proofing_lightbox'] = ( isset( $_POST['_envira_gallery']['proofing_lightbox'] ) ? 1 : 0 );

		// Proofing Settings.
		$settings['config']['proofing']        = ( isset( $_POST['_envira_gallery']['proofing'] ) ? 1 : 0 );
		$settings['config']['proofing_edit']   = ( isset( $_POST['_envira_gallery']['proofing_edit'] ) ? 1 : 0 );
		$settings['config']['proofing_delete'] = ( isset( $_POST['_envira_gallery']['proofing_delete'] ) ? 1 : 0 );

		// Envira Proofing Integration Settings.
		$settings['config']['proofing_multiple_enabled']        = ( isset( $_POST['_envira_gallery']['proofing_multiple_enabled'] ) ? 1 : 0 );
		$settings['config']['proofing_name_enabled']            = ( isset( $_POST['_envira_gallery']['proofing_name_enabled'] ) ? 1 : 0 );
		$settings['config']['proofing_hide_gallery']            = ( isset( $_POST['_envira_gallery']['proofing_hide_gallery'] ) ? 1 : 0 );
		$settings['config']['proofing_quantity_enabled']        = ( isset( $_POST['_envira_gallery']['proofing_quantity_enabled'] ) ? 1 : 0 );
		$settings['config']['proofing_size_enabled']            = ( isset( $_POST['_envira_gallery']['proofing_size_enabled'] ) ? 1 : 0 );
		$settings['config']['proofing_add_to_order_label']      = isset( $_POST['_envira_gallery']['proofing_add_to_order_label'] ) ? esc_attr( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_add_to_order_label'] ) ) ) : false;
		$settings['config']['proofing_save_button_label']       = isset( $_POST['_envira_gallery']['proofing_save_button_label'] ) ? esc_attr( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_save_button_label'] ) ) ) : false;
		$settings['config']['proofing_edit_button_label']       = isset( $_POST['_envira_gallery']['proofing_edit_button_label'] ) ? esc_attr( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_edit_button_label'] ) ) ) : false;
		$settings['config']['proofing_delete_button_label']     = isset( $_POST['_envira_gallery']['proofing_delete_button_label'] ) ? esc_attr( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_delete_button_label'] ) ) ) : false;
		$settings['config']['proofing_notes_placeholder_text']  = isset( $_POST['_envira_gallery']['proofing_notes_placeholder_text'] ) ? esc_attr( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_notes_placeholder_text'] ) ) ) : false;
		$settings['config']['proofing_save_notes_button_label'] = isset( $_POST['_envira_gallery']['proofing_save_notes_button_label'] ) ? esc_attr( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_save_notes_button_label'] ) ) ) : false;
		$settings['config']['proofing_submit_button_label']     = isset( $_POST['_envira_gallery']['proofing_submit_button_label'] ) ? esc_attr( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_submit_button_label'] ) ) ) : false;
		$settings['config']['proofing_submitted_message']       = isset( $_POST['_envira_gallery']['proofing_submitted_message'] ) ? trim( sanitize_textarea_field( wp_unslash( $_POST['_envira_gallery']['proofing_submitted_message'] ) ) ) : false;
		$settings['config']['proofing_email']                   = isset( $_POST['_envira_gallery']['proofing_email'] ) ? sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_email'] ) ) : false;
		if ( isset( $_POST['_envira_gallery']['proofing_email_box_position'] ) ) {
			$settings['config']['proofing_email_box_position'] = isset( $_POST['_envira_gallery']['proofing_email_box_position'] ) ? sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_email_box_position'] ) ) : false;
		}
		$settings['config']['proofing_email_subject'] = isset( $_POST['_envira_gallery']['proofing_email_subject'] ) ? sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_email_subject'] ) ) : false;
		$settings['config']['proofing_email_message'] = isset( $_POST['_envira_gallery']['proofing_email_message'] ) ? trim( sanitize_textarea_field( wp_unslash( $_POST['_envira_gallery']['proofing_email_message'] ) ) ) : false;

		// Sizes.
		$sizes = [];
		if ( isset( $_POST['_envira_gallery']['proofing_sizes'] ) && ! empty( $_POST['_envira_gallery']['proofing_sizes']['name'] ) ) {
			// Build array of sizes.
			$names_sanitized = array_map( 'sanitize_text_field', wp_unslash( $_POST['_envira_gallery']['proofing_sizes']['name'] ) );
			foreach ( $names_sanitized as $index => $name ) {
				// Skip empty sizes.
				if ( empty( $_POST['_envira_gallery']['proofing_sizes']['name'][ $index ] ) ) {
					continue;
				}

				// Generate slug.
				$slug = str_replace( ' ', '_', esc_attr( strtolower( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_sizes']['name'][ $index ] ) ) ) ) );

				// Add to sizes array.
				$sizes[ $slug ] = [
					'slug'        => $slug,
					'name'        => isset( $_POST['_envira_gallery']['proofing_sizes']['name'][ $index ] ) ? esc_attr( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_sizes']['name'][ $index ] ) ) ) : false,
					'orientation' => isset( $_POST['_envira_gallery']['proofing_sizes']['orientation'][ $index ] ) ? esc_attr( sanitize_text_field( wp_unslash( $_POST['_envira_gallery']['proofing_sizes']['orientation'][ $index ] ) ) ) : false,
				];
			}
		}

		$settings['config']['proofing_sizes'] = $sizes;

		return $settings;
	}

	/**
	 * Retrieve the plugin basename from the plugin slug.
	 *
	 * @since 1.7.0
	 *
	 * @param string $slug The plugin slug.
	 * @return string         The plugin basename if found, else the plugin slug.
	 */
	public function get_plugin_basename_from_slug( $slug ) {

		$keys = array_keys( get_plugins() );

		foreach ( $keys as $key ) {
			if ( preg_match( '|^' . $slug . '|', $key ) ) {
				return $key;
			}
		}

		return $slug;
	}

	/**
	 * Add Settings page to plugin action links in the Plugins table.
	 *
	 * @since 1.7.0
	 *
	 * @param array $links  Default plugin action links.
	 * @return array $links Amended plugin action links.
	 */
	public function settings_link( $links ) {

		$settings_link = sprintf(
			'<a href="%s">%s</a> | <a href="%s">%s</a>',
			esc_url(
				add_query_arg(
					[
						'post_type' => 'envira_proof_order',
						'page'      => 'envira-gallery-settings-proofing',
					],
					admin_url( 'edit.php' )
				)
			),
			__( 'Settings', 'envira-gallery' ),
			esc_url(
				add_query_arg(
					[
						'post_type' => 'envira_proof_order',
					],
					admin_url( 'edit.php' )
				)
			),
			__( 'Orders', 'envira-gallery' )
		);
		array_unshift( $links, $settings_link );

		return $links;
	}
}
