<?php
/**
 * WP List Table Admin Class.
 *
 * @since 1.3.0
 *
 * @package Envira_Albums
 * @author  Envira Team
 */

namespace Envira\Proofing\Admin;

/**
 * WP List Table Admin Class.
 *
 * @since 1.3.0
 *
 * @package Envira_Albums
 * @author  Envira Team
 */
class Table {

	/**
	 * Holds the base class object.
	 *
	 * @since 1.3.0
	 *
	 * @var object
	 */
	public $base;

	/**
	 * Holds the metabox class object.
	 *
	 * @since 1.3.0
	 *
	 * @var object
	 */
	public $metabox;

	/**
	 * Primary class constructor.
	 *
	 * @since 1.3.0
	 */
	public function __construct() {

		// Append data to various admin columns.
		add_filter( 'manage_edit-envira_proof_order_columns', [ $this, 'columns' ] );
		add_action( 'manage_envira_proof_order_posts_custom_column', [ $this, 'custom_columns' ], 10, 2 );

		add_action( 'pre_get_posts', [ $this, 'pre_get_posts' ], 51 ); // upped to 51 because of Event Calendar plugin.

		// Expand search with IDs in addition to WordPress default of post/album titles.
		add_action( 'posts_where', [ $this, 'enable_search_by_album_id' ], 10 );

		// Adds filter to the orders table.
		add_action( 'restrict_manage_posts', [ $this, 'filter_tracked_plugins' ], 10 );
		add_filter( 'parse_query', [ $this, 'sort_plugins_by_slug' ], 10 );
	}

	/**
	 * Enables Search By Proofing ID
	 *
	 * @since 1.6.4.1
	 *
	 * @param  string $where Search.
	 * @return null Return early if not on the proper value.
	 */
	public function enable_search_by_album_id( $where ) {

		// Bail if we are not in the admin area or not doing a search.
		if ( ! is_admin() || ! is_search() ) {
			return $where;
		}

		// Bail if this is not the envira page.
		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		if ( empty( $_GET['post_type'] ) || 'envira_proof_order' !== $_GET['post_type'] ) {
			return $where;
		}

		global $wpdb;

		// Get the value that is being searched.
		$search_string = get_query_var( 's' );

		if ( is_numeric( $search_string ) ) {

			$where = str_replace( '(' . $wpdb->posts . '.post_title LIKE', '(' . $wpdb->posts . '.ID = ' . $search_string . ') OR (' . $wpdb->posts . '.post_title LIKE', $where );

		} elseif ( preg_match( '/^(\d+)(,\s*\d+)*$/', $search_string ) ) { // string of post IDs.

			$where = str_replace( '(' . $wpdb->posts . '.post_title LIKE', '(' . $wpdb->posts . '.ID in (' . $search_string . ')) OR (' . $wpdb->posts . '.post_title LIKE', $where );
		}

		return $where;
	}

	/**
	 * Get Posts
	 *
	 * @since 1.3.0
	 *
	 * @param  array $query Query.
	 */
	public function pre_get_posts( $query ) {

		if ( is_admin() && 'edit.php' === $GLOBALS['pagenow']

			&& $query->is_main_query()
			&& $query->get( 'post_type' ) === 'envira_proof_order' ) {

			$this->stickies   = [];
			$this->stickies[] = get_option( 'envira_default_album' );
			$this->stickies[] = get_option( 'envira_dynamic_album' );

			add_filter( 'post_class', [ $this, 'post_class' ], 10, 3 );
			add_filter( 'option_sticky_posts', [ $this, 'custom_stickies' ] );
			$query->is_home = 1;
			$query->set( 'ignore_sticky_posts', 0 );

		}
	}

	/**
	 * Customize the Stickies
	 *
	 * @since 1.3.0
	 *
	 * @param  array $data Data.
	 * @return array $data Data.
	 */
	public function custom_stickies( $data ) {

		if ( count( $this->stickies ) > 0 ) {

			$data = $this->stickies;

		}

		return $data;
	}

	/**
	 * Customize the CSS
	 *
	 * @since 1.3.0
	 *
	 * @param array   $classes Array of classes.
	 * @param string  $css_class Class Name.
	 * @param integer $id      Class ID.
	 *
	 * @return array $classes CSS.
	 */
	public function post_class( $classes, $css_class, $id ) {

		if ( in_array( $id, $this->stickies, true ) ) {

			$classes[] = 'is-admin-sticky';

		}

		return $classes;
	}

	/**
	 * Customize the post columns for the Envira Album post type.
	 *
	 * @since 1.3.0
	 *
	 * @param array $columns  The default columns.
	 * @return array $columns Amended columns.
	 */
	public function columns( $columns ) {

		// Add additional columns we want to display.
		$envira_columns = [
			'cb'             => '<input type="checkbox" />',
			'title'          => __( 'Order', 'envira-proofing' ),
			'customer_name'  => __( 'Customer Name', 'envira-proofing' ),
			'customer_email' => __( 'Customer Email', 'envira-proofing' ),
			'gallery_id'     => __( 'Gallery ID', 'envira-proofing' ),
			'order_id'       => __( 'Order ID', 'envira-proofing' ),
			'status'         => __( 'Status', 'envira-proofing' ),
			'date'           => __( 'Date', 'envira-proofing' ),
		];

		unset( $columns['title'] );
		unset( $columns['author'] );

		// Allow filtering of columns.
		$envira_columns = apply_filters( 'envira_proof_orders_table_columns', $envira_columns, $columns );

		// Return merged column set.  This allows plugins to output their columns (e.g. Yoast SEO),
		// and column management plugins, such as Admin Columns, should play nicely.
		return array_merge( $envira_columns, $columns );
	}

	/**
	 * Add data to the custom columns added to the Envira Album post type.
	 *
	 * @since 1.3.0
	 *
	 * @global object $post  The current post object
	 * @param string $column The name of the custom column.
	 * @param int    $post_id   The current post ID.
	 */
	public function custom_columns( $column, $post_id ) {

		$post_id = absint( $post_id );

		switch ( $column ) {
			/**
			* Customer Name
			*/
			case 'customer_name':
				$proofing_name = get_post_meta( $post_id, 'proofing_name', true );
				echo ( isset( $proofing_name ) && ! empty( trim( esc_html( $proofing_name ) ) ) ? esc_html( $proofing_name ) : 'Unknown' );
				break;

			/**
			* Customer Email
			*/
			case 'customer_email':
				$proofing_email = get_post_meta( $post_id, 'proofing_email', true );
				echo ( isset( $proofing_email ) && is_email( esc_html( $proofing_email ) ) ? esc_html( $proofing_email ) : 'Unknown' );
				break;

			/**
			* Gallery ID
			*/
			case 'gallery_id':
				$gallery_id = get_post_meta( $post_id, 'gallery_id', true );
				$gallery    = get_post( $gallery_id );
				echo ( isset( $gallery_id ) && ! empty( $gallery ) ? '<a target="_blank" href="' . esc_url( get_edit_post_link( $gallery_id ) ) . '">' . esc_html( $gallery->post_title ) . '</a>' : 'Unknown' );
				break;

			/**
			* Order ID
			*/
			case 'order_id':
				echo intval( $post_id );
				break;

			/**
			* Custom Post Status
			*/
			case 'status':
				$post = get_post( $post_id );
				echo esc_html( ucfirst( str_replace( 'proofing-', '', $post->post_status ) ) );
				break;

			/**
			* Last Modified
			*/
			case 'date':
				the_modified_date();
				break;
		}
	}

	/**
	 * Placeholder description.
	 *
	 * @since 2.0.0
	 *
	 * @param  array $query The Query.
	 */
	public function sort_plugins_by_slug( $query ) {
		global $pagenow;

		// Get the post type.
		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		$post_type = isset( $_GET['post_type'] ) ? sanitize_text_field( wp_unslash( $_GET['post_type'] ) ) : false;

		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		if ( is_admin() && 'edit.php' === $pagenow && 'envira_proof_order' === $post_type && isset( $_GET['envira_gallery_id'] ) && strtolower( sanitize_text_field( wp_unslash( $_GET['envira_gallery_id'] ) ) ) !== 'all' ) {
			// TODO CHECK WHY meta_key was removed.
			$query->query_vars['meta_key'] = 'gallery_id';
			// phpcs:ignore WordPress.Security.NonceVerification.Recommended
			$query->query_vars['meta_value']   = intval( $_GET['envira_gallery_id'] );
			$query->query_vars['meta_compare'] = '=';
		}
	}

	/**
	 * Placeholder description.
	 *
	 * @since 2.0.0
	 */
	public function filter_tracked_plugins() {
		global $current_type;
		global $wp_query;
		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		if ( isset( $_GET['post_type'] ) && 'envira_proof_order' === esc_html( strtolower( sanitize_text_field( wp_unslash( $_GET['post_type'] ) ) ) ) ) {

			$galleries     = envira_get_galleries( false );
			$gallery_array = [];

			$query      = [
				'post_type'   => 'envira_proof_order',
				'post_status' => [ 'publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit', 'proofing-completed', 'proofing-pending', 'proofing-active' ],
			];
			$post_count = new \WP_Query( $query ); // find a better way.

			if ( is_array( $galleries ) && ! empty( $galleries ) && 0 !== $post_count->post_count ) {
				echo '<select name="envira_gallery_id" id="envira_gallery_id">';

				echo '<option value="all" ' . selected( 'all', $current_type ) . '>'; // used to be $current_plugin.
				esc_html_e( 'All', 'envira-proofing' );
				echo '</option>';
				// phpcs:ignore WordPress.Security.NonceVerification.Recommended
				$envira_gallery_id = isset( $_GET['envira_gallery_id'] ) ? sanitize_text_field( wp_unslash( $_GET['envira_gallery_id'] ) ) : false;

				foreach ( $galleries as $gallery ) {

					if ( ! isset( $gallery['id'] ) || ! $gallery['id'] ) {
						continue;
					}

					// Instead of pulling the title from config, attempt to pull it from the gallery post first.
					$gallery_post = get_post( $gallery['id'] );

					if ( $gallery_post && ! empty( $gallery_post->post_title ) ) {
						$title = $gallery_post->post_title;
					} elseif ( ! empty( $gallery['config']['title'] ) ) {
						$title = $gallery['config']['title'];
					} elseif ( ! empty( $gallery['config']['slug'] ) ) {
						$title = $gallery['config']['slug'];
					} else {
						// translators: note.
						$title = sprintf( __( 'Gallery ID #%s', 'envira-gallery' ), $gallery['id'] );
					}

					$gallery_array[ $title ] = [
						'gallery_id' => '' . $gallery['id'] . '',
					];

				}

				// sort the array.
				ksort( $gallery_array );

				// loop through the sorted array.
				foreach ( $gallery_array as $gallery_title => $gallery_id ) {
					echo '<option value="' . esc_html( $gallery_id['gallery_id'] ) . '"' . selected( $gallery_id['gallery_id'], $envira_gallery_id ) . '>' . esc_html( $gallery_title ) . '</option>';
				}

				echo '</select>';
			} else {
				return;
			}
		}
	}
}
