<?php
/**
 * Order class.
 *
 * @since 1.0.0
 *
 * @package Envira Gallery
 * @subpackage Envira Proofing
 * @author Envira Gallery Team <support@enviragallery.com>
 */

namespace Envira\Proofing\Frontend;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Order class.
 *
 * @since 1.0.0
 *
 * @package Envira_Proofing
 * @author  Envira Team
 */
class Email {

	/**
	 * Holds the class object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public static $instance;

	/**
	 * Path to the file.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	public $file = __FILE__;

	/**
	 * Holds the base class object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public $base;

	/**
	 * Primary class constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {

		add_action( 'envira_proofing_email_template_before', [ $this, 'customize_email_logo' ], 10, 1 );
		add_action( 'envira_proofing_email_template_before', [ $this, 'customize_email_from_address' ], 10, 1 );
		add_filter( 'envira_proofing_email_notification_admin_subject', [ $this, 'customize_email_subject' ], 10, 5 );
	}

	/**
	 * A wrapper dedicated to customizating admin emails via customize_email().
	 *
	 * @since 2.0.0
	 *
	 * @param string  $email_subject Message.
	 * @param string  $email_to Email address.
	 * @param integer $proofing_id Proofing ID.
	 * @param array   $order Order.
	 * @param string  $status_slug Status Slug.
	 */
	public function customize_email_subject( $email_subject, $email_to, $proofing_id, $order, $status_slug ) {

		$gallery = ! empty( $order['gallery_id'] ) ? get_post( $order['gallery_id'] ) : false;

		if ( ! empty( $gallery ) ) {
			$email_subject = str_replace( '{title}', $gallery->post_title, $email_subject );
		}

		return $email_subject;
	}


	/**
	 * Sends the admin email.
	 *
	 * @since 2.0.0
	 *
	 * @param integer $proofing_id Proofing ID.
	 * @param array   $order Order.
	 * @param string  $status_slug Status Slug.
	 */
	public function send_admin_email( $proofing_id, $order, $status_slug ) {

		$gallery_id = empty( $order['gallery_id'] ) ? false : intval( $order['gallery_id'] );

		// If the global AND the individual gallery settings aren't checked off, then don't send the admin notification.
		if ( 0 === intval( envira_proofing_get_global_setting( 'global_proofing_admin_email_notification' ) ) && false === envira_proofing_get_setting_from_gallery( 'proofing_admin_email_notification', $gallery_id ) ) {
			return;
		}

		// Setup some variables.

		$subtype = ( 'order-submitted' === $status_slug ) ? 'submitted' : false;
		$subtype = ( 'order-saved' === $status_slug ) ? 'saved' : $subtype;

		$setting_type = ( 'order-submitted' === $status_slug ) ? 'submit' : false;
		$setting_type = ( 'order-saved' === $status_slug ) ? 'save' : $setting_type;

		// Use global or is there a specific admin email for this?
		$email_to = envira_proofing_get_setting_from_gallery( 'proofing_admin_email', $gallery_id ) ? envira_proofing_get_setting_from_gallery( 'proofing_admin_email', $gallery_id ) : envira_proofing_get_global_setting( 'global_proofing_admin_email' );

		// If there isn't a valid email address at this point, revert to default.
		if ( ! is_email( $email_to ) ) {
			$email_to = envira_proofing_get_global_config_defaults( 'global_proofing_admin_email' );
		}
		$email_to = apply_filters( 'envira_proofing_email_notification_admin_email_to', $email_to, $proofing_id, $order, $status_slug, $gallery_id );

		// Use global or is there a specific admin subject for this email?
		$email_subject = envira_proofing_get_setting_from_gallery( 'proofing_admin_email_subject', $gallery_id ) ? envira_proofing_get_setting_from_gallery( 'proofing_admin_email_subject', $gallery_id ) : envira_proofing_get_global_setting( 'global_proofing_admin_email_subject_' . $setting_type );
		// If there isn't a valid email address at this point, revert to default.
		if ( empty( $email_subject ) ) {
			$email_subject = envira_proofing_get_global_config_defaults( 'global_proofing_admin_email_subject_' . $setting_type );
		}
		$email_subject = apply_filters( 'envira_proofing_email_notification_admin_subject', $email_subject, $email_to, $proofing_id, $order, $status_slug, $gallery_id );
		$email_subject = $this->customize_subject( $email_subject, $proofing_id, $status_slug, $order, $subtype );

		// Email body... step 1: load the template.
		$template_path = $this->locate_template( [ 'admin_email' ] );
		$email_message = file_get_contents( $template_path );

		// step 2: customize the template.
		$email_message = $this->customize_admin_email( $email_message, $proofing_id, $status_slug, $order );

		// final check: if there is no email or subject, don't send.
		if ( empty( $email_to ) || ! is_email( $email_to ) ) {
			// debugging log here?
			return;
		}

		// final step: send the email, with HTML content type.
		add_filter( 'wp_mail_content_type', [ $this, 'proofing_set_html_mail_content_type' ] );

		$confirmation = wp_mail( $email_to, $email_subject, $email_message );

		remove_filter( 'wp_mail_content_type', [ $this, 'proofing_set_html_mail_content_type' ] );

		return $confirmation;
	}

	/**
	 * Sends the customer email.
	 *
	 * @since 2.0.0
	 *
	 * @param integer $proofing_id Proofing ID.
	 * @param array   $order Order.
	 * @param string  $status_slug Status Slug.
	 */
	public function send_customer_email( $proofing_id, $order, $status_slug ) {

		$gallery_id = empty( $order['gallery_id'] ) ? false : intval( $order['gallery_id'] );

		// Setup some variables.

		$subtype = ( 'order-submitted' === $status_slug ) ? 'submitted' : false;
		$subtype = ( 'order-saved' === $status_slug ) ? 'saved' : $subtype;

		$setting_type = ( 'order-submitted' === $status_slug ) ? 'submit' : false;
		$setting_type = ( 'order-saved' === $status_slug ) ? 'save' : $setting_type;

		$email_to = apply_filters( 'envira_proofing_email_notification_customer_email_to', sanitize_email( $order['email'] ), $proofing_id, $order, $status_slug, $gallery_id );

		// Use global or is there a specific admin subject for this email?
		$email_subject = envira_proofing_get_setting_from_gallery( 'proofing_customer_email_subject', $gallery_id ) ? envira_proofing_get_setting_from_gallery( 'proofing_customer_email_subject', $gallery_id ) : envira_proofing_get_global_setting( 'global_proofing_customer_email_subject_' . $setting_type );

		// If there isn't a valid email address at this point, revert to default.
		if ( empty( $email_subject ) ) {
			$email_subject = envira_proofing_get_global_config_defaults( 'global_proofing_customer_email_subject_' . $setting_type );
		}
		$email_subject = apply_filters( 'envira_proofing_email_notification_customer_subject', $email_subject, $email_to, $proofing_id, $order, $status_slug, $gallery_id );
		$email_subject = $this->customize_subject( $email_subject, $proofing_id, $status_slug, $order, $subtype );

		// Email body... step 1: load the template and set variables.
		$subtype       = ( 'order-submitted' === $status_slug ) ? 'submit' : 'save'; // other choice is order-saved.
		$template_path = $this->locate_template( [ 'customer_email_' . $subtype ] );
		$email_message = file_get_contents( $template_path );
		// step 2: customize the template.
		$email_message = $this->customize_customer_email_message( $email_message, $proofing_id, $status_slug, $order, $subtype );

		// final check: if there is no email or subject, don't send.
		if ( empty( $email_to ) || ! is_email( $email_to ) ) {
			// debugging log here?
			return;
		}

		// final step: send the email, with HTML content type.
		add_filter( 'wp_mail_content_type', [ $this, 'proofing_set_html_mail_content_type' ] );

		$confirmation = wp_mail( $email_to, $email_subject, $email_message );

		remove_filter( 'wp_mail_content_type', [ $this, 'proofing_set_html_mail_content_type' ] );

		return $confirmation;
	}

	/**
	 * A wrapper dedicated to customizating admin emails via customize_email().
	 *
	 * @since 2.0.0
	 *
	 * @param string  $email_message Message.
	 * @param integer $proofing_id Proofing ID.
	 * @param string  $status_slug Status Slug.
	 * @param array   $order Order.
	 * @param string  $subtype Subtype - Save or Submitted.
	 */
	public function customize_admin_email( $email_message, $proofing_id = false, $status_slug = false, $order = false, $subtype = false ) {

		return $this->customize_email( $email_message, $proofing_id, $status_slug, $order, 'admin', $subtype );
	}

	/**
	 * A wrapper dedicated to customizating customer emails via customize_email().
	 *
	 * @since 2.0.0
	 *
	 * @param string  $email_message Message.
	 * @param integer $proofing_id Proofing ID.
	 * @param string  $status_slug Status Slug.
	 * @param array   $order Order.
	 * @param string  $subtype Subtype - Save or Submitted.
	 */
	public function customize_customer_email_message( $email_message, $proofing_id = false, $status_slug = false, $order = false, $subtype = false ) {

		return $this->customize_email( $email_message, $proofing_id, $status_slug, $order, 'customer', $subtype );
	}

	/**
	 * Replaces Envira Proofing tags with actual info for email subject.
	 *
	 * @since 2.0.0
	 *
	 * @param string  $email_subject Message.
	 * @param integer $proofing_id Proofing ID.
	 * @param string  $status_slug Status Slug.
	 * @param array   $order Order.
	 * @param string  $subtype Subtype - Save or Submitted.
	 */
	public function customize_subject( $email_subject, $proofing_id, $status_slug, $order, $subtype ) {

		if ( false === $proofing_id ) {
			return $email_subject;
		}

		/* get the gallery id in case we need to look in the settings to pass any overrides */
		$gallery_id = empty( $order['gallery_id'] ) ? false : intval( $order['gallery_id'] );
		if ( ! empty( $gallery_id ) ) {
			$gallery = ! empty( $order['gallery_id'] ) ? get_post( $order['gallery_id'] ) : false;
			if ( ! empty( $gallery ) ) {
				$email_subject = str_replace( '{title}', $gallery->post_title, $email_subject );
				$email_subject = str_replace( '{url}', get_permalink( $gallery->ID ), $email_subject );
			}
			if ( ! empty( $order['email'] ) ) {
				$email_subject = str_replace( '{order_email}', $order['email'], $email_subject );
			}
		}

		return $email_subject;
	}

	/**
	 * Replaces Envira Proofing tags with actual info, with filters before and after.
	 *
	 * @since 2.0.0
	 *
	 * @param string  $email_message Message.
	 * @param integer $proofing_id Proofing ID.
	 * @param string  $status_slug Status Slug.
	 * @param array   $order Order.
	 * @param string  $type Customer or Admin.
	 * @param string  $subtype Subtype - Save or Submitted.
	 */
	public function customize_email( $email_message, $proofing_id = false, $status_slug = false, $order = false, $type = false, $subtype = 'saved' ) {

		if ( false === $proofing_id || false === $type ) {
			return $email_message;
		}

		$proofing_order_post = get_post( $proofing_id );

		/* get the gallery id in case we need to look in the settings to pass any overrides */
		$gallery_id = empty( $order['gallery_id'] ) ? false : intval( $order['gallery_id'] );

		/* general filter before modifications */
		$email_message = apply_filters( 'envira_proofing_email_template_before', $email_message, $proofing_id, $status_slug, $order, $type, $subtype );

		/* order specifics */
		$email_message = str_replace( '{envira_proofing_order_id}', $proofing_id, $email_message );
		$email_message = str_replace( '{envira_proofing_order_created}', gmdate( 'F jS, Y g:ia T', strtotime( $proofing_order_post->post_date ) ), $email_message );
		$email_message = str_replace( '{envira_proofing_order_customer_name}', esc_html( $order['name'] ), $email_message );
		$email_message = str_replace( '{envira_proofing_order_customer_email}', esc_html( $order['email'] ), $email_message );
		$email_message = str_replace( '{envira_proofing_order_images_ordered}', $this->customize_email_images_ordered( $order['images'] ), $email_message );
		$email_message = str_replace( '{envira_proofing_order_images_total}', $this->customize_email_images_total( $order['images'] ), $email_message );
		$email_message = str_replace( '{envira_proofing_order_customer_notes}', esc_html( $order['notes'] ), $email_message );

		/* admin related */

		if ( 'admin' === $type ) {

			$admin_email_message = envira_proofing_get_setting( 'global_proofing_admin_email_message' );

			$gallery = ! empty( $order['gallery_id'] ) ? get_post( $order['gallery_id'] ) : false;
			if ( ! empty( $gallery ) ) {
				$admin_email_message = str_replace( '{title}', $gallery->post_title, $admin_email_message );
				$admin_email_message = str_replace( '{url}', get_permalink( $gallery->ID ), $admin_email_message );
				$admin_email_message = str_replace( '{order_email}', $order['email'], $admin_email_message );
				// make any links HTML linkable.
				$admin_email_message = make_clickable( $admin_email_message );
				$admin_email_message = wpautop( $admin_email_message );
			}

			$email_message = str_replace( '{envira_proofing_order_admin_footer}', '', $email_message );
			$email_message = str_replace( '{envira_proofing_order_message_to_admin}', $admin_email_message, $email_message );

		}

		/* customer related */

		if ( 'customer' === $type ) {

			if ( 'saved' === $subtype ) {

				$customer_email_message = ( ! empty( envira_proofing_get_setting( 'proofing_customer_email_message_save', $gallery_id ) ) ) ? envira_proofing_get_setting( 'proofing_customer_email_message_save', $gallery_id ) : envira_proofing_get_global_config_defaults( 'global_proofing_customer_email_message_save' );

			} else {

				$customer_email_message = ( ! empty( envira_proofing_get_setting( 'proofing_customer_email_message_submit', $gallery_id ) ) ) ? envira_proofing_get_setting( 'proofing_customer_email_message_submit', $gallery_id ) : envira_proofing_get_global_config_defaults( 'global_proofing_customer_email_message_submit' );

			}

			$gallery = ! empty( $order['gallery_id'] ) ? get_post( $order['gallery_id'] ) : false;
			if ( ! empty( $gallery ) ) {
				$customer_email_message = str_replace( '{title}', $gallery->post_title, $customer_email_message );
				$customer_email_message = str_replace( '{url}', get_permalink( $gallery->ID ), $customer_email_message );
				$customer_email_message = str_replace( '{order_email}', $order['email'], $customer_email_message );
				// make any links HTML linkable.
				$customer_email_message = make_clickable( $customer_email_message );
				$customer_email_message = wpautop( $customer_email_message );
			}

			$email_message = str_replace( '{envira_proofing_order_customer_footer}', '', $email_message );
			$email_message = str_replace( '{envira_proofing_order_message_to_customer}', $customer_email_message, $email_message );
		}

		/* general filter after modifications */
		$email_message = apply_filters( 'envira_proofing_email_template_after', $email_message, $proofing_id, $status_slug, $order, $type, $subtype );

		return $email_message;
	}

	/**
	 * Outputs total count of images in the email template.
	 *
	 * @since 2.0.0
	 *
	 * @param array $proofing_images Array of image information.
	 * @return string $total_count Modified String.
	 */
	public function customize_email_images_total( $proofing_images ) {

		$total_count = 0;

		if ( empty( $proofing_images ) ) {
			return $total_count;
		}

		foreach ( $proofing_images as $proofing_image_id => $proofing_image_details ) {

			$proofing_image       = get_post( $proofing_image_id );
			$proofing_image_title = ( ! empty( $gallery_data['gallery'][ $proofing_image_id ]['title'] ) ) ? esc_html( $gallery_data['gallery'][ $proofing_image_id ]['title'] ) : $proofing_image->post_title;

			$proofing_sizes = ( is_array( $proofing_image_details ) ) ? $proofing_image_details : false;

			if ( ! empty( $proofing_sizes ) ) {
				foreach ( $proofing_sizes as $size => $count ) {
					$total_count += intval( $count );
				}
			}
		}

		return $total_count;
	}

	/**
	 * Outputs ordered image and info in the email template.
	 *
	 * @since 2.0.0
	 *
	 * @param array $proofing_images Array of image information.
	 * @return string $html Modified String.
	 */
	public function customize_email_images_ordered( $proofing_images ) {

		if ( empty( $proofing_images ) ) {
			return false;
		}

		$html = '<tr class="heading">
			<td>
				Item
			</td>

			<td>
				Size
			</td>

			<td>
				Amount
			</td>
		</tr>';

		$total_count = 0;

		foreach ( $proofing_images as $proofing_image_id => $proofing_image_details ) {

			$proofing_image       = get_post( $proofing_image_id );
			$proofing_image_title = ( ! empty( $gallery_data['gallery'][ $proofing_image_id ]['title'] ) ) ? esc_html( $gallery_data['gallery'][ $proofing_image_id ] ['title'] ) : $proofing_image->post_title;

			$proofing_sizes = ( is_array( $proofing_image_details ) ) ? $proofing_image_details : false;

			$html .= '
			<tr class="item">
				<td>
					' . wp_get_attachment_image( $proofing_image_id, [ '50', '50' ], '', [ 'class' => 'img-responsive' ] ) . '<br/>
					' . $proofing_image_title . '
				</td>

				<td colspan="2">';

			if ( ! empty( $proofing_sizes ) ) {
				foreach ( $proofing_sizes as $size => $count ) {
					$total_count += intval( $count );
					$html        .= '

					<table style="width: 100%;">
						<tr class="item">
							<td style="padding: 0; width: 50%;">' . esc_html( $size ) . '</td>
							<td style="padding: 0; text-align:right;">' . intval( $count ) . '</td>
						</tr>
					</table>';
				}
			}

				$html .= ' </td>
			</tr>';

		}

		return $html;
	}

	/**
	 * Customizes/adds a logo to the email template.
	 *
	 * @since 2.0.0
	 *
	 * @param string $email_message Message to change.
	 * @return string $email_message Modified String.
	 */
	public function customize_email_logo( $email_message ) {

		$proofing_email_logo      = envira_proofing_get_setting( 'proofing_email_logo' );
		$proofing_email_logo_html = ( ! empty( $proofing_email_logo ) ) ? '<img src="' . $proofing_email_logo . '" style="width:100%; max-width:300px;">' : false;
		$proofing_email_logo_html = apply_filters( 'envia_proofing_email_logo_invoice', $proofing_email_logo_html );

		$email_message = str_replace( '{envira_proofing_order_logo}', $proofing_email_logo_html, $email_message );
		return $email_message;
	}

	/**
	 * Cusotmizes/adds an address to the email template.
	 *
	 * @since 2.0.0
	 *
	 * @param string $email_message Message to change.
	 * @return string $email_message Modified String.
	 */
	public function customize_email_from_address( $email_message ) {

		$proofing_email_address      = envira_proofing_get_setting( 'proofing_email_mailing_address' );
		$proofing_email_address_html = apply_filters( 'envia_proofing_email_address_invoice', nl2br( $proofing_email_address ) );

		$email_message = str_replace( '{envira_proofing_order_from_address}', $proofing_email_address_html, $email_message );
		return $email_message;
	}

	/**
	 * Set format of email to HTML
	 *
	 * @since 1.0.8
	 */
	public function proofing_set_html_mail_content_type() {

		return 'text/html';
	}

	/**
	 * Locates the email template.
	 *
	 * @since 2.0.0
	 *
	 * @param array $template_names Template names.
	 */
	public function locate_template( $template_names ) {

		// No file found yet.
		$located = false;

		// Try to find a template file.
		foreach ( (array) $template_names as $template_name ) {

			// Continue if template is empty.
			if ( empty( $template_name ) ) {
				continue;
			}

			// Trim off any slashes from the template name.
			$template_name = ltrim( $template_name, '/' );

			// Add the .html extension.
			$template_name .= '.html';

			if ( file_exists( trailingslashit( get_stylesheet_directory() ) . 'envira/proofing/email/' . $template_name ) ) {
				// Check child theme first.
				$located = trailingslashit( get_stylesheet_directory() ) . 'envira/proofing/email/' . $template_name;
				break;
			} elseif ( file_exists( trailingslashit( get_template_directory() ) . 'envira/proofing/email/' . $template_name ) ) {
				// Check parent theme next.
				$located = trailingslashit( get_template_directory() ) . 'envira/proofing/email/' . $template_name;
				break;
			} elseif ( file_exists( trailingslashit( ENVIRA_PROOFING_DIR ) . 'templates/email/' . $template_name ) ) {
				// Check theme compatibility last.
				$located = trailingslashit( ENVIRA_PROOFING_DIR ) . 'templates/email/' . $template_name;
				break;
			}
		}

		return $located;
	}

	/**
	 * Returns the singleton instance of the class.
	 *
	 * @since 1.0.0
	 *
	 * @return object The Envira_Proofing_Order object.
	 */
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Envira_Proofing_Order ) ) {
			self::$instance = new Envira_Proofing_Order();
		}

		return self::$instance;
	}
}
