<?php
/**
 * Order class.
 *
 * @since 1.0.0
 *
 * @package Envira Gallery
 * @subpackage Envira Proofing
 * @author Envira Gallery Team <support@enviragallery.com>
 */

namespace Envira\Proofing\Frontend;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use Envira\Proofing\Frontend\Email;
use Envira\Admin\Notices;
use WP_Query;

/**
 * Order class.
 *
 * @since 1.0.0
 *
 * @package Envira_Proofing
 * @author  Envira Team
 */
class Order {

	/**
	 * Holds the class object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public static $instance;

	/**
	 * Path to the file.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	public $file = __FILE__;

	/**
	 * Holds the base class object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public $base;

	/**
	 * Primary class hooks.
	 *
	 * @since 1.0.0
	 */
	public function hooks() {
		/* legacy */
		add_action( 'init', [ $this, 'migrate' ] );

		/* catch admin actions */
		add_action( 'admin_init', [ $this, 'resend_email_confirmation' ] );
		add_action( 'admin_init', [ $this, 'create_gallery_from_order' ] );

		/* various times to force update of active count in admin */
		add_action( 'save_post_envira_proof_order', [ $this, 'reset_active_order_count' ] );

		// Delete cookies on trash.
		add_action( 'trash_envira_proof_order', [ $this, 'trash_envira_proof_order' ], 20 );
	}

	/**
	 * Iterates through galleries, updating the meta structure for 1.0.4+
	 *
	 * @since 1.0.4
	 */
	public function migrate() {

		// Bail if the 1.0.4 migration process completed.
		$migrated = get_option( 'envira_proofing_104' );
		if ( $migrated ) {
			return;
		}

		// Get galleries.
		$galleries = envira_get_galleries( true, true ); // ignore empty but don't ignore cache.
		if ( ! $galleries || count( $galleries ) === 0 ) {
			return;
		}

		// Iterate through galleries.
		foreach ( $galleries as $gallery ) {
			// Check for 1.0.3- post meta.
			$images = get_post_meta( $gallery['id'], '_envira_proofing_order', true );
			if ( empty( $images ) ) {
				continue;
			}

			// Move order data.
			$order = [
				'images'    => get_post_meta( $gallery['id'], '_envira_proofing_order', true ),
				'email'     => false,
				'notes'     => get_post_meta( $gallery['id'], '_envira_proofing_order_notes', true ),
				'submitted' => get_post_meta( $gallery['id'], '_envira_proofing_order_submitted', true ),
			];

			// Save order.
			$this->save_order( $gallery['id'], $order );

		}

		// Mark the migration as completed.
		update_option( 'envira_proofing_104', true );
	}

	/**
	 * Retrieves an order from the given Gallery ID
	 *
	 * @since 1.0
	 *
	 * @param   int     $gallery_id     Gallery ID.
	 * @param   string  $email          Email Address.
	 * @param   integer $limit Limit.
	 * @param   boolean $order_id Order ID.
	 * @return  WP_Query Orders
	 */
	public function get_orders( $gallery_id, $email = false, $limit = false, $order_id = false ) {

		// Find the custom post type with the associated gallery id.
		$args = [
			'post_type'      => 'envira_proof_order',
			'post_status'    => [ 'proofing-active', 'proofing-pending', 'proofing-complete' ],
			'posts_per_page' => ( false !== $limit ) ? $limit : -1,
			'meta_query'     => [  // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
				[
					'key'   => 'gallery_id',
					'value' => $gallery_id,
				],
			],
		];

		if ( false !== $email ) {
			$args['meta_query'][] = [
				'key'   => 'proofing_email',
				'value' => $email,
			];
		}

		$proofing_orders = new WP_Query( $args );

		// Filter.
		$proofing_orders = apply_filters( 'envira_proofing_order_cpt_get_orders', $proofing_orders, $gallery_id );

		return $proofing_orders;
	}

	/**
	 * This forces an active order recount, usually hooked to where the custom posts are updated or deleted.
	 *
	 * @since 2.0.0
	 *
	 * @return  integer Order total.
	 */
	public function reset_active_order_count() {

		return ( $this->get_active_orders_total( true ) );
	}

	/**
	 * Retrieves all active orders, usually to keep track of the total.
	 *
	 * @since 2.0.0
	 *
	 * @param boolean $flush_cache Flush cache.
	 * @return integer Order total.
	 */
	public function get_active_orders_total( $flush_cache = false ) {

		$order_total = get_transient( '_eg_proofing_active_orders_total' );

		// Attempt to return the transient first, otherwise generate the new query to retrieve the data.
		if ( true === $flush_cache || false === $order_total ) {

			// Find the custom post type with the associated gallery id.
			$args = [
				'post_type'      => 'envira_proof_order',
				'post_status'    => [ 'proofing-active' ],
				'posts_per_page' => -1,
				'meta_query'     => [ // phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
					[
						'key'     => 'gallery_id',
						'compare' => 'EXISTS',
					],
				],
			];

			$active_orders = new WP_Query( $args );
			// Store this in a transient.
			$expiration  = $this->get_transient_expiration_time();
			$order_total = ! empty( $active_orders->post_count ) ? intval( $active_orders->post_count ) : 0;
			set_transient( '_eg_proofing_active_orders_total', $order_total, $expiration );

		}

		return $order_total;
	}



	/**
	 * Attempts to find an order for the given email address.
	 *
	 * If one doesn't exist, creates a new, blank order comprising of email + optional name
	 *
	 * @param   int    $gallery_id     Gallery ID.
	 * @param   array  $data           Gallery Config.
	 * @param   string $email          Email Address.
	 * @param   string $name           Name.
	 * @param   string $notes          Notes.
	 * @return  array                   Order
	 */
	public function get_or_create_order( $gallery_id, $data, $email, $name = '', $notes = '' ) {

		// Get orders.
		$orders = $this->get_orders( $gallery_id, $email );

		// Get order.
		if ( ! empty( $orders->posts ) && ! empty( $orders->post->ID ) ) {
			// Existing order - update name + notes.
			$proofing_order_id = intval( $orders->post->ID );
			// No order - create.
		} elseif ( ! empty( $email ) ) {
				$proofing_order_id = wp_insert_post(
					[
						'post_title'   => 'order from gallery ' . $gallery_id,
						'post_type'    => 'envira_proof_order',
						'post_status'  => 'proofing-pending',
						'post_content' => $notes,
					]
				);

				// add custom meta.
				add_post_meta( $proofing_order_id, 'proofing_name', $name );
				add_post_meta( $proofing_order_id, 'proofing_email', $email );
				add_post_meta( $proofing_order_id, 'proofing_notes', $notes );
				add_post_meta( $proofing_order_id, 'proofing_images', [] );
				add_post_meta( $proofing_order_id, 'gallery_id', $gallery_id );

				// update the title to reflect order number?
				wp_update_post(
					[
						'ID'         => $proofing_order_id,
						'post_title' => 'Order #' . $proofing_order_id,
					]
				);

				$this->add_to_order_history( $proofing_order_id, $this->get_order_status_text( 'order-created', $proofing_order_id ), get_current_user_id() );

		} else {
			return false;
		}

		$order = $this->get_order_from_post_id( $proofing_order_id );

		// Return order ID.
		return $order;
	}

	/**
	 * Gets the order status text.
	 *
	 * @since 1.0
	 *
	 * @param   string $slug The action slug.
	 * @param   array  $proofing_order_id Proofing Order ID.
	 * @return  string
	 */
	public function get_order_status_text( $slug, $proofing_order_id = false ) {

		switch ( $slug ) {
			case 'order-created':
				$text = esc_html__( 'Order created and status set to "Pending"', 'envira-proofing' );
				break;

			case 'order-saved':
				$post = get_post( $proofing_order_id );
				$text = esc_html__( 'Order saved and status set to "', 'envira-proofing' ) . $post->post_status . esc_html__( '"', 'envira-proofing' );
				break;

			case 'order-submitted':
				$text = esc_html__( 'Order submitted by customer.', 'envira-proofing' );
				break;

			case 'order-active':
				$post = get_post( $proofing_order_id );
				$text = esc_html__( 'Order status set to "', 'envira-proofing' ) . $post->post_status . esc_html__( '"', 'envira-proofing' );
				break;

			case 'customer-email-resubmitted':
				$text = esc_html__( 'Customer email resubmmited.', 'envira-proofing' );
				break;

			case 'order-deleted-by-customer':
				$text = esc_html__( 'Customer deleted order.', 'envira-proofing' );
				break;

			case 'order-edited-by-customer':
				$text = esc_html__( 'Customer edited order.', 'envira-proofing' );
				break;

			case 'gallery-created':
				$text = esc_html__( 'Gallery created from order.', 'envira-proofing' );
				break;

			default:
				$text = 'No Text for ' . esc_html( $slug );
				break;
		}

		return apply_filters( 'envira_proofing_order_status_text', $text, $slug, $proofing_order_id );
	}

	/**
	 * Retrieves an order from the given Gallery ID
	 *
	 * @since 1.0
	 *
	 * @param   int    $gallery_id     Gallery ID.
	 * @param   array  $data           Gallery Config.
	 * @param   string $email          Email Address.
	 * @return  array                   Order
	 */
	public function get_order( $gallery_id, $data, $email ) {

		// Attempt to get the order by ID if there is no email or data.
		$order_id = ! empty( $_COOKIE['envira_proofing_order_id'] ) ? intval( $_COOKIE['envira_proofing_order_id'] ) : false;
		if ( $order_id ) {
			$order = $this->get_order_from_post_id( $order_id );
			return $order;
		}

		// Get orders.
		$proofing_orders = $this->get_orders( $gallery_id, $email, 1 );

		if ( false === $proofing_orders || 0 === $proofing_orders->found_posts ) {
			return false;
		}

		$order = $this->get_order_from_post_id( $proofing_orders->post->ID );

		return $order;
	}

	/**
	 * Get an order from a post (or gallery) ID.
	 *
	 * @since 2.0.0
	 *
	 * @param int $post_id Gallery or post ID.
	 * @return array
	 */
	public function get_order_from_post_id( $post_id = false ) {

		if ( empty( $post_id ) ) {
			return [];
		}

		$proofing_order = get_post( $post_id );

		if ( empty( $proofing_order ) || 'trash' === $proofing_order->post_status ) {
			return [];
		}

		$order = [
			'order_id'   => $proofing_order->ID,
			'name'       => get_post_meta( $proofing_order->ID, 'proofing_name', true ),
			'email'      => get_post_meta( $proofing_order->ID, 'proofing_email', true ),
			'images'     => get_post_meta( $proofing_order->ID, 'proofing_images', true ),
			'submitted'  => $this->is_order_submitted( $proofing_order->post_status, $proofing_order->ID ),
			'notes'      => get_post_meta( $proofing_order->ID, 'proofing_notes', true ),
			'gallery_id' => get_post_meta( $proofing_order->ID, 'gallery_id', true ),
			'status'     => $proofing_order->post_status,
		];

		return apply_filters( 'envira_proofing_order_data_from_post_id', $order, $post_id );
	}

	/**
	 * Defines what makes an order "submitted", from legacy.
	 *
	 * @since 2.0.0
	 *
	 * @param string $post_status Status slug.
	 * @param int    $proofing_order_id Proofing order ID.
	 * @return boolean
	 */
	public function is_order_submitted( $post_status = 'proofing-pending', $proofing_order_id = false ) {

		// check for post status first.
		if ( 'proofing-active' === $post_status ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Saves the given order and order notes against the given Gallery ID
	 *
	 * @since 2.0.0
	 *
	 * @param int    $gallery_id Gallery ID.
	 * @param array  $order Proofing order info.
	 * @param int    $proofing_order_id Proofing order ID.
	 * @param string $status Status slug.
	 */
	public function save_order( $gallery_id = false, $order = false, $proofing_order_id = false, $status = false ) {

		// status default value was 'proofing-pending'.
		if ( false === $gallery_id || false === $order ) {
			return;
		}

		$proofing_order_id = ( false === $proofing_order_id && ! empty( $order['order_id'] ) ) ? intval( $order['order_id'] ) : $proofing_order_id;

		update_post_meta( $proofing_order_id, 'proofing_name', $order['name'] );
		update_post_meta( $proofing_order_id, 'proofing_email', $order['email'] );
		update_post_meta( $proofing_order_id, 'proofing_notes', $order['notes'] );
		update_post_meta( $proofing_order_id, 'proofing_images', $order['images'] );
		update_post_meta( $proofing_order_id, 'gallery_id', $gallery_id );

		// if the post status was pending, but order is now submitted, the customer has submitted the order just now.

		// change the post status of the order if a status was provided.
		if ( false !== $status ) {
			wp_update_post(
				[
					'ID'          => $proofing_order_id,
					'post_status' => $status,
				]
			);
		}

		$this->add_to_order_history( $proofing_order_id, $this->get_order_status_text( 'order-saved', $proofing_order_id ), get_current_user_id() );

		// return order, but grab latest to be sure.
		$order = $this->get_order_from_post_id( $proofing_order_id );

		return $order;
	}

	/**
	 * Creates the given order and order notes against the given Gallery ID
	 *
	 * @since 2.0.0
	 *
	 * @param int    $gallery_id Gallery ID.
	 * @param array  $data Gallery data.
	 * @param string $email Order email, if any.
	 * @param string $name Order email, if any.
	 * @param string $notes Order email, if any.
	 */
	public function create_order( $gallery_id, $data, $email = false, $name = false, $notes = false ) {

		$proofing_order_id = wp_insert_post(
			[
				'post_title'   => 'order from gallery ' . $gallery_id,
				'post_type'    => 'envira_proof_order',
				'post_status'  => 'proofing-pending',
				'post_content' => $notes,
			]
		);

		// add custom meta.
		add_post_meta( $proofing_order_id, 'proofing_name', $name );
		add_post_meta( $proofing_order_id, 'proofing_email', $email );
		add_post_meta( $proofing_order_id, 'proofing_notes', $notes );
		add_post_meta( $proofing_order_id, 'proofing_images', [] );
		add_post_meta( $proofing_order_id, 'gallery_id', $gallery_id );
		add_post_meta( $proofing_order_id, 'user_id', get_current_user_id() );

		// update the title to reflect order number?
		wp_update_post(
			[
				'ID'         => $proofing_order_id,
				'post_title' => 'Order #' . $proofing_order_id,
			]
		);

		return $proofing_order_id;
	}

	/**
	 * Creates the given order and order notes against the given Gallery ID
	 *
	 * @since 2.0.0
	 *
	 * @param int    $order_id Order ID.
	 * @param int    $gallery_id Gallery ID.
	 * @param array  $data Gallery data.
	 * @param string $name Order email, if any.
	 * @param string $email Order email, if any.
	 * @param string $notes Order email, if any.
	 * @param array  $selected_images Data.
	 * @param array  $selected_images_ids Data.
	 * @param array  $selected_images_fields Data.
	 */
	public function update_order( $order_id, $gallery_id, $data, $name = '', $email = '', $notes = '', $selected_images = false, $selected_images_ids = false, $selected_images_fields = false ) {

		$order = $this->get_order_from_post_id( $order_id );

		$order['name']  = $name;
		$order['email'] = $email;
		$order['notes'] = $notes;

		// clear the order before adding to it.
		$order['images'] = [];

		if ( isset( $selected_images ) && isset( $selected_images_ids ) && is_array( $selected_images_ids ) ) {
			foreach ( $selected_images_ids as $image_id ) {
				$image_id = intval( $image_id );
				if ( ! $image_id ) {
					continue;
				}
				$order['images'][ $image_id ] = [];

				if ( ! empty( $selected_images_fields ) ) {
					foreach ( $selected_images_fields as $selected_images_field_id => $sizes_and_quantites ) {
						if ( intval( $image_id ) === $selected_images_field_id ) {
							if ( is_array( $sizes_and_quantites ) ) {
								foreach ( $sizes_and_quantites as $size => $quantity ) {
									$order['images'][ $image_id ][ $size ] = intval( $quantity );
								}
							} else {
								$order['images'][ $image_id ] = intval( $sizes_and_quantites );
							}
						}
					}
				}
			}
		}

		// save the order but do not alter it's status.
		$this->save_order( $gallery_id, $order, $order_id );

		// return the (updated) order.
		return $order;
	}

	/**
	 * Attempts to save an order id cookie.
	 *
	 * @since 2.0.0
	 *
	 * @param int $order_id Proofing order ID.
	 */
	public function save_cookie( $order_id ) {
		setcookie(
			'envira_proofing_order_id',
			$order_id,
			[
				'expires'  => time() + 2592000,
				'path'     => '/',
				'httponly' => true,
				'secure'   => true,
			]
		);
	}

	/**
	 * Attempts to unset an order id cookie.
	 *
	 * @param int $gallery_id Gallery ID.
	 */
	public function unset_cookie( $gallery_id = 0 ) {
		setcookie(
			'envira_proofing_order_id',
			'',
			[
				'expires'  => time() - 3600,
				'path'     => '/',
				'httponly' => true,
				'secure'   => true,
			]
		);
		setcookie(
			'envira_proof_image_ids_' . $gallery_id,
			'',
			[
				'expires'  => time() - 3600,
				'path'     => '/',
				'httponly' => true,
				'secure'   => true,
			]
		);
		setcookie(
			'envira_proof_images_' . $gallery_id,
			'',
			[
				'expires'  => time() - 3600,
				'path'     => '/',
				'httponly' => true,
				'secure'   => true,
			]
		);
	}

	/**
	 * Attempts to unset all order id cookies.
	 */
	public function unset_all_cookies() {
		$cookie_prefixes = [ 'envira_proof_image_ids_', 'envira_proof_images_', 'envira_proofing_order_id' ];
		$all_cookies     = $_COOKIE;

		foreach ( $all_cookies as $cookie_name => $cookie_value ) {
			foreach ( $cookie_prefixes as $prefix ) {
				if ( strpos( $cookie_name, $prefix ) === 0 ) {
					unset( $_COOKIE[ $cookie_name ] );
					setcookie(
						$cookie_name,
						'',
						[
							'expires'  => time() - 3600,
							'path'     => '/',
							'httponly' => true,
							'secure'   => true,
						]
					);
				}
			}
		}
	}

	/**
	 * Deletes cookies on post trash.
	 *
	 * @since 2.0.0
	 *
	 * @param int $post_id Proofing order ID.
	 */
	public function trash_envira_proof_order( $post_id ) {
		$gallery_id = (int) get_post_meta( $post_id, 'gallery_id', true );
		if ( empty( $gallery_id ) ) {
			return;
		}

		setcookie(
			'envira_proof_image_ids_' . $gallery_id,
			'',
			[
				'expires'  => time() - 3600,
				'path'     => '/',
				'httponly' => true,
				'secure'   => true,
			]
		);
		setcookie(
			'envira_proof_images_' . $gallery_id,
			'',
			[
				'expires'  => time() - 3600,
				'path'     => '/',
				'httponly' => true,
				'secure'   => true,
			]
		);
		$proofing_order_id = isset( $_COOKIE['envira_proofing_order_id'] ) ? absint( $_COOKIE['envira_proofing_order_id'] ) : 0;
		if ( $proofing_order_id && $post_id === $proofing_order_id ) {
			$this->unset_cookie();
		}
	}

	/**
	 * Submits a notification to the admin, likely an email (although this definition can be expanded)
	 *
	 * @since 2.0.0
	 *
	 * @param int    $proofing_order_id Proofing order ID.
	 * @param array  $order Proofing order info.
	 * @param string $slug Status slug.
	 */
	public function send_admin_notification( $proofing_order_id, $order, $slug = 'order-saved' ) {

		/* for testing purposes, this global will disable admin notifications */
		if ( defined( 'ENVIRA_PROOFING_DISABLE_ADMIN_NOTIFICATIONS' ) && true === filter_var( ENVIRA_PROOFING_DISABLE_ADMIN_NOTIFICATIONS, FILTER_VALIDATE_BOOLEAN ) ) {
			return false;
		}

		$proofing_email = new Email();
		$confirmation   = $proofing_email->send_admin_email( $proofing_order_id, $order, $slug );

		return $confirmation;
	}

	/**
	 * Submits a notification to the customer, likely an email (although this definition can be expanded)
	 *
	 * @since 2.0.0
	 *
	 * @param int    $proofing_order_id Proofing order ID.
	 * @param array  $order Proofing order info.
	 * @param string $slug Status slug.
	 */
	public function send_customer_notification( $proofing_order_id, $order, $slug = 'order-saved' ) {

		/* for testing purposes, this global will disable admin notifications */
		if ( defined( 'ENVIRA_PROOFING_DISABLE_CUSTOMER_NOTIFICATIONS' ) && true === filter_var( ENVIRA_PROOFING_DISABLE_CUSTOMER_NOTIFICATIONS, FILTER_VALIDATE_BOOLEAN ) ) {
			return false;
		}

		$proofing_email = new Email();
		$confirmation   = $proofing_email->send_customer_email( $proofing_order_id, $order, $slug );

		return $confirmation;
	}

	/**
	 * Add an action to order history.
	 *
	 * @since 2.0.0
	 *
	 * @param int    $gallery_id Gallery ID.
	 * @param string $message The message.
	 * @param int    $user_id User ID.
	 */
	public function add_to_order_history( $gallery_id, $message, $user_id = false ) {

		if ( empty( $gallery_id ) ) {
			return;
		}

		$history = get_post_meta( $gallery_id, '_envira_proofing_order_history', true );

		if ( empty( $history ) ) {
			$history = [];
		}

		// if there is no supplied user_id, but this was done by a user that was logged in, record that as the user id.
		if ( false === $user_id && is_user_logged_in() ) {
			$user    = wp_get_current_user();
			$user_id = $user->ID;
		}

		$args = [
			'message' => $message,
			'user_id' => $user_id,
		];

		$history[ time() ] = apply_filters( 'envira_proofing_order_message_to_add', $args, $gallery_id );

		$history = apply_filters( 'envira_proofing_order_message_history_update', $history, $gallery_id );

		update_post_meta( $gallery_id, '_envira_proofing_order_history', $history );

		return $history;
	}

	/**
	 * Reprocesses an email confirmation, usually requested by the admin on a order detail page.
	 *
	 * @since 2.0.0
	 */
	public function resend_email_confirmation() {

		// Check nonce is valid.
		if ( ! isset( $_GET['_wpnonce'] ) || ! wp_verify_nonce( sanitize_key( wp_unslash( $_GET['_wpnonce'] ) ), 'envira_gallery_proofing_resend_confirmation' ) ) {
			return;
		}

		$proofing_order_id = isset( $_GET['post'] ) ? intval( $_GET['post'] ) : false;
		$action            = isset( $_GET['resend_email'] ) ? intval( $_GET['resend_email'] ) : false;
		$customer_email    = get_post_meta( $proofing_order_id, 'proofing_email', true );

		if ( empty( $customer_email ) ) {
			wp_safe_redirect( admin_url( 'post.php?post=' . $proofing_order_id . '&action=edit' ) );
			exit;
		}

		$order = $this->get_order_from_post_id( $proofing_order_id );

		if ( ! empty( $order ) ) {

			$confirmation = $this->send_customer_notification( $proofing_order_id, $order, 'order-saved' );

			$notices = new Notices();

			// If there is no license key, output nag about ensuring key is set for automatic updates.
			if ( 1 === intval( $confirmation ) ) :

				$message = sprintf(
					/* translators: %s: URL */
					__( '<strong>Envira Proofing</strong>: The customer email confirmation has been resubmitted.', 'envira-proofing' )
				);

				$notices->display_inline_notice( 'customer-email-results', false, $message, 'warning', false, false, true, false );

				$this->add_to_order_history( $proofing_order_id, $this->get_order_status_text( 'customer-email-resubmitted', $proofing_order_id ), get_current_user_id() );

			endif;

		}

		// nonce envira-gallery-proofing-resend-confirmation.

		wp_safe_redirect( admin_url( 'post.php?post=' . $proofing_order_id . '&action=edit' ) );
		exit;
	}

	/**
	 * Create a gallery based on the order.
	 *
	 * @since 2.0.0
	 */
	public function create_gallery_from_order() {
		// Check nonce is valid.
		if ( ! isset( $_GET['_wpnonce'] ) || ! wp_verify_nonce( sanitize_key( wp_unslash( $_GET['_wpnonce'] ) ), 'envira_gallery_create_gallery_confirmation' ) ) {
			return;
		}

		$proofing_order_id = isset( $_GET['post'] ) ? intval( $_GET['post'] ) : false;
		$order             = $this->get_order_from_post_id( $proofing_order_id );
		$gallery           = isset( $order['gallery_id'] ) ? envira_get_gallery( $order['gallery_id'] ) : false;
		$gallery_post      = get_post( $order['gallery_id'] );
		$action            = isset( $_GET['create_gallery'] ) ? intval( $_GET['create_gallery'] ) : false;
		$customer_email    = get_post_meta( $proofing_order_id, 'proofing_email', true );
		$customer_name     = get_post_meta( $proofing_order_id, 'proofing_name', true );
		$gallery_name      = isset( $_GET['gallery_name'] ) ? sanitize_text_field( wp_unslash( $_GET['gallery_name'] ) ) : 'Gallery For ' . $gallery_post->post_title . ' (' . $customer_name . ')';

		if ( false === $action || empty( $order ) || false === $gallery || false === $gallery_post ) {
			wp_safe_redirect( admin_url( 'post.php?post=' . $proofing_order_id . '&action=edit' ) );
			exit;
		}

		if ( empty( $order['images'] ) ) {
			wp_safe_redirect( admin_url( 'post.php?post=' . $proofing_order_id . '&action=edit' ) );
			exit;
		}

		$data = [];

		// Create the gallery.
		$cpt_args = [
			'post_title'  => ( false !== $gallery_name ) ? $gallery_name : $gallery_post->post_title,
			'post_status' => 'draft',
			'post_type'   => 'envira',
			'post_author' => get_current_user_id(),
		];

		$envira_new_gallery_id = wp_insert_post( $cpt_args );

		// Add settings.
		$data['config'] = envira_get_config_defaults( $envira_new_gallery_id );

		// Add title and slug into settings.
		$data['config']['title'] = trim( wp_strip_all_tags( $gallery_name ) );
		$data['config']['slug']  = sanitize_text_field( $gallery_name );

		// Store post metadata.
		update_post_meta( $envira_new_gallery_id, '_eg_gallery_data', $data );

		// Add images - loop through the images and add them to the gallery.
		$in_gallery = [];
		foreach ( (array) $order['images'] as $image_id => $size_info ) {

			// Update the attachment image post meta first.
			$has_gallery = get_post_meta( $image_id, '_eg_has_gallery', true );
			if ( empty( $has_gallery ) ) {
				$has_gallery = [];
			}

			$has_gallery[] = $envira_new_gallery_id;
			update_post_meta( $image_id, '_eg_has_gallery', $has_gallery );

			// Now add the image to the gallery for this particular post.
			$in_gallery[] = $image_id;
			$data         = envira_gallery_ajax_prepare_gallery_data( $data, $image_id );
		}

		$order     = isset( $_POST['order'] ) ? sanitize_text_field( wp_unslash( $_POST['order'] ) ) : false;
		$direction = isset( $_POST['direction'] ) ? sanitize_text_field( wp_unslash( $_POST['direction'] ) ) : false;

		$gallery_data = envira_sort_gallery( $data, $data['config']['random'], $data['config']['sorting_direction'] );

		// Update the gallery data.
		update_post_meta( $envira_new_gallery_id, '_eg_in_gallery', $in_gallery );
		update_post_meta( $envira_new_gallery_id, '_eg_gallery_data', $gallery_data );

		// If there is no license key, output nag about ensuring key is set for automatic updates.
		if ( 0 !== intval( $envira_new_gallery_id ) ) :
			$this->add_to_order_history( $proofing_order_id, $this->get_order_status_text( 'gallery-created', $proofing_order_id ), get_current_user_id() );
		endif;

		wp_safe_redirect( admin_url( 'post.php?post=' . $proofing_order_id . '&action=edit&new_gallery=' . $envira_new_gallery_id ) );
		exit;
	}

	/**
	 * Determines how long we hold a cache for total orders. Usually a minute should be good enough for normal customers.
	 *
	 * @since 2.0.0
	 */
	public function get_transient_expiration_time() {
		return MINUTE_IN_SECONDS;
	}
}
