<?php
/**
 * Envira Proofing Process Order.
 *
 * @package Envira Gallery
 * @subpackage Envira Proofing
 */

namespace Envira\Proofing\Frontend;

use WP_Query;

/**
 * Process Order Class.
 */
class Process_Order {
	/**
	 * Holds the instance.
	 *
	 * @var null|Process_Order
	 */
	private static $instance = null;
	/**
	 * Holds messages.
	 *
	 * @var array Holds messages.
	 */
	public $messages = [];
	/**
	 * Holds the order object.
	 *
	 * @var null|Order
	 */
	public $order_object = null;

	/**
	 * If a new order is created during process the id will be stored here.
	 * This is useful since we cannot read a new order id from the cookies on the same request.
	 *
	 * @var int
	 */
	public $new_order_id = null;

	/**
	 * Process_Order constructor.
	 */
	private function __construct() {
		$this->order_object = new Order();
	}

	/**
	 * Process_Order get instance.
	 */
	public static function instance() {
		if ( null === self::$instance ) {
			self::$instance = new static();
		}
		return self::$instance;
	}

	/**
	 * Process the order.
	 */
	public function process() {
		$this->validate_order();

		$post_vars = self::get_post_vars();
		if ( empty( $post_vars ) ) {
			return;
		}

		$action = $post_vars['action'];

		switch ( $action ) {
			case 'save':
				// Save the order.
				self::save_order( $post_vars );
				break;
			case 'submit':
				// Submit the order.
				self::save_order( $post_vars, true );
				break;
			case 'delete':
				// Submit the order.
				self::delete_order( $post_vars );
				break;
			case 'email_submit':
				// Submit the order.
				self::email_submit( $post_vars );
				break;
			// no default.
		}
	}

	/**
	 * Validate the order.
	 *
	 * @return array
	 */
	private function get_post_vars() {
		// Don't run on the admin side.
		if ( is_admin() ) {
			return [];
		}

		$this->messages     = [];
		$this->new_order_id = null;

		// Check if a gallery ID was specified.
		if ( ! isset( $_POST['envira_gallery_id'], $_POST['envira_proofing_nonce'] ) ) {
			return [];
		}

		// Check nonce is valid.
		if ( ! wp_verify_nonce( sanitize_key( $_POST['envira_proofing_nonce'] ), 'envira_proofing_nonce' ) ) {
			return [];
		}

		// Get gallery.
		$gallery_id = absint( $_POST['envira_gallery_id'] );
		$data       = envira_get_gallery( $gallery_id );

		// If no gallery found, bail.
		if ( ! $data ) {
			return [];
		}

		// Check if Proofing is enabled.
		if ( ! envira_get_config( 'proofing', $data ) ) {
			return [];
		}

		// envira_proofing_save.
		$action = '';

		if ( isset( $_POST['envira_proofing_save'] ) ) {
			$action = 'save';
		}

		if ( isset( $_POST['envira_proofing_submit'] ) ) {
			$action = 'submit';
		}

		if ( isset( $_POST['envira_proofing_delete'] ) ) {
			$action = 'delete';
		}

		if ( isset( $_POST['envira_proofing_email_submit'] ) ) {
			$action = 'email_submit';
		}

		if ( empty( $action ) ) {
			$this->messages[] = [
				'type'    => 'error',
				'message' => __( 'No action specified.', 'envira-proofing' ),
			];

			return [];
		}

		if ( envira_get_config( 'proofing_multiple_enabled', $data ) ) {
			if ( ! isset( $_POST['envira_proofing_email'] ) || ! is_email( wp_unslash( $_POST['envira_proofing_email'] ) ) ) {
				$this->messages[] = [
					'type'    => 'error',
					'message' => __( 'Invalid email.', 'envira-proofing' ),
				];

				return [];
			}
		}

		$name  = isset( $_POST['envira_proofing_name'] ) ? sanitize_text_field( wp_unslash( $_POST['envira_proofing_name'] ) ) : '';
		$email = isset( $_POST['envira_proofing_email'] ) ? sanitize_text_field( wp_unslash( $_POST['envira_proofing_email'] ) ) : '';
		$notes = isset( $_POST['envira_proofing_notes'] ) ? sanitize_text_field( wp_unslash( $_POST['envira_proofing_notes'] ) ) : '';

		$selected_images        = ( ! empty( $_POST['envira_proofing_selected_images'] ) ?
		json_decode( ( sanitize_text_field( wp_unslash( $_POST['envira_proofing_selected_images'] ) ) ) ) : [] );
		$selected_images_ids    = ( ! empty( $_POST['envira_proofing_selected_images_ids'] ) ?
		json_decode( ( sanitize_text_field( wp_unslash( $_POST['envira_proofing_selected_images_ids'] ) ) ) ) : [] );
		$selected_images_fields = ( ! empty( $_POST['envira_proofing_selected_images_fields'] ) ?
		json_decode( ( sanitize_text_field( wp_unslash( $_POST['envira_proofing_selected_images_fields'] ) ) ) ) : [] );

		if ( ! empty( $_POST['envira_proofing'] ) ) {
			$selected_images_fields = [];
			$selected_images_ids    = [];
			$image_info             = isset( $_POST['envira_proofing']['images'] )
				? array_map( 'sanitize_text_field', wp_unslash( $_POST['envira_proofing']['images'] ) )
				: [];

			foreach ( $image_info as $image_id => $quantity ) {
				$selected_images_ids[] = $image_id;
				$selected_images[]     = wp_get_attachment_image_src( $image_id );
			}
			if ( ! empty( $_POST['envira_proofing']['quantities'] ) && is_array( $_POST['envira_proofing']['quantities'] ) ) {
				// phpcs:ignore WordPress.Security.ValidatedSanitizedInput
				foreach ( $_POST['envira_proofing']['quantities'] as $image_id => $size_and_quantities ) {
					if ( empty( $size_and_quantities ) ) {
						continue;
					}
					if ( is_array( $size_and_quantities ) ) {
						foreach ( $size_and_quantities as $size => $quantity ) {
							if ( ! is_string( $size ) || ! is_numeric( $quantity ) ) {
								continue;
							}
							$selected_images_fields[ $image_id ][ $size ] = (int) $quantity;
						}
					} else {
						$selected_images_fields[ $image_id ] = (int) $size_and_quantities;
					}
				}
			}
		}

		$order_id = isset( $_COOKIE['envira_proofing_order_id'] ) ? absint( $_COOKIE['envira_proofing_order_id'] ) : 0;

		return [
			'gallery_id'             => $gallery_id,
			'data'                   => $data,
			'action'                 => $action,
			'order_id'               => $order_id,
			'email'                  => $email,
			'name'                   => $name,
			'selected_images'        => $selected_images,
			'selected_images_ids'    => $selected_images_ids,
			'selected_images_fields' => $selected_images_fields,
			'notes'                  => wp_strip_all_tags( $notes ),
		];
	}

	/**
	 * Save the order.
	 *
	 * @param array $post_vars Post vars.
	 * @param bool  $submit    Submit or save.
	 */
	private function save_order( $post_vars, $submit = false ) {
		if ( $post_vars['order_id'] ) {
			$order = $this->order_object->get_order_from_post_id( $post_vars['order_id'] );
			if ( ! empty( $order ) ) {
				if ( envira_get_config( 'proofing_multiple_enabled', $post_vars['data'] ) ) {
					if ( $order['email'] === $post_vars['email'] ) {
						self::save_or_submit_order( $order, $post_vars, $submit );
					} else {
						// Try to get order that matches email and gallery id.
						$orders = new WP_Query(
							[
								'post_type'      => 'envira_proof_order',
								'post_status'    => [ 'proofing-active', 'proofing-pending', 'proofing-complete' ],
								'posts_per_page' => -1,
								// phpcs:ignore WordPress.DB.SlowDBQuery.slow_db_query_meta_query
								'meta_query'     => [
									'relation' => 'AND',
									[
										'key'   => 'gallery_id',
										'value' => $post_vars['gallery_id'],
									],
									[
										'key'   => 'proofing_email',
										'value' => $post_vars['email'],
									],
								],
							]
						);

						if ( $orders->found_posts > 1 ) {
							$this->messages[] = [
								'type'    => 'error',
								'message' => __( 'Multiple orders matching email and gallery are present, contact site administrator.', 'envira-proofing' ),
							];

							return;
						}

						if ( $orders->have_posts() ) {
							$this->order_object->save_cookie( $orders->posts[0]->ID );
							$order_from_email = $this->order_object->get_order_from_post_id( $orders->posts[0]->ID );
							self::save_or_submit_order( $order_from_email, $post_vars, $submit );
						} else {
							// Create new order.
							self::create_order_set_cookie( $post_vars, $submit );
							$this->messages[] = [
								'type'    => 'success',
								'message' => $submit ? self::output_submit_message( $post_vars['gallery_id'] ) : self::output_save_message( $post_vars['gallery_id'] ),
							];
						}
					}
				} else {
					self::save_or_submit_order( $order, $post_vars, $submit );
				}
			} else {
				// Create new order.
				self::create_order_set_cookie( $post_vars, $submit );
				$this->messages[] = [
					'type'    => 'success',
					'message' => $submit ? self::output_submit_message( $post_vars['gallery_id'] ) : self::output_save_message( $post_vars['gallery_id'] ),
				];
			}
		} else {
			self::create_order_set_cookie( $post_vars, $submit );
			$this->messages[] = [
				'type'    => 'success',
				'message' => $submit ? self::output_submit_message( $post_vars['gallery_id'] ) : self::output_save_message( $post_vars['gallery_id'] ),
			];
		}
	}

	/**
	 * Create order and set cookie.
	 *
	 * @param array $order Order.
	 * @param array $post_vars Post vars.
	 * @param bool  $submit    Submit or save.
	 */
	private function save_or_submit_order( $order, $post_vars, $submit = false ) {
		$notification_type = $submit ? 'order-submitted' : 'order-saved';

		if ( 'proofing-pending' === $order['status'] ) {
			// update order.
			$this->order_object->update_order(
				$post_vars['order_id'],
				$post_vars['gallery_id'],
				$post_vars['data'],
				$post_vars['name'],
				$post_vars['email'],
				$post_vars['notes'],
				$post_vars['selected_images'],
				$post_vars['selected_images_ids'],
				$post_vars['selected_images_fields']
			);

			if ( $submit ) {
				wp_update_post(
					[
						'ID'          => $post_vars['order_id'],
						'post_status' => 'proofing-active',
					]
				);
			}

			$this->messages[] = [
				'type'    => 'success',
				'message' => $submit ? self::output_submit_message( $post_vars['gallery_id'] ) : self::output_save_message( $post_vars['gallery_id'] ),
			];

			$this->order_object->add_to_order_history(
				$order['order_id'],
				$this->order_object->get_order_status_text( 'order-saved', $order['order_id'] ),
				get_current_user_id()
			);

			$this->order_object->send_admin_notification( $post_vars['order_id'], $order, $notification_type );
			$this->order_object->send_customer_notification( $post_vars['order_id'], $order, $notification_type );
		} elseif ( $submit ) {
			if ( envira_get_config( 'proofing_edit', $post_vars['data'] ) ) {
				// update order.
				$this->order_object->update_order(
					$post_vars['order_id'],
					$post_vars['gallery_id'],
					$post_vars['data'],
					$post_vars['name'],
					$post_vars['email'],
					$post_vars['notes'],
					$post_vars['selected_images'],
					$post_vars['selected_images_ids'],
					$post_vars['selected_images_fields']
				);
				$this->messages[] = [
					'type'    => 'success',
					'message' => self::output_submit_message( $post_vars['gallery_id'] ),
				];

				$this->order_object->send_admin_notification( $post_vars['order_id'], $order, $notification_type );
				$this->order_object->send_customer_notification( $post_vars['order_id'], $order, $notification_type );

				$this->order_object->add_to_order_history(
					$order['order_id'],
					$this->order_object->get_order_status_text( 'order-edited-by-customer', $order['order_id'] ),
					get_current_user_id()
				);

			} else {
				$this->messages[] = [
					'type'    => 'error',
					'message' => __( 'Not allowed to save submitted order.', 'envira-proofing' ),
				];
			}
		} else {
			$this->messages[] = [
				'type'    => 'error',
				'message' => __( 'Not allowed to save submitted order.', 'envira-proofing' ),
			];
		}
	}

	/**
	 * Output the submit message on the screen
	 * - save the order
	 * - submit the order
	 * - edit (unlock) the order
	 * - delete the order
	 *
	 * @since 1.0
	 * @param int $gallery_id Gallery Data.
	 */
	public function output_submit_message( $gallery_id ) {
		$markup  = '<div class="envira-proofing-submit-message">';
		$markup .= envira_proofing_get_setting_from_gallery( 'proofing_submitted_message', $gallery_id )
							? envira_proofing_get_setting_from_gallery( 'proofing_submitted_message', $gallery_id )
							: envira_proofing_get_global_setting( 'global_proofing_submitted_message' );
		$markup  = wpautop( $markup );
		$markup .= '</div>';

		return wp_kses( $markup, envira_proofing_wp_kses_allowed_html() );
	}

	/**
	 * Output the save message on the screen
	 * - save the order
	 * - submit the order
	 * - edit (unlock) the order
	 * - delete the order
	 *
	 * @since 1.0
	 * @param int $gallery_id The markup.
	 */
	public function output_save_message( $gallery_id ) {
		$markup  = '<div class="envira-proofing-submit-message">';
		$markup .= envira_proofing_get_setting_from_gallery( 'proofing_saved_message', $gallery_id )
							? envira_proofing_get_setting_from_gallery( 'proofing_saved_message', $gallery_id )
							: envira_proofing_get_global_setting( 'global_proofing_saved_message' );
		$markup  = wpautop( $markup );
		$markup .= '</div>';

		return wp_kses( $markup, envira_proofing_wp_kses_allowed_html() );
	}

	/**
	 * Create order and set cookie.
	 *
	 * @param array $post_vars Post vars.
	 * @param bool  $submit    Submit or save.
	 */
	private function create_order_set_cookie( $post_vars, $submit = false ) {
		$notification_type = $submit ? 'order-submitted' : 'order-saved';

		$order_id = $this->order_object->create_order(
			$post_vars['gallery_id'],
			$post_vars['data'],
			$post_vars['email'],
			$post_vars['name'],
			$post_vars['notes']
		);

		$this->new_order_id = $order_id;

		$order = $this->order_object->update_order(
			$order_id,
			$post_vars['gallery_id'],
			$post_vars['data'],
			$post_vars['name'],
			$post_vars['email'],
			$post_vars['notes'],
			$post_vars['selected_images'],
			$post_vars['selected_images_ids'],
			$post_vars['selected_images_fields']
		);

		$this->order_object->save_cookie( $order_id );

		$this->order_object->send_admin_notification( $post_vars['order_id'], $order, $notification_type );
		$this->order_object->send_customer_notification( $post_vars['order_id'], $order, $notification_type );

		$this->order_object->add_to_order_history(
			$order_id,
			$this->order_object->get_order_status_text( 'order-created', $order['order_id'] ),
			get_current_user_id()
		);

		if ( $submit ) {
			wp_update_post(
				[
					'ID'          => $order_id,
					'post_status' => 'proofing-active',
				]
			);
			$this->order_object->add_to_order_history(
				$order_id,
				$this->order_object->get_order_status_text( 'order-submitted', $order['order_id'] ),
				get_current_user_id()
			);
		}
	}

	/**
	 * Delete order.
	 *
	 * @param array $post_vars Post vars.
	 */
	private function delete_order( $post_vars ) {
		$order = $this->order_object->get_order_from_post_id( $post_vars['order_id'] );
		if ( ! empty( $order ) ) {
			if ( ( 'proofing-pending' !== $order['status'] ) && ! envira_get_config( 'proofing_edit', $post_vars['data'] ) ) {
				$this->messages[] = [
					'type'    => 'error',
					'message' => __( 'Not allowed to delete submitted order.', 'envira-proofing' ),
				];
				return;
			}

			$trash_forever = apply_filters( 'envira_proofing_delete_order_forever', false );

			$this->order_object->add_to_order_history(
				$order['order_id'],
				$this->order_object->get_order_status_text( 'order-deleted-by-customer', $order['order_id'] ),
				get_current_user_id()
			);

			wp_delete_post( $order['order_id'], $trash_forever );
			// Ensure the cookie is deleted.
		}

		// Redirect to delete cookies for the next request.
		// phpcs:ignore WordPress.Security.NonceVerification.Recommended, WordPress.Security.ValidatedSanitizedInput.InputNotValidated
		wp_safe_redirect( site_url( sanitize_text_field( wp_unslash( $_REQUEST['_wp_http_referer'] ) ) ) );
	}

	/**
	 * Email submit.
	 *
	 * @param array $post_vars Post vars.
	 */
	private function email_submit( $post_vars ) {
		if ( ! is_email( $post_vars['email'] ) ) {
			$this->messages[] = [
				'type'    => 'error',
				'message' => __( 'Please enter a valid email address', 'envira-proofing' ),
			];
		}

		$orders = $this->order_object->get_orders( $post_vars['gallery_id'], $post_vars['email'] );
		if ( $orders->found_posts > 1 ) {
			$this->messages[] = [
				'type'    => 'error',
				'message' => __( 'Multiple orders matching email and gallery are present, contact site administrator.', 'envira-proofing' ),
			];

			return;
		}

		if ( $orders->have_posts() ) {
			$order = $orders->posts[0];
			$this->order_object->save_cookie( $order->ID );
		} else {
			// Create new order.
			self::create_order_set_cookie( $post_vars );
		}

		// Redirect to set cookies for the next request.
		// phpcs:ignore WordPress.Security.NonceVerification.Recommended, WordPress.Security.ValidatedSanitizedInput.InputNotValidated
		wp_safe_redirect( site_url( sanitize_text_field( wp_unslash( $_REQUEST['_wp_http_referer'] ) ) ) );
	}

	/**
	 * Validate order in cookie
	 */
	private function validate_order() {
		$order_id = isset( $_COOKIE['envira_proofing_order_id'] ) ? (int) $_COOKIE['envira_proofing_order_id'] : 0;
		if ( $order_id > 0 ) {
			$status = get_post_status( $order_id );
			if ( false === $status ) {
				global $wp;

				$this->order_object->unset_all_cookies();

				// phpcs:ignore WordPress.Security.NonceVerification.Recommended, WordPress.Security.ValidatedSanitizedInput.InputNotValidated
				wp_safe_redirect( site_url( add_query_arg( $_GET, $wp->request ) ) );
			}
		}
	}
}
