<?php
/**
 * Shortcode class.
 *
 * @since 1.6.0
 *
 * @package Envira Gallery
 * @subpackage Envira Proofing
 * @author Envira Gallery Team <support@enviragallery.com>
 */

namespace Envira\Proofing\Frontend;

/**
 * Shortcode class.
 *
 * @since 1.0.0
 *
 * @package Envira_Proofing
 * @author  Envira Team
 */
class Shortcode {
	/**
	 * Holds the order object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public $order = [];

	/**
	 * Holds the current order id.
	 *
	 * @var int
	 */
	public $current_order_id = 0;

	/**
	 * Primary class constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		$process_order = Process_Order::instance();
		add_action( 'wp', [ $process_order, 'process' ] );

		// Register scripts.
		$this->register_scripts();

		// Filter gallery data.
		add_filter( 'envira_gallery_pre_data', [ $this, 'pre_data' ], 10, 3 );

		// Disable cache for proofing gallery.
		add_filter( 'envira_gallery_should_cache', [ $this, 'should_cache' ], 10, 2 );

		add_filter( 'envira_gallery_container', [ $this, 'gallery_container' ], 10, 2 );

		// Gallery or Album container class names.
		add_filter( 'envira_gallery_output_extra_css', [ $this, 'extra_css' ], 10, 2 );

		// Add markup at the beginning of the container.
		add_filter( 'envira_gallery_output_before_container', [ $this, 'before_container' ], 10, 2 );

		// Add select image's checkbox.
		add_filter( 'envira_gallery_output_item_container_classes', [ $this, 'item_container_classes' ], 10, 3 );

		add_filter( 'envira_gallery_output_after_link', [ $this, 'after_link' ], 10, 4 );

		add_filter( 'envira_gallery_output_after_container', [ $this, 'after_container' ], 10, 2 );
	}

	/**
	 * Enqueue scripts. The addon is hooked into 'init' so no need to register scripts in a hook.
	 */
	public function register_scripts() {
		$version = is_envira_debug_on() ? time() . '-' . ENVIRA_PROOFING_VERSION : ENVIRA_PROOFING_VERSION;

		// Register CSS.
		wp_register_style( ENVIRA_PROOFING_SLUG . '-style', plugins_url( 'assets/css/proofing-style.css', ENVIRA_PROOFING_FILE ), [], $version );

		// Register JS.
		wp_register_script( ENVIRA_PROOFING_SLUG . '-cookie', envira_script( 'assets/js/min/envira-js-cookie-min.js', ENVIRA_PROOFING_FILE ), [ 'jquery' ], $version, true );
		wp_register_script( ENVIRA_PROOFING_SLUG . '-script', envira_script( 'assets/js/min/envira-proofing-min.js', ENVIRA_PROOFING_FILE ), [ 'jquery' ], $version, true );

		// Enqueue scripts.
		add_action( 'envira_gallery_before_output', [ $this, 'enqueue_scripts' ] );
		add_action( 'envira_link_before_output', [ $this, 'enqueue_scripts' ] );
	}

	/**
	 * Enqueue CSS and JS if Proofing is enabled
	 *
	 * @since 1.0.0
	 *
	 * @param array $data Gallery Data.
	 */
	public function enqueue_scripts( $data ) {
		// Check if Proofing is enabled.
		if ( ! envira_get_config( 'proofing', $data ) ) {
			return;
		}

		// Enqueue CSS + JS.
		wp_enqueue_style( ENVIRA_PROOFING_SLUG . '-style' );
		wp_enqueue_script( ENVIRA_PROOFING_SLUG . '-cookie' );
		wp_enqueue_script( ENVIRA_PROOFING_SLUG . '-script' );
	}

	/**
	 * Disables cache on proofing.
	 *
	 * @since 1.0.2
	 *
	 * @param string $should_cache Cached.
	 * @param array  $data Gallery Data.
	 */
	public function should_cache( $should_cache, $data ) {
		if ( envira_get_config( 'proofing', $data ) ) {
			return false;
		}
		return $should_cache;
	}

	/**
	 * Add CSS Class
	 *
	 * @since 1.0.2
	 *
	 * @param string $css CSS.
	 * @return string
	 */
	public function extra_css( $css ) {
		if ( $this->current_order_id ) {
			$css .= ' envira-proofing-yes-order ';
		} else {
			$css .= ' envira-proofing-no-order ';
		}
		return $css;
	}

	/**
	 * Iterates through each image in the gallery, defining its orientation
	 *
	 * @param array $data Gallery Data.
	 * @param int   $gallery_id Gallery ID.
	 * @param null  $should_cache_default Should Cache Default.
	 *
	 * @return array $data Gallery Data
	 */
	public function pre_data( $data, $gallery_id, $should_cache_default = null ) {

		// Check if Proofing is enabled.
		if ( ! envira_get_config( 'proofing', $data ) ) {
			return $data;
		}

		// Check images exist.
		if ( ! isset( $data['gallery'] ) || count( $data['gallery'] ) === 0 ) {
			return $data;
		}

		$this->current_order_id = isset( $_COOKIE['envira_proofing_order_id'] )
			? absint( $_COOKIE['envira_proofing_order_id'] )
			: Process_Order::instance()->new_order_id; // If there is no cookie then maybe there is an order recently created.

		if ( $this->current_order_id ) {
			$this->order = Process_Order::instance()->order_object->get_order_from_post_id( $this->current_order_id );
			if ( empty( $this->order ) ) {
				$this->current_order_id = 0;
			}
		}

		if ( null === $should_cache_default ) {
			$should_cache_default = false;
		}

		$should_cache = apply_filters( 'envira_gallery_should_cache', $should_cache_default, $data );

		foreach ( $data['gallery'] as $index => $image ) {
			if ( isset( $data['gallery'][ $index ]['landscape'] ) && isset( $data['gallery'][ $index ]['portrait'] ) && $should_cache ) {
				continue;
			}
			// Get image size and assume it's neither landscape nor portrait.
			$data['gallery'][ $index ]['landscape'] = false;
			$data['gallery'][ $index ]['portrait']  = false;

			$path = wp_get_original_image_path( $index );

			if ( ! $path ) {
				// Get attachment path via $image['src'].
				$path = attachment_url_to_postid( $image['src'] );
			}

			if ( ! $path ) {
				// Get attachment path via $image['src'].
				$path = str_replace( content_url(), WP_CONTENT_DIR, $image['src'] );
			}

			if ( ! file_exists( $path ) ) {
				continue;
			}

			$size = getimagesize( $path );

			if ( ! $size ) {
				continue;
			}

			// If width > height = landscape.
			if ( $size[0] > $size[1] ) {
				$data['gallery'][ $index ]['landscape'] = true;
			} else {
				$data['gallery'][ $index ]['portrait'] = true;
			}
		}

		return $data;
	}

	/**
	 * Prepends a form tag to a gallery, when Proofing is enabled
	 * Outputs any messages set in the class' $message object
	 *
	 * @since 1.0.0
	 *
	 * @param string $html HTML.
	 * @param array  $data Data.
	 * @return string HTML
	 */
	public function before_container( $html, $data ) {
		// Check if Proofing is enabled.
		if ( ! envira_get_config( 'proofing', $data ) ) {
			return $html;
		}

		$append_html = '';
		$messages    = Process_Order::instance()->messages;

		if ( empty( $messages ) && $this->current_order_id && $this->order['submitted'] && ! envira_get_config( 'proofing_edit', $data ) ) {
			$messages[] = [
				'type'    => 'success',
				'message' => __( 'Your order has been submitted.', 'envira-proofing' ),
			];
		}

		// Maybe output success/error messages, if any have been set.
		if ( ! empty( $messages ) ) {
			$append_html .= $this->get_messages( $messages );
		}

		$form_action_url = isset( $_SERVER['REQUEST_URI'] ) ? esc_url( sanitize_text_field( wp_unslash( $_SERVER['REQUEST_URI'] ) ) ) : '';

		// Open form.
		$append_html .= "<form action=\"$form_action_url\" method=\"post\" class=\"envira-proofing-form\" data-envira-id=\"{$data['id']}\">";

		// If proofing_email_box_position === 'below' add fields here.
		if ( ! envira_get_config( 'proofing_multiple_enabled', $data ) || 'below' !== envira_get_config( 'proofing_email_box_position', $data ) ) {
			$append_html .= $this->fields( $data );
		}

		// Return.
		return $html . $append_html;
	}

	/**
	 * Get error/success messages for the gallery.
	 *
	 * @param array $messages Messages.
	 *
	 * @return string
	 */
	private function get_messages( $messages ) {
		$html = '';
		if ( ! empty( $messages ) ) {
			$html .= '<div id="envira-proofing-messages">';
			foreach ( $messages as $message ) {
				$html .= '<div class="envira-proofing-message-' . $message['type'] . '">' . $message['message'] . '</div>';
			}
			$html .= '</div>';
		}

		return $html;
	}

	/**
	 * Appends fields to the proofing form
	 *
	 * @param array $data Data.
	 * @return string HTML
	 */
	public function fields( $data ) {
		$html = '';

		$notes = $this->current_order_id ? $this->order['notes'] : '';
		$name  = $this->current_order_id ? $this->order['name'] : '';
		$email = $this->current_order_id ? $this->order['email'] : '';

		$disabled    = $this->disabled( $data ) ? ' disabled' : '';
		$placeholder = esc_attr( envira_get_config( 'proofing_notes_placeholder_text', $data ) );

		// Add notes field.
		$html .= "<textarea name=\"envira_proofing_notes\" placeholder=\"$placeholder\"$disabled>";
		$html .= $notes;
		$html .= '</textarea>';

		// Add nonce field.
		$html .= wp_nonce_field( 'envira_proofing_nonce', 'envira_proofing_nonce', true, false );

		// Add selected image id field.
		$html .= '<input type="hidden" id="envira_proofing_selected_images" name="envira_proofing_selected_images" value="">';
		$html .= '<input type="hidden" id="envira_proofing_selected_images_ids" name="envira_proofing_selected_images_ids" value="">';
		$html .= '<input type="hidden" id="envira_proofing_selected_images_fields" name="envira_proofing_selected_images_fields" value="">';
		$html .= "<input type=\"hidden\" name=\"envira_gallery_id\" value=\"{$data['id']}\" />";

		// Require email address === proofing_multiple_enabled.
		if ( envira_get_config( 'proofing_multiple_enabled', $data ) ) {
			$name_placeholder  = __( 'Name', 'envira-proofing' );
			$email_placeholder = __( 'Email Address', 'envira-proofing' );

			if ( envira_get_config( 'proofing_name_enabled', $data ) ) {
				$html .= "<input type=\"text\" name=\"envira_proofing_name\" value=\"$name\" placeholder=\"$name_placeholder\"$disabled\">";
			}

			$html .= "<input type=\"email\" name=\"envira_proofing_email\" value=\"$email\" placeholder=\"$email_placeholder\"$disabled\">";
		}

		$save_button   = '<input type="submit" name="envira_proofing_save" class="envira_proofing_save" value="' . esc_attr( envira_get_config( 'proofing_save_notes_button_label', $data ) ) . '" />';
		$submit_button = '<input type="submit" name="envira_proofing_submit" class="envira_proofing_submit" value="' . esc_attr( envira_get_config( 'proofing_submit_button_label', $data ) ) . '" />';
		$edit_button   = '<input type="submit" name="envira_proofing_submit" class="envira_proofing_submit" value="' . esc_attr( envira_get_config( 'proofing_edit_button_label', $data ) ) . '" />';

		$html .= '<div class="envira-proofing-buttons">';
		// Buttons.
		if ( empty( $this->order ) || ( 'proofing-active' !== $this->order['status'] ) ) {
			$html .= $submit_button;
			$html .= $save_button;
		} elseif ( ( 'proofing-active' === $this->order['status'] ) && envira_get_config( 'proofing_edit', $data ) ) {
				$html .= $edit_button;
		}

		if ( envira_get_config( 'proofing_delete', $data ) ) {
			$html .= '<input type="submit" name="envira_proofing_delete" class="envira_proofing_delete" value="' . esc_attr( envira_get_config( 'proofing_delete_button_label', $data ) ) . '" />';
		}

		$html .= '</div>';

		return $html;
	}

	/**
	 * Determines if fields should be disabled.
	 *
	 * @param array $data Gallery data.
	 *
	 * @return bool
	 */
	private function disabled( $data ) {
		if ( empty( $this->order ) ) {
			return false;
		}

		if ( envira_get_config( 'proofing_edit', $data ) && ( 'proofing-active' === $this->order['status'] ) ) {
			return false;
		}

		if ( 'proofing-pending' === $this->order['status'] ) {
			return false;
		}

		return true;
	}

	/**
	 * Item container classes
	 *
	 * @param array   $classes Classes array.
	 * @param numeric $id Id.
	 * @param array   $data Gallery Data.
	 *
	 * @return string[]
	 */
	public function item_container_classes( $classes, $id, $data ) {
		if ( envira_get_config( 'proofing', $data ) && isset( $this->order['images'][ $id ] ) ) {
			$classes[] = 'envira-proofing-selected';
		}
		return $classes;
	}

	/**
	 * Appends a checkbox and other fields (if applicable) to the end of each image
	 *
	 * @since 1.0.0
	 *
	 * @param string $output Output.
	 * @param int    $id ID.
	 * @param array  $item Item data.
	 * @param array  $data Gallery data.
	 * @return string Amended Image HTML
	 */
	public function after_link( $output, $id, $item, $data ) {

		// Check if Proofing is enabled.
		if ( ! envira_get_config( 'proofing', $data ) ) {
			return $output;
		}

		// Append quantity / size options.
		$quantity_enabled = envira_get_config( 'proofing_quantity_enabled', $data );
		$sizes_enabled    = envira_get_config( 'proofing_size_enabled', $data );
		$sizes            = envira_get_config( 'proofing_sizes', $data );
		$valid_sizes      = ( is_array( $sizes ) && count( $sizes ) > 0 ) && $sizes_enabled;

		// If the order has already been submitted, disable the form field inputs.
		$disabled = $this->disabled( $data ) ? ' disabled' : '';

		// Append checkbox.
		$padding          = absint( envira_get_config( 'gutter', $data ) );
		$checkbox_id      = "envira_proofing_images_$id";
		$checkbox_name    = "envira_proofing[images][$id]";
		$checkbox_style   = "top:{$padding}px;left:{$padding}px";
		$checkbox_checked = isset( $this->order['images'][ $id ] ) ? ' checked' : '';

		$output .= "<input type=\"checkbox\" id=\"$checkbox_id\" name=\"$checkbox_name\" value=\"1\" style=\"$checkbox_style\" class=\"envira-proofing-select-image\"$checkbox_checked$disabled />";

		$output .= "<label for=\"$checkbox_id\" class=\"envira-proofing-select-image\"></label>";

		if ( ! $quantity_enabled && ! $valid_sizes ) {
			return $output;
		}

		$output .= '<div class="envira-proofing-fields">';

		if ( $quantity_enabled && $valid_sizes ) {
			// Output sizes with quantity beside each.
			foreach ( $sizes as $size ) {
				// If the size has an orientation defined, only show this size if the image matches the given orientation.
				$orientation = ( isset( $size['orientation'] ) ? $size['orientation'] : '' );
				if ( empty( $orientation ) || ( 'landscape' === $orientation && $item['landscape'] ) || ( 'portrait' === $orientation && $item['portrait'] ) ) {
					// No orientation defined for this size, or orientation is defined and matches the image.
					$output .= '<div class="envira-proofing-field">
                        <label for="envira_proofing_quantity_' . $id . '_' . $size['slug'] . '">' . esc_attr( $size['name'] ) . '</label>
                        <input type="number" name="envira_proofing[quantities][' . $id . '][' . $size['name'] . ']" id="envira_proofing_quantity_' . $id . '_' . $size['slug'] . '" min="0" value="' . ( isset( $this->order['images'][ $id ][ $size['name'] ] ) ? $this->order['images'][ $id ][ $size['name'] ] : 0 ) . '" class="envira-proofing-number"' . $disabled . ' />
                    	</div>';
				}
			}
		} elseif ( $quantity_enabled && ! $valid_sizes ) {
			// Output quantity field only.
			$output .= '<div class="envira-proofing-field">
	                	<label for="envira_proofing_quantity_' . $id . '">' . __( 'Quantity', 'envira-proofing' ) . '</label>
	                	<input type="number" name="envira_proofing[quantities][' . $id . ']" id="envira_proofing_quantity_' . $id . '" min="0" value="' . esc_attr( ( isset( $this->order['images'][ $id ] ) ? $this->order['images'][ $id ] : 1 ) ) . '" class="envira-proofing-number"' . $disabled . ' />
	            		</div>';

		} else {
			// Output sizes only.
			foreach ( $sizes as $size ) {
				// If the size has an orientation defined, only show this size if the image matches the given orientation.
				$orientation = ( isset( $size['orientation'] ) ? $size['orientation'] : '' );
				if ( empty( $orientation ) || ( 'landscape' === $orientation && $item['landscape'] ) || ( 'portrait' === $orientation && $item['portrait'] ) ) {
					// No orientation defined for this size, or orientation is defined and matches the image.
					$output .= '<div class="envira-proofing-field">
                        <input type="checkbox" name="envira_proofing[quantities][' . $id . '][' . $size['name'] . ']" id="envira_proofing_quantity_' . $id . '_' . $size['slug'] . '" value="1" class="envira-proofing-checkbox" ' . ( isset( $this->order['images'][ $id ][ $size['name'] ] ) ? ' checked' : '' ) . $disabled . ' />
                        <label class="envira-proofing-checkbox-label" for="envira_proofing_quantity_' . $id . '_' . $size['slug'] . '">' . esc_html( $size['name'] ) . '</label>
                    </div>';
				}
			}
		}

		// Close .envira-proofing-fields.
		$output .= '</div>';

		return $output;
	}

	/**
	 * Appends a closing form tag to a gallery, when Proofing is enabled
	 *
	 * @since 1.0.0
	 *
	 * @param string $html HTML.
	 * @param array  $data Data.
	 * @return string HTML
	 */
	public function after_container( $html, $data ) {
		// Check if Proofing is enabled.
		if ( ! envira_get_config( 'proofing', $data ) ) {
			return $html;
		}

		$save_button   = '<button name="envira_proofing_save">' . esc_attr( envira_get_config( 'proofing_save_button_label', $data ) ) . '</button>';
		$submit_button = '<button name="envira_proofing_submit">' . esc_attr( envira_get_config( 'proofing_submit_button_label', $data ) ) . '</button>';
		$edit_button   = '<button name="envira_proofing_submit">' . esc_attr( envira_get_config( 'proofing_edit_button_label', $data ) ) . '</button>';

		// Summary Box.
		$summary_box  = "<div class=\"envira-proofing-summary-box\" data-envira-id=\"{$data['id']}\">";
		$summary_box .= '<div class="envira-proofing-summary-box-inner">';
		$summary_box .= '<div class="images"><div class="images-inner"></div></div>';
		$summary_box .= '<div class="buttons">';

		if ( empty( $this->order ) || ( 'proofing-active' !== $this->order['status'] ) ) {
			$summary_box .= $submit_button;
			$summary_box .= $save_button;
		} elseif ( ( 'proofing-active' === $this->order['status'] ) && envira_get_config( 'proofing_edit', $data ) ) {
				$summary_box .= $edit_button;
		}

		$summary_box .= '</div>';
		$summary_box .= '</div>';
		$summary_box .= '</div>';

		$fields = '';

		if ( envira_get_config( 'proofing_multiple_enabled', $data ) && 'below' === envira_get_config( 'proofing_email_box_position', $data ) ) {
			$fields .= $this->fields( $data );
		}

		return $html . $fields . $summary_box . '</form>';
	}

	/**
	 * Replaces the gallery for a form when Proofing is enabled and the hide gallery option is enabled.
	 *
	 * @param string $container HTML.
	 * @param array  $data Data.
	 *
	 * @return string
	 */
	public function gallery_container( $container, $data ) {
		// Check if Proofing is enabled.
		if ( ! envira_get_config( 'proofing', $data ) ) {
			return $container;
		}

		if ( ! ( envira_get_config( 'proofing_multiple_enabled', $data ) && envira_get_config( 'proofing_hide_gallery', $data ) && empty( $this->order ) ) ) {
			return $container;
		}

		$email_placeholder = __( 'Email Address', 'envira-proofing' );
		$form_action_url   = isset( $_SERVER['REQUEST_URI'] ) ? esc_url( sanitize_text_field( wp_unslash( $_SERVER['REQUEST_URI'] ) ) ) : '';

		$messages = Process_Order::instance()->messages;

		$html  = "<form action=\"$form_action_url\" method=\"post\" class=\"envira-proofing-form\" data-envira-id=\"{$data['id']}\">";
		$html .= '<p>' . __( 'Enter email to view gallery', 'envira-proofing' ) . '</p>';
		if ( ! empty( $messages ) ) {
			$html .= $this->get_messages( $messages );
		}
		$html .= '<div>';
		$html .= "<input type=\"email\" name=\"envira_proofing_email\" value=\"\" placeholder=\"$email_placeholder\">";
		$html .= wp_nonce_field( 'envira_proofing_nonce', 'envira_proofing_nonce', true, false );
		$html .= "<input type=\"hidden\" name=\"envira_gallery_id\" value=\"{$data['id']}\" />";
		$html .= '</div>';
		$html .= '<input type="submit" name="envira_proofing_email_submit" class="envira_proofing_submit" value="' . esc_attr(
			__(
				'Submit',
				'envira-proofing'
			)
		) . '" />';
		$html .= '</form>';

		remove_filter( 'envira_gallery_output_before_container', [ $this, 'before_container' ] );
		remove_filter( 'envira_gallery_output_after_link', [ $this, 'after_link' ] );
		remove_filter( 'envira_gallery_output_after_container', [ $this, 'after_container' ] );

		add_filter( 'envira_gallery_output_loader', [ $this, 'output_loader' ] );

		return $html;
	}

	/**
	 * Removes the output loader.
	 *
	 * @param string $markup HTML.
	 */
	public function output_loader( $markup ) {
		return str_replace( '<div class="envira-loader"><div></div><div></div><div></div><div></div></div>', '', $markup );
	}
}
