<?php
/**
 * AJAX class.
 *
 * @since 1.0.0
 *
 * @package Envira_Watermarking
 * @author  Envira Team
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * AJAX class.
 *
 * @since 1.0.0
 *
 * @package Envira_Watermarking
 * @author  Envira Team
 */
class Envira_Watermarking_Ajax {

	/**
	 * Holds the class object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public static $instance;

	/**
	 * Path to the file.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	public $file = __FILE__;

	/**
	 * Holds the base class object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public $base;

	/**
	 * Primary class constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		add_action( 'envira_gallery_ajax_load_image', [ $this, 'ajax_load_image' ], 11, 2 );
		// add ajax action to generate watermarking preview.
		add_action( 'wp_ajax_envira_watermarking_preview', [ $this, 'watermarking_preview' ] );
	}

	/**
	 * AJAX callback to generate watermarking preview.
	 */
	public function watermarking_preview() {
		check_ajax_referer( 'envira_watermarking_save_settings', 'security' );

		if ( ! current_user_can( 'edit_posts' ) ) {
			wp_send_json_error();
		}

		if ( ! isset(
			$_POST['type'],
			$_POST['watermarkAsData'],
			$_POST['watermarkID'],
			$_POST['firstImageId'],
			$_POST['scale'],
			$_POST['position'],
			$_POST['rotation'],
			$_POST['repeat'],
			$_POST['margin']
		) ) {
			wp_send_json_error( 'Invalid Request', 400 );
		}

		$first_image_id    = (int) sanitize_key( wp_unslash( $_POST['firstImageId'] ) );
		$watermark_as_data = sanitize_text_field( wp_unslash( $_POST['watermarkAsData'] ) );
		$watermark_id      = (int) sanitize_key( wp_unslash( $_POST['watermarkID'] ) );
		$watermark_path    = '';
		$settings          = [
			'type'     => sanitize_text_field( wp_unslash( $_POST['type'] ) ),
			'scale'    => (int) sanitize_key( wp_unslash( $_POST['scale'] ) ),
			'position' => sanitize_text_field( wp_unslash( $_POST['position'] ) ),
			'rotation' => (int) sanitize_key( wp_unslash( $_POST['rotation'] ) ),
			'repeat'   => (bool) sanitize_key( wp_unslash( $_POST['repeat'] ) ),
			'margin'   => (int) sanitize_key( wp_unslash( $_POST['margin'] ) ),
		];

		if (
			! in_array( $settings['type'], [ 'image', 'text' ], true )
			|| ! in_array( $settings['position'], [ 'top-left', 'top-right', 'center', 'bottom-left', 'bottom-right' ], true )
		) {
			wp_send_json_error( 'Invalid Request', 400 );
		}

		if ( 'image' === $settings['type'] ) {
			$watermark_image_string = file_get_contents( get_attached_file( $watermark_id ) );
		} else {
			$image_data             = str_replace( 'data:image/png;base64,', '', $watermark_as_data );
			$watermark_image_string = base64_decode( $image_data );
		}

		$image_for_preview = $this->image_for_preview( $first_image_id );

		try {
			$watermark_path = Envira_Watermarking_Process::watermark_it( $image_for_preview, $watermark_image_string, $settings );
		} catch ( Exception $exception ) {
			wp_send_json_error( $exception->getMessage(), 500 );
		}

		$upload_dir      = wp_upload_dir();
		$watermarked_url = str_replace( $upload_dir['basedir'], $upload_dir['baseurl'], $watermark_path );
		$watermarked_url = add_query_arg( 't', time(), $watermarked_url );
		$watermarked_url = apply_filters( 'envira_gallery_image_src', $watermarked_url, 0, [], [] );

		wp_send_json_success( [ 'url' => $watermarked_url ] );
	}

	/**
	 * Creates a copy of the attachment file and returns the  full path of the copy.
	 *
	 * @param int $attachment_id The ID of the attachment to copy.
	 *
	 * @return string The path to the copy of the attachment.
	 */
	private function image_for_preview( int $attachment_id ): string {
		global $wp_filesystem;

		if ( ! function_exists( 'WP_Filesystem' ) || ! $wp_filesystem ) {
			require_once ABSPATH . 'wp-admin/includes/file.php';

			WP_Filesystem();
		}

		// Create tmp folder in uploads.
		$upload_dir = wp_upload_dir();
		$tmp_dir    = trailingslashit( $upload_dir['basedir'] ) . 'envira-watermarking-preview';
		if ( ! file_exists( $tmp_dir ) ) {
			wp_mkdir_p( $tmp_dir );
		}

		// Get the attachment path and copy it to the tmp folder.
		$attachment_path     = get_attached_file( $attachment_id );
		$attachment_filename = $attachment_id . '-' . basename( $attachment_path );

		$tmp_attachment_path = trailingslashit( $tmp_dir ) . $attachment_filename;

		// Clear the temp folder always to avoid rogue files.
		foreach ( $wp_filesystem->dirlist( $tmp_dir ) as $file ) {
			$file_path = trailingslashit( $tmp_dir ) . $file['name'];
			$wp_filesystem->delete( $file_path );
		}

		$wp_filesystem->copy( $attachment_path, $tmp_attachment_path );

		return $tmp_attachment_path;
	}

	/**
	 * Overlay a watermark, if specified, to the uploaded image
	 *
	 * @param int $attachment_id    Attachment ID.
	 * @param int $gallery_id       Gallery ID.
	 *
	 * @since 1.0.0
	 */
	public function ajax_load_image( $attachment_id, $gallery_id ) {
		// Settings should already be saved at this point.
		// If the user wants to apply the watermark settings on new images it has to save the gallery settings first.
		Envira_Watermarking_Process::watermark_gallery_item( $attachment_id, $gallery_id );
	}

	/**
	 * Returns the singleton instance of the class.
	 *
	 * @since 1.0.0
	 *
	 * @return object The Envira_Watermarking_Ajax object.
	 */
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Envira_Watermarking_Ajax ) ) {
			self::$instance = new Envira_Watermarking_Ajax();
		}

		return self::$instance;
	}
}

// Load the AJAX class.
$envira_watermarking_ajax = Envira_Watermarking_Ajax::get_instance();
