<?php
/**
 * Watermarking Process class.
 *
 * @package Envira_Gallery_Watermarking
 */

/**
 * Class Envira_Watermarking_Process
 *
 * This class handles the watermarking process for images.
 * It provides a method to apply watermarks to images using either the GD or Imagick PHP extensions.
 */
class Envira_Watermarking_Process {
	/**
	 * Watermark an image using the settings from the gallery.
	 *
	 * @param int   $attachment_id The ID of the attachment to watermark.
	 * @param int   $gallery_id The ID of the gallery.
	 * @param array $passed_data The gallery data.
	 */
	public static function watermark_gallery_item( int $attachment_id, int $gallery_id, array $passed_data = [] ) {
		$data = empty( $passed_data ) ? get_post_meta( $gallery_id, '_eg_gallery_data', true ) : $passed_data;

		if ( ! envira_get_config( 'watermarking', $data ) || empty( $data['gallery'] ) ) {
			return '';
		}

		$watermark_id = envira_get_config( 'watermarking_image_id', $data );
		$type         = envira_get_config( 'watermarking_type', $data );
		$text_image   = get_post_meta( $gallery_id, '_envira_gallery_watermarking_text_image', true );
		$text         = envira_get_config( 'watermarking_text', $data );

		if (
			'image' === $type && empty( $watermark_id )
			|| ( 'text' === $type && ( empty( $text_image ) || empty( $text ) ) )
			|| get_post_meta( $attachment_id, '_envira_watermarking_applied', true )
		) {
			return '';
		}

		$image_path = get_attached_file( $attachment_id );
		$filename   = basename( $image_path );
		$ext        = pathinfo( $image_path, PATHINFO_EXTENSION );

		$settings = [
			'type'     => $type,
			'position' => envira_get_config( 'watermarking_position', $data ),
			'margin'   => (int) envira_get_config( 'watermarking_margin', $data ),
			'repeat'   => (bool) envira_get_config( 'watermarking_repeat', $data ),
			'opacity'  => (int) envira_get_config( 'watermarking_opacity', $data ),
			'rotation' => (int) envira_get_config( 'watermarking_rotation', $data ),
			'scale'    => (int) envira_get_config( 'watermarking_image_scale', $data ),
			'filename' => 'watermarked-' . uniqid() . ".$ext",
		];

		if ( 'image' === $type ) {
			$watermark_image_string = file_get_contents( get_attached_file( $watermark_id ) );
		} else {
			$image_data             = str_replace( 'data:image/png;base64,', '', $text_image );
			$watermark_image_string = base64_decode( $image_data );
		}

		try {
			$watermarked_image = self::watermark_it( $image_path, $watermark_image_string, $settings );
		} catch ( Exception $exception ) {
			$error = 'Fail to watermark image: ' . $exception->getMessage();

			if ( wp_doing_ajax() ) {
				wp_send_json_error( esc_html( $error ) );
			}
			wp_die( esc_html( $error ) );
		}

		if ( envira_get_config( 'watermarking_keep_original', $data ) ) {
			$backup_id = self::backup_attachment( $attachment_id );

			// Save unmodified image reference for later use.
			update_post_meta( $attachment_id, '_envira_watermarking_original', $backup_id );
		}

		update_post_meta( $attachment_id, '_wp_attached_file', $watermarked_image );

		// Regenerate the image metadata.
		wp_update_attachment_metadata( $attachment_id, wp_generate_attachment_metadata( $attachment_id, $watermarked_image ) );

		// Set watermarking applied flag.
		update_post_meta( $attachment_id, '_envira_watermarking_applied', true );

		// If data is being passed then its assumed that the save will occur later.
		if ( empty( $passed_data ) ) {
			// Update filename in gallery data.
			$data['gallery'][ $attachment_id ]['link']  = str_replace( $filename, $settings['filename'], $data['gallery'][ $attachment_id ]['link'] );
			$data['gallery'][ $attachment_id ]['src']   = str_replace( $filename, $settings['filename'], $data['gallery'][ $attachment_id ]['src'] );
			$data['gallery'][ $attachment_id ]['thumb'] = str_replace( $filename, $settings['filename'], $data['gallery'][ $attachment_id ]['thumb'] );

			update_post_meta( $gallery_id, '_eg_gallery_data', $data );
		}

		return $settings['filename'];
	}

	/**
	 * Watermark an image using GD or Imagick and store it in a custom destination.
	 *
	 * @param string $image_path Path to the image.
	 * @param string $watermark_image_string Watermark image contents.
	 * @param array  $settings Watermark settings.
	 *
	 * @throws Exception If no image manipulation library is present or ImagickException has been thrown.
	 * @return string Full path to the watermarked image.
	 */
	public static function watermark_it( string $image_path, string $watermark_image_string, array $settings ): string {
		$prefer_gd = apply_filters( 'envira_watermarking_prefer_gd', false );

		// If no destination is set, overwrite the original image.
		$settings['destination'] = $settings['destination'] ?? dirname( $image_path );
		$settings['filename']    = $settings['filename'] ?? basename( $image_path );

		if ( ! self::has_gd_extension() && ! self::has_imagick_extension() ) {
			throw new Exception( 'No image manipulation library present.' );
		}

		if ( self::has_gd_extension() && ( $prefer_gd || ! self::has_imagick_extension() ) ) {
			return self::watermark_gd( $image_path, $watermark_image_string, $settings );
		}

		try {
			return self::watermark_imagick( $image_path, $watermark_image_string, $settings );
		} catch ( ImagickException $exception ) {
			if ( self::has_gd_extension() ) {
				return self::watermark_gd( $image_path, $watermark_image_string, $settings );
			}
			throw new Exception( esc_html( $exception->getMessage() ) );
		}
	}

	/**
	 * Flag to determine if the GD library has been compiled.
	 *
	 * @return bool True if it has proper extension, false otherwise.
	 */
	public static function has_gd_extension(): bool {
		return extension_loaded( 'gd' ) && function_exists( 'gd_info' );
	}

	/**
	 * Flag to determine if the Imagick library has been compiled.
	 *
	 * @return bool True if it has proper extension, false otherwise.
	 */
	public static function has_imagick_extension(): bool {
		return extension_loaded( 'imagick' );
	}

	/**
	 * Watermark image using GD
	 *
	 * @param string $image_path Path to the image.
	 * @param string $watermark_image_string Watermark image contents.
	 * @param array  $settings Watermark settings.
	 *
	 * @throws Exception If GD has an error.
	 * @return string Full path to the watermarked image.
	 */
	private static function watermark_gd( string $image_path, string $watermark_image_string, array $settings ): string {
		$save_to = trailingslashit( $settings['destination'] ) . $settings['filename'];

		// Create Images from string.
		$image     = imagecreatefromstring( file_get_contents( $image_path ) );
		$watermark = imagecreatefromstring( $watermark_image_string );

		if ( false === $watermark ) {
			throw new Exception( 'Unsupported media type as watermark.' );
		}

		if ( false === $image ) {
			throw new Exception( 'Unsupported source media type.' );
		}

		if ( 0 !== $settings['rotation'] ) {
			$transparent_color = imagecolorallocatealpha( $watermark, 0, 0, 0, 127 );
			imagefill( $watermark, 0, 0, $transparent_color );

			$watermark = imagerotate( $watermark, $settings['rotation'] * - 1, $transparent_color );

			imagealphablending( $watermark, true );
		}

		// Get widths and heights for the image and watermark.
		$image_width      = imagesx( $image );
		$image_height     = imagesy( $image );
		$watermark_width  = (float) imagesx( $watermark );
		$watermark_height = (float) imagesy( $watermark );

		if ( 'image' === $settings['type'] && ( 100 !== (int) $settings['scale'] ) ) {
			$scale_factor = (int) $settings['scale'] / 100;

			// Calculate new watermark size.
			$new_watermark_width  = floor( $watermark_width * $scale_factor );
			$new_watermark_height = floor( $watermark_height * $scale_factor );

			// Create resized watermark image.
			$watermark = imagescale( $watermark, $new_watermark_width, $new_watermark_height );

			// From here on out, the "new" values are the actual width/height values to consider.
			$watermark_width  = $new_watermark_width;
			$watermark_height = $new_watermark_height;
		}

		// Enable imagealphablending for correct PNG rendering.
		imagealphablending( $image, true );
		imagealphablending( $watermark, true );

		// If repeat is enabled repeat the watermark image vertically and horizontally.
		if ( $settings['repeat'] ) {
			// Calculate the number of times the watermark image can be repeated.
			$repeat_x = floor( $image_width / $watermark_width );
			$repeat_y = floor( $image_height / $watermark_height );

			// Loop through the number of times the watermark image can be repeated.
			for ( $i = 0; $i <= $repeat_x; $i++ ) {
				for ( $j = 0; $j <= $repeat_y; $j++ ) {
					// Copy the entire $watermark image onto a matching sized portion of the $image.
					imagecopy( $image, $watermark, ( $i * $watermark_width ), ( $j * $watermark_height ), 0, 0, $watermark_width, $watermark_height );
				}
			}
		} else {
			// Calculate position of watermark based on settings.
			$position = self::get_position( $settings, $image_width, $image_height, $watermark_width, $watermark_height );

			// Copy the entire $watermark image onto a matching sized portion of the $image.
			imagecopy( $image, $watermark, $position['x'], $position['y'], 0, 0, $watermark_width, $watermark_height );
		}

		// Save the image based on its MIME type.
		$image_meta = getimagesize( $image_path );
		switch ( $image_meta['mime'] ) {
			case 'image/jpeg':
			case 'image/jpg':
				// Save image as JPEG.
				imagejpeg( $image, $save_to );
				break;

			case 'image/png':
				// Save image as PNG.
				imagepng( $image, $save_to );
				break;

			case 'image/webp':
				// Save image as webp.
				imagewebp( $image, $save_to );
				break;

			case 'image/gif':
				// Save image as GIF.
				imagegif( $image, $save_to );
				break;
		}

		// Free up resources.
		imagedestroy( $image );
		imagedestroy( $watermark );

		return $save_to;
	}

	/**
	 * Watermark image using Imagick
	 *
	 * @param string $image_path Path to the image.
	 * @param string $watermark_image_string Watermark image contents.
	 * @param array  $settings Watermark settings.
	 *
	 * @throws ImagickException If Imagick has an exception.
	 * @return string Full path to the watermarked image.
	 */
	private static function watermark_imagick( string $image_path, string $watermark_image_string, array $settings ): string {
		$save_to   = trailingslashit( $settings['destination'] ) . $settings['filename'];
		$image     = new Imagick( $image_path );
		$watermark = new Imagick();

		$watermark->readImageBlob( $watermark_image_string );

		if ( 0 !== $settings['rotation'] ) {
			$watermark->rotateImage( new ImagickPixel( 'none' ), $settings['rotation'] );
		}

		// Get widths and heights for the image and watermark.
		$image_size   = $image->getImageGeometry();
		$image_width  = $image_size['width'];
		$image_height = $image_size['height'];

		$watermark_size   = $watermark->getImageGeometry();
		$watermark_width  = (float) $watermark_size['width'];
		$watermark_height = (float) $watermark_size['height'];

		if ( 'image' === $settings['type'] && ( ! in_array( (int) $settings['scale'], [ 0, 100 ], true ) ) ) {
			$scale_factor = (int) $settings['scale'] / 100;
			// Calculate new watermark size.
			$new_watermark_width  = floor( $watermark_width * $scale_factor );
			$new_watermark_height = floor( $watermark_height * $scale_factor );

			// Create resized watermark image.
			$watermark->scaleImage( $new_watermark_width, $new_watermark_height );

			$watermark_width  = $new_watermark_width;
			$watermark_height = $new_watermark_height;
		}

		// If repeat is enabled repeat the watermark image vertically and horizontally.
		if ( $settings['repeat'] ) {
			// Calculate the number of times the watermark image can be repeated.
			$repeat_x = floor( $image_width / $watermark_width );
			$repeat_y = floor( $image_height / $watermark_height );

			// Loop through the number of times the watermark image can be repeated.
			for ( $i = 0; $i <= $repeat_x; $i++ ) {
				for ( $j = 0; $j <= $repeat_y; $j++ ) {
					// Copy the entire $watermark image onto a matching sized portion of the $image.
					$image->compositeImage( $watermark, Imagick::COMPOSITE_OVER, ( $i * $watermark_width ), ( $j * $watermark_height ) );
				}
			}
		} else {
			// Calculate position of watermark based on settings.
			$position = self::get_position( $settings, $image_width, $image_height, $watermark_width, $watermark_height );

			// Copy the entire $watermark image onto a matching sized portion of the $image.
			$image->compositeImage( $watermark, Imagick::COMPOSITE_OVER, $position['x'], $position['y'] );
		}

		// Save.
		$image->writeImage( $save_to );

		// Free up resources.
		unset( $image );
		unset( $watermark );

		return $save_to;
	}

	/**
	 * Get the position of the watermark based on the settings.
	 *
	 * @param array $settings Watermark settings.
	 * @param int   $image_width Width of the image.
	 * @param int   $image_height Height of the image.
	 * @param float $watermark_width Width of the watermark.
	 * @param float $watermark_height Height of the watermark.
	 *
	 * @return array{y: int, x: int}
	 */
	private static function get_position(
		array $settings,
		int $image_width,
		int $image_height,
		float $watermark_width,
		float $watermark_height
	): array {
		$watermark_margin = round( $settings['margin'] );

		switch ( $settings['position'] ) {
			case 'top-left':
				$position = [
					'x' => $watermark_margin,
					'y' => $watermark_margin,
				];
				break;
			case 'top-right':
				$position = [
					'x' => $image_width - $watermark_width - $watermark_margin,
					'y' => $watermark_margin,
				];
				break;
			case 'center':
				$position = [
					'x' => floor( ( $image_width - $watermark_width ) / 2 ),
					'y' => floor( ( $image_height - $watermark_height ) / 2 ),
				];
				break;
			case 'bottom-left':
				$position = [
					'x' => $watermark_margin,
					'y' => $image_height - $watermark_height - $watermark_margin,
				];
				break;
			default:
			case 'bottom-right':
				$position = [
					'x' => $image_width - $watermark_width - $watermark_margin,
					'y' => $image_height - $watermark_height - $watermark_margin,
				];
				break;
		}

		return $position;
	}

	/**
	 * Creates a new attachment on the database but with the same file as the original.
	 * Basically creates a copy of the attachment as if we were uploading the same file again.
	 *
	 * We could also copy the taxonomies and other metadata from the original attachment.
	 * Not sure if this is what the user would expect.
	 *
	 * @param int $attachment_id The ID of the attachment to clone.
	 *
	 * @return int The ID of the new attachment post.
	 */
	public static function backup_attachment( int $attachment_id ): int {
		$original_attachment = get_post( $attachment_id );

		$new_attachment = [
			'guid'           => $original_attachment->guid,
			'post_mime_type' => $original_attachment->post_mime_type,
			'post_title'     => $original_attachment->post_title,
			'post_content'   => $original_attachment->post_content,
			'post_status'    => 'inherit',
			'post_type'      => 'attachment',
		];

		$new_attachment_id  = wp_insert_post( $new_attachment );
		$metadata           = wp_get_attachment_metadata( $attachment_id );
		$original_file_path = get_attached_file( $attachment_id );

		wp_update_attachment_metadata( $new_attachment_id, $metadata );

		update_post_meta( $new_attachment_id, '_wp_attached_file', $original_file_path );
		delete_post_meta( $new_attachment_id, '_envira_watermarking_applied' );
		delete_post_meta( $new_attachment_id, '_envira_watermarking_original' );

		return $new_attachment_id;
	}
}
