<?php
/**
 * Plugin Name: Envira Gallery - WooCommerce Addon
 * Plugin URI:  http://enviragallery.com
 * Description: Link Envira Gallery images to WooCommerce Products to enable eCommerce functionality from within your Envira Galleries
 * Author:      Envira Gallery Team
 * Author URI:  http://enviragallery.com
 * Version:     1.5.9
 * WC requires at least: 4.0.0
 * WC tested up to: 6.3.1
 * Text Domain: envira-woocommerce
 * Domain Path: languages
 *
 * @package Envira Gallery
 * @subpackage Envira WooCommerce
 *
 * Envira Gallery is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * any later version.
 *
 * Envira Gallery is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Envira Gallery. If not, see <http://www.gnu.org/licenses/>.
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( function_exists( 'envira_license_checker' ) && false === envira_license_checker() ) {
	return false;
}

use Envira\WooCommerce\Frontend\Frontend_Container;
use Envira\WooCommerce\Admin\Admin_Container;

use Envira\Utils\Updater;

if ( ! class_exists( 'Envira_WooCommerce' ) ) :
	/**
	 * Main plugin class.
	 *
	 * @since 1.0.0
	 *
	 * @package Envira_WooCommerce
	 * @author  Envira Team
	 */
	class Envira_WooCommerce {

		/**
		 * Holds the class object.
		 *
		 * @since 1.0.0
		 *
		 * @var object
		 */
		public static $instance;

		/**
		 * Plugin version, used for cache-busting of style and script file references.
		 *
		 * @since 1.0.0
		 *
		 * @var string
		 */
		public $version = '1.5.9';

		/**
		 * The name of the plugin.
		 *
		 * @since 1.0.0
		 *
		 * @var string
		 */
		public $plugin_name = 'Envira WooCommerce';

		/**
		 * Unique plugin slug identifier.
		 *
		 * @since 1.0.0
		 *
		 * @var string
		 */
		public $plugin_slug = 'envira-woocommerce';

		/**
		 * Plugin file.
		 *
		 * @since 1.0.0
		 *
		 * @var string
		 */
		public $file = __FILE__;

		/**
		 * Primary class constructor.
		 *
		 * @since 1.0.0
		 */
		public function __construct() {

			// Fire a hook before the class is setup.
			do_action( 'envira_woocommerce_pre_init' );

			// Load the plugin textdomain.
			add_action( 'plugins_loaded', [ $this, 'load_plugin_textdomain' ] );

			// Load the plugin.
			add_action( 'envira_gallery_init', [ $this, 'init' ], 99 );

			// Load the updater.
			add_action( 'envira_gallery_updater', [ $this, 'updater' ], 10, 1 );
		}

		/**
		 * Setup Constants function.
		 *
		 * @since 1.6.0
		 *
		 * @access public
		 * @return void
		 */
		public function setup_constants() {

			if ( ! defined( 'ENVIRA_WOOCOMMERCE_VERSION' ) ) {

				define( 'ENVIRA_WOOCOMMERCE_VERSION', $this->version );

			}

			if ( ! defined( 'ENVIRA_WOOCOMMERCE_SLUG' ) ) {

				define( 'ENVIRA_WOOCOMMERCE_SLUG', $this->plugin_slug );

			}

			if ( ! defined( 'ENVIRA_WOOCOMMERCE_FILE' ) ) {

				define( 'ENVIRA_WOOCOMMERCE_FILE', $this->file );

			}

			if ( ! defined( 'ENVIRA_WOOCOMMERCE_DIR' ) ) {

				define( 'ENVIRA_WOOCOMMERCE_DIR', plugin_dir_path( __FILE__ ) );

			}

			if ( ! defined( 'ENVIRA_WOOCOMMERCE_URL' ) ) {

				define( 'ENVIRA_WOOCOMMERCE_URL', plugin_dir_url( __FILE__ ) );

			}
		}

		/**
		 * Loads the plugin textdomain for translation.
		 *
		 * @since 1.0.0
		 */
		public function load_plugin_textdomain() {

			load_plugin_textdomain( $this->plugin_slug, false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
		}

		/**
		 * Loads the plugin into WordPress.
		 *
		 * @since 1.0.0
		 */
		public function init() {

			// Run hook once Envira has been initialized.
			do_action( 'envira_woocommerce_init' );

			// Load admin only components.
			if ( is_admin() ) {
				$admin = new \Envira\WooCommerce\Admin\Admin_Container();
			}

			$frontend = new \Envira\WooCommerce\Frontend\Frontend_Container();
		}

		/**
		 * Loads all admin related files into scope.
		 *
		 * @since 1.0.0
		 */
		public function require_admin() {
		}

		/**
		 * Initializes the addon updater.
		 *
		 * @since 1.0.0
		 *
		 * @param string $key The user license key.
		 */
		public function updater( $key ) {

			$args = [
				'plugin_name' => $this->plugin_name,
				'plugin_slug' => $this->plugin_slug,
				'plugin_path' => plugin_basename( __FILE__ ),
				'plugin_url'  => trailingslashit( WP_PLUGIN_URL ) . $this->plugin_slug,
				'remote_url'  => 'https://enviragallery.com/',
				'version'     => $this->version,
				'key'         => $key,
			];

			$updater = new Updater( $args );
		}

		/**
		 * Autoload function.
		 *
		 * @access public
		 * @static
		 *
		 * @param mixed $php_class The class.
		 *
		 * @return void
		 */
		public static function autoload( $php_class ) {

			// Prepare variables.
			$prefix   = 'Envira\\WooCommerce\\';
			$base_dir = __DIR__ . '/src/';
			$length   = mb_strlen( $prefix );

			// If the class is not using the namespace prefix, return.
			if ( 0 !== strncmp( $prefix, $php_class, $length ) ) {
				return;
			}

			// Prepare classes to be autoloaded.
			$relative_class = mb_substr( $php_class, $length );
			$file           = $base_dir . str_replace( '\\', '/', $relative_class ) . '.php';

			// If the file exists, load it.
			if ( file_exists( $file ) ) {
				require_once $file;
			}
		}

		/**
		 * Returns the singleton instance of the class.
		 *
		 * @since 1.4.1
		 *
		 * @return object The Envira_WooCommerce object.
		 */
		public static function get_instance() {

			if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Envira_WooCommerce ) ) {
				self::$instance = new self();
				self::$instance->setup_constants();
				self::$instance->require_admin();
			}

			return self::$instance;
		}
	}

	spl_autoload_register( 'Envira_WooCommerce::autoload' );

	// Load the plugin.
	add_action(
		'envira_gallery_init',
		function () {
			Envira_WooCommerce::get_instance();
		}
	);

endif;
