<?php
/**
 * Metabox class.
 *
 * @since 1.0.0
 *
 * @package Envira_WooCommerce
 * @author  Envira Team
 */

namespace Envira\WooCommerce\Admin;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use Envira\WooCommerce\Frontend\Common;
use Envira\Admin\Metaboxes as EnviraMetaboxes;
use Envira\Albums\Admin\Metaboxes as AlbumMetaboxes;

/**
 * Metabox class.
 *
 * @since 1.0.0
 *
 * @package Envira_WooCommerce
 * @author  Envira Team
 */
class Metaboxes {

	/**
	 * Holds the class object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public static $instance;

	/**
	 * Path to the file.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	public $file = __FILE__;

	/**
	 * Holds the base class object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public $base;

	/**
	 * Primary class constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {

		// Gallery.
		add_action( 'envira_gallery_metabox_scripts', [ $this, 'scripts' ] );
		add_action( 'admin_enqueue_scripts', [ $this, 'styles' ] );

		add_filter( 'envira_gallery_tab_nav', [ $this, 'register_tabs' ] );
		add_action( 'envira_gallery_tab_woocommerce', [ $this, 'woocommerce_tab' ] );
		add_filter( 'envira_gallery_save_settings', [ $this, 'save_gallery' ], 10, 2 );
		add_action( 'envira_gallery_mobile_box', [ $this, 'mobile_screen' ] );

		// Gallery: Individual Image Settings.
		add_action( 'print_media_templates', [ $this, 'display_meta_settings' ], 10, 1 );

		// Album.
		add_filter( 'envira_albums_tab_nav', [ $this, 'register_tabs' ] );
		add_action( 'envira_gallery_mobile_box', [ $this, 'mobile_screen' ] );
		add_action( 'envira_albums_tab_woocommerce', [ $this, 'woocommerce_tab' ] );
		add_filter( 'envira_albums_save_settings', [ $this, 'save_album' ], 10, 2 );

		// WooCommerce Auto-Product Creation.
		add_filter( 'envira_gallery_ajax_gallery_images', [ $this, 'create_woo_product' ], 10, 3 );
		add_action( 'envira_gallery_ajax_remove_image', [ $this, 'remove_woo_product' ], 10, 3 );
	}

	/**
	 * Remove woo products when an image is removed from the gallery.
	 *
	 * @since 1.0.0
	 *
	 * @param numeric $attach_id Attach ID.
	 * @param array   $post_id Post ID.
	 * @param integer $gallery_data Gallery Data.
	 */
	public function remove_woo_product( $attach_id, $post_id, $gallery_data = false ) {

		if ( ! class_exists( 'WooCommerce' ) || ! class_exists( 'WC_Admin_Duplicate_Product' ) ) {
			return;
		}

		if ( false === $gallery_data ) {
			$gallery_data = envira_get_gallery( $post_id );
		}

		if ( empty( $gallery_data ) ) {
			return;
		}

		// see if the global setting AND the setting for the gallery both are unchecked.

		// gallery level settings.
		$local_woocommerce_product_delete = envira_get_config( 'woocommerce_product_delete', $gallery_data );

		// global settings.
		$global_woocommerce_product_delete = envira_get_setting( 'woocommerce_product_delete' );

		// if local settings are turned off then no remove!
		if ( false === $local_woocommerce_product_delete || 0 === intval( $local_woocommerce_product_delete ) ) {
			return $gallery_data;
		}

		// if global settings are turned off then no remove!
		if ( false === $global_woocommerce_product_delete || 0 === intval( $global_woocommerce_product_delete ) ) {
			return $gallery_data;
		}

		// figure out the product ID associated with this image.
		$product_id = ( ! empty( $gallery_data['gallery'][ $attach_id ]['woocommerce_product'] ) ) ? $gallery_data['gallery'][ $attach_id ]['woocommerce_product'] : false;

		if ( empty( $product_id ) ) {
			return;
		}

		// Delete the product.
		wp_delete_post( $product_id, false ); // false = no force delete.
	}

	/**
	 * Adds woo products based on the template - we check local AND then global settings
	 *
	 * @since 1.0.0
	 *
	 * @param array   $gallery_data Gallery Data.
	 * @param array   $added_images Array of images just added via the ajax in gallery edit.
	 * @param integer $post_id Post ID.
	 */
	public function create_woo_product( $gallery_data, $added_images, $post_id ) {

		if ( empty( $added_images ) || false === $post_id ) {
			return $gallery_data;
		}

		if ( ! class_exists( 'WooCommerce' ) || ! class_exists( 'WC_Admin_Duplicate_Product' ) ) {
			return $gallery_data;
		}

		$post_type = get_post_type( $post_id );
		$data      = get_post_meta( $post_id, '_eg_gallery_data', true );

		switch ( $post_type ) {
			/**
			* Gallery
			*/
			case 'envira':
				$instance = new EnviraMetaboxes();
				$key      = '_envira_gallery';
				break;

			/**
			* Album
			*/
			case 'envira_album':
				$instance = new AlbumMetaboxes();
				$key      = '_eg_album_data[config]';
				break;
		}

		// gallery level settings.
		$local_standalone_template = envira_get_config( 'woocommerce_standalone_template', $data );
		$local_auto_create         = envira_get_config( 'woocommerce_auto_create', $data );

		// global settings.
		$global_standalone_template = envira_get_setting( 'woocommerce_standalone_template' );
		$global_auto_create         = envira_get_setting( 'woocommerce_auto_create' );

		// if local settings are turned off or there is no template set then, then no auto create!
		if ( false === $local_auto_create || 0 === intval( $local_auto_create ) ) {
			return $gallery_data;
		}

		// if there is no template defined, then no auto create!
		if ( false === $local_standalone_template && false === $global_standalone_template ) {
			return $gallery_data;
		}

		// Get product.
		$product = wc_get_product( $local_standalone_template );
		if ( false === $product ) {
			$product = wc_get_product( $global_standalone_template );
			if ( false === $product ) {
				return $gallery_data;
			}
		}

		foreach ( $added_images as $added_image ) {
			foreach ( $gallery_data['gallery'] as $image_id => $image ) {
				if ( $image_id === $added_image['id'] ) {

					// Duplicate the product.
					$wc_admin              = new \WC_Admin_Duplicate_Product();
					$duplicated_product    = $wc_admin->product_duplicate( $product );
					$duplicated_product_id = $duplicated_product->get_id();

					// Update post.
					$product_post = [
						'ID'         => $duplicated_product_id,
						'post_title' => apply_filters( 'envira_woocommerce_admin_product_create_title', $image['title'] . ' Product', $duplicated_product, $image, $post_id ),
					];

					// Update the post into the database.
					wp_update_post( $product_post );

					// Associate the WooCommerce Product with the Envira image.
					$gallery_data['gallery'][ $image_id ]['woocommerce_product'] = $duplicated_product_id;
				}
			}
		}

		// resave gallery settings.
		update_post_meta( $post_id, '_eg_gallery_data', $gallery_data );

		return $gallery_data;
	}


	/**
	 * Enqueues the Media Editor script, which is used when editing a gallery image
	 * This outputs the WooCommerce settings for each individual image
	 *
	 * @since 1.0.4
	 */
	public function styles() {

		$version = ( defined( 'ENVIRA_DEBUG' ) && 'true' === ENVIRA_DEBUG ) ? $version = time() . '-' . ENVIRA_WOOCOMMERCE_VERSION : ENVIRA_WOOCOMMERCE_VERSION;

		wp_enqueue_style( ENVIRA_WOOCOMMERCE_SLUG . '-metabox-style', plugins_url( 'assets/css/envira-woocommerce-admin.css', ENVIRA_WOOCOMMERCE_FILE ), [], $version );
	}

	/**
	 * Enqueues the Media Editor script, which is used when editing a gallery image
	 * This outputs the WooCommerce settings for each individual image
	 *
	 * @since 1.0.4
	 */
	public function scripts() {

		$version = ( defined( 'ENVIRA_DEBUG' ) && 'true' === ENVIRA_DEBUG ) ? $version = time() . '-' . ENVIRA_WOOCOMMERCE_VERSION : ENVIRA_WOOCOMMERCE_VERSION;

		wp_enqueue_script( ENVIRA_WOOCOMMERCE_SLUG . '-media-edit', envira_script( 'assets/js/min/media-edit-min.js', ENVIRA_WOOCOMMERCE_FILE ), [ 'jquery' ], $version, true );
		wp_enqueue_script( ENVIRA_WOOCOMMERCE_SLUG . '-conditional-fields-script', envira_script( 'assets/js/min/conditional-fields-min.js', ENVIRA_WOOCOMMERCE_FILE ), [ 'jquery' ], $version, true );

		wp_localize_script(
			ENVIRA_WOOCOMMERCE_SLUG . '-media-edit',
			'envira_woocommerce',
			[
				'get_product_attributes_nonce' => wp_create_nonce( 'envira-woocommerce-get-product-attributes' ),
			]
		);
	}

	/**
	 * Registers tab(s) for this Addon in the Settings screen
	 *
	 * @since 1.0.0
	 *
	 * @param   array $tabs   Tabs.
	 * @return  array           Tabs
	 */
	public function register_tabs( $tabs ) {

		$tabs['woocommerce'] = __( 'WooCommerce', 'envira-woocommerce' );
		return $tabs;
	}

	/**
	 * Adds addon settings UI to the WooCommerce tab
	 *
	 * @since 1.0.0
	 *
	 * @param object $post The current post object.
	 */
	public function woocommerce_tab( $post ) {

		wp_nonce_field( 'envira_woocommerce_save_settings', 'envira_woocommerce_nonce' );

		$defaults = [];

		// Get post type so we load the correct metabox instance and define the input field names
		// Input field names vary depending on whether we are editing a Gallery or Album.
		$post_type = get_post_type( $post );

		switch ( $post_type ) {
			/**
			* Gallery
			*/
			case 'envira':
				$data     = get_post_meta( $post->ID, '_eg_gallery_data', true );
				$common   = new Common();
				$defaults = $common->defaults( $defaults, $post->ID );

				$woocommerce_auto_create         = envira_get_config( 'woocommerce_auto_create', $data, $defaults['woocommerce_auto_create'] );
				$woocommerce_standalone_template = envira_get_config( 'woocommerce_standalone_template', $data, $defaults['woocommerce_standalone_template'] );
				$woocommerce_product_delete      = envira_get_config( 'woocommerce_product_delete', $data, $defaults['woocommerce_product_delete'] );
				$woocommerce                     = envira_get_config( 'woocommerce', $data, $defaults['woocommerce'] );
				$lightbox_woocommerce            = envira_get_config( 'lightbox_woocommerce', $data, $defaults['lightbox_woocommerce'] );

				$key = '_envira_gallery';
				break;

			/**
			* Album
			*/
			case 'envira_album':
				$data     = get_post_meta( $post->ID, '_eg_album_data', true );
				$common   = new Common();
				$defaults = $common->album_defaults( $defaults, $post->ID );

				$woocommerce          = envira_get_config( 'woocommerce', $data, $defaults['woocommerce'] );
				$lightbox_woocommerce = envira_get_config( 'lightbox_woocommerce', $data, $defaults['lightbox_woocommerce'] );

				$key = '_eg_album_data[config]';
				break;
		}

		$product_args  = [
			'numberposts' => -1,
			'post_status' => 'published',
		];
		$products_args = apply_filters( 'envira_woocommerce_admin_product_templates_args', $product_args, $post );
		$products      = function_exists( 'wc_get_products' ) ? apply_filters( 'envira_woocommerce_admin_product_templates', wc_get_products( $products_args ) ) : false;

		$woocommerce_add_product_url = admin_url( 'post-new.php?post_type=product' );

		if ( ! class_exists( 'WooCommerce' ) ) : ?>
				<div class="envira-external-req">
					<h2><?php esc_html_e( 'Missing WooCommerce Plugin', 'envira-woocommerce' ); ?></h2>
					<p><?php esc_html_e( 'In order to use WooCommerce in galleries, you must first enable and setup WooCommerce.', 'envira-woocommerce' ); ?></p>
					<p><a href="<?php echo esc_html( admin_url( 'plugin-install.php?tab=search&s=woocommerce' ) ); ?>" class="button button-primary"><?php esc_html_e( 'Click here to download WooCommerce', 'envira-woocommerce' ); ?></a></p>
				</div>
			<?php
			else :
				// Gallery options only apply to Galleries, not Albums.
				if ( 'envira' === $post_type ) {
					?>
					<p class="envira-intro">
						<?php esc_html_e( 'WooCommerce Options.', 'envira-woocommerce' ); ?>
						<small>
						<?php esc_html_e( 'The settings below adjust the WooCommerce options for the gallery.', 'envira-woocommerce' ); ?>
					<br/>
					<?php esc_html_e( 'Need some help?', 'envira-woocommerce' ); ?>
					<a href="http://enviragallery.com/docs/woocommerce-addon/" class="envira-doc" target="_blank" rel="noopener">
					<?php esc_html_e( 'Read the Documentation', 'envira-woocommerce' ); ?>
					</a>
					or
					<a href="https://www.youtube.com/embed/H5zAzLUJZTg/?rel=0" class="envira-video" target="_blank" rel="noopener">
					<?php esc_html_e( 'Watch a Video', 'envira-woocommerce' ); ?>
					</a>
				</small>
			</p>
			<table class="form-table">
				<tbody>
					<tr id="envira-config-woocommerce-auto-create">
						<th scope="row">
							<label for="envira-config-woocommerce"><?php esc_html_e( 'Automatically Create WooCommerce Product?', 'envira-woocommerce' ); ?></label>
						</th>
						<td>
							<input id="envira-config-woocommerce" type="checkbox" name="<?php echo esc_html( $key ); ?>[woocommerce_auto_create]" value="1"<?php checked( $woocommerce_auto_create, 1 ); ?> />
							<span class="description"><?php esc_html_e( 'When you add an image to a gallery, Envira will automatically create a (draft) product associated with it. When checked this gallery\'s settings override any global WooCommerce settings.', 'envira-woocommerce' ); ?></span>
						</td>
					</tr>
					<tr id="envira-config-woocommerce-template">
							<th scope="row">
								<label for="envira-config-standalone-template"><?php esc_html_e( 'Default Product Template', 'envira-standalone' ); ?></label>
							</th>
							<td>
								<?php if ( ! empty( $products ) ) : ?>
								<select id="envira-config-standalone-template" name="<?php echo esc_html( $key ); ?>[woocommerce_standalone_template]">
									<option value="">(None)</option>
									<?php foreach ( (array) $products as $product_key => $product ) : ?>

									<option value="<?php echo esc_html( $product->get_id() ); ?>"<?php selected( $woocommerce_standalone_template, $product->get_id() ); ?>><?php echo esc_html( $product->get_name() ); ?></option>
									<?php endforeach; ?>
								</select>
								<p class="description">
									<?php
									printf(
										wp_kses_post(
											/* translators: Link */
											__( 'Select an existing product that will act as a "template" for automatically generated WooCommerce products. <a target="_blank" href="%s">Create a new product</a> to get started.', 'envira-gallery' )
										),
										esc_url( $woocommerce_add_product_url )
									);
									?>
								</p>

								<?php else : ?>

								<p class="description"><?php esc_html_e( 'Your current theme does not have any custom templates. If you want to use a template besides the default, you need to add a custom template to your theme.', 'envira-gallery' ); ?></p>

								<?php endif; ?>

							</td>
					</tr>
					<tr id="envira-config-woocommerce-product-delete">
							<th scope="row">
								<label for="envira-config-standalone-template"><?php esc_html_e( 'Delete WooCommerce Products', 'envira-standalone' ); ?></label>
							</th>
							<td>
								<input type="checkbox" name="<?php echo esc_html( $key ); ?>[woocommerce_product_delete]" id="envira-woocommerce-product-delete" value="1" <?php checked( $woocommerce_product_delete, 1 ); ?> />
								<span class="description"><?php esc_html_e( 'If enabled, Envira will delete a WooCommerce product associated with an image when the image is removed from a gallery.', 'envira-woocommerce' ); ?></span>
							</td>
					</tr>
				</tbody>
			</table>


					<p class="envira-intro">
						<?php esc_html_e( 'WooCommerce Gallery Options.', 'envira-woocommerce' ); ?>
			</p>
			<table class="form-table">
				<tbody>
					<tr id="envira-config-woocommerce-box">
						<th scope="row">
							<label for="envira-config-woocommerce"><?php esc_html_e( 'Enable WooCommerce?', 'envira-woocommerce' ); ?></label>
						</th>
						<td>
							<input id="envira-config-woocommerce" type="checkbox" name="<?php echo esc_html( $key ); ?>[woocommerce]" value="1"<?php checked( $woocommerce, 1 ); ?> />
							<span class="description"><?php esc_html_e( 'Enables WooCommerce Add to Cart functionality for each image in the gallery grid, if the image is assigned to a WooCommerce Product.', 'envira-woocommerce' ); ?></span>
						</td>
					</tr>
				</tbody>
			</table>
					<?php
				}

				// Lightbox Options.
				?>
		<p class="envira-intro">
				<?php esc_html_e( 'WooCommerce Lightbox Options.', 'envira-woocommerce' ); ?>
		</p>
		<table class="form-table">
			<tbody>
				<tr id="envira-config-woocommerce-box">
					<th scope="row">
						<label for="envira-config-lightbox-woocommerce"><?php esc_html_e( 'Enable WooCommerce?', 'envira-woocommerce' ); ?></label>
					</th>
					<td>
						<input id="envira-config-lightbox-woocommerce" type="checkbox" name="<?php echo esc_html( $key ); ?>[lightbox_woocommerce]" value="1"<?php checked( $lightbox_woocommerce, 1 ); ?> />
						<span class="description"><?php esc_html_e( 'Enables WooCommerce Add to Cart functionality for each image in the Lightbox view, if the image is assigned to a WooCommerce Product.', 'envira-woocommerce' ); ?></span>
					</td>
				</tr>
			</tbody>
		</table>
				<?php
		endif;
	}

	/**
	 * Saves the addon's settings for Galleries.
	 *
	 * @since 1.0.0
	 *
	 * @param array $settings  Array of settings to be saved.
	 * @param int   $post_id     The current post ID.
	 * @return array $settings Amended array of settings to be saved.
	 */
	public function save_gallery( $settings, $post_id ) {

		if (
			! isset( $_POST['_envira_gallery'], $_POST['envira_woocommerce_nonce'] )
			|| ! wp_verify_nonce( sanitize_key( $_POST['envira_woocommerce_nonce'] ), 'envira_woocommerce_save_settings' )
		) {
			return $settings;
		}

		$settings['config']['woocommerce']                  = ( isset( $_POST['_envira_gallery']['woocommerce'] ) ? 1 : 0 );
		$settings['config']['lightbox_woocommerce']         = ( isset( $_POST['_envira_gallery']['lightbox_woocommerce'] ) ? 1 : 0 );
		$settings['config']['mobile_disable_woo_galleries'] = ( isset( $_POST['_envira_gallery']['mobile_disable_woo_galleries'] ) ? 1 : 0 );
		$settings['config']['woocommerce_auto_create']      = ( isset( $_POST['_envira_gallery']['woocommerce_auto_create'] ) ? 1 : 0 );
		$settings['config']['woocommerce_product_delete']   = ( isset( $_POST['_envira_gallery']['woocommerce_product_delete'] ) ? 1 : 0 );

		$settings['config']['woocommerce_standalone_template'] = ( isset( $_POST['_envira_gallery']['woocommerce_standalone_template'] ) ? intval( $_POST['_envira_gallery']['woocommerce_standalone_template'] ) : 0 );

		return $settings;
	}

	/**
	 * Saves the addon's settings for Albums.
	 *
	 * @since 1.0.9
	 *
	 * @param array $settings  Array of settings to be saved.
	 * @param int   $post_id     The current post ID.
	 * @return array $settings Amended array of settings to be saved.
	 */
	public function save_album( $settings, $post_id ) {

		if (
			! isset( $_POST['_eg_album_data'], $_POST['envira_woocommerce_nonce'] )
			|| ! wp_verify_nonce( sanitize_key( $_POST['envira_woocommerce_nonce'] ), 'envira_woocommerce_save_settings' )
		) {
			return $settings;
		}

		$settings['config']['lightbox_woocommerce']         = ( isset( $_POST['_eg_album_data']['config']['lightbox_woocommerce'] ) ? 1 : 0 );
		$settings['config']['mobile_disable_woo_galleries'] = ( isset( $_POST['_envira_gallery']['mobile_disable_woo_galleries'] ) ? 1 : 0 );

		return $settings;
	}


	/**
	 * Display Meta Settings.
	 *
	 * @since 1.0.9
	 *
	 * @param int $post_id The current post ID.
	 */
	public function display_meta_settings( $post_id = false ) {
		$html = $this->meta_settings();

		// phpcs:disable WordPress.Security.EscapeOutput.OutputNotEscaped -- Intentionally not escaped.
		echo $html;
	}

	/**
	 * Outputs fields in the modal window when editing an existing image,
	 * allowing the user to choose which WooCommerce Product/Variant to
	 * link the image to.
	 *
	 * @since 1.0.0
	 */
	public function meta_settings() {

		// Get WooCommerce Products.
		$args     = [
			'post_type'      => 'product',
			'posts_per_page' => -1,
		];
		$products = new \WP_Query( $args );

		$html = '';

		// WooCommerce Meta Editor
		// Use: wp.media.template( 'envira-meta-editor-woocommerce' ).

		$html .= '<script type="text/html" id="tmpl-envira-meta-editor-woocommerce">
			<label class="setting">
				<span class="name"> ' . esc_html__( 'WooCommerce Product', 'envira-woocommerce' ) . '</span>
				<select name="woocommerce_product" size="1">
					<option value="0">' . esc_html__( '(No Product)', 'envira-woocommerce' ) . '</option>';

		if ( $products->have_posts() ) {
			foreach ( $products->posts as $product ) {
				$html .= '<option value="' . esc_html( $product->ID ) . '">' . esc_html( $product->post_title ) . '</option>';
			}
		}

		$html .= '
				</select>
				<span class="description">';

		$html .= esc_html__( 'Choose a WooCommerce Product which relates to this image.  Visitors will then be able to order the selected product in the gallery and/or lightbox views.', 'envira-woocommerce' );

		$html .= '
				</span>
			</label>
		</script>';

		return $html;
	}

	/**
	 * Adds addon settings UI to the Mobile tab
	 *
	 * @since 1.0.0
	 *
	 * @param object $post The current post object.
	 */
	public function mobile_screen( $post ) {

		// Get post type so we load the correct metabox instance and define the input field names
		// Input field names vary depending on whether we are editing a Gallery or Album.
		$post_type = get_post_type( $post );
		switch ( $post_type ) {
			/**
			* Gallery
			*/
			case 'envira':
				$key = '_envira_gallery';
				break;

			/**
			* Album
			*/
			case 'envira_album':
				$key = '_eg_album_data[config]';
				break;
		}
		?>
		<tr id="envira-config-woo-galleries-mobile-box">
			<th scope="row">
				<label for="envira-config-woo-galleries-mobile"><?php esc_html_e( 'Disable WooCommerce In Galleries On Mobile?', 'envira-woocommerce' ); ?></label>
			</th>
			<td>
				<input id="envira-config-woo-galleries-mobile" type="checkbox" name="<?php echo esc_html( $key ); ?>[mobile_disable_woo_galleries]" value="1" <?php checked( envira_get_config( 'mobile_disable_woo_galleries', envira_get_config_default( 'mobile_disable_woo_galleries' ) ), 1 ); ?> />
				<span class="description"><?php esc_html_e( 'If enabled, no WooCommerce functionality or buttons will be displayed in galleries or lightboxes on mobile.', 'envira-woocommerce' ); ?></span>
			</td>
		</tr>
		<?php
	}

	/**
	 * Returns the singleton instance of the class.
	 *
	 * @since 1.0.0
	 *
	 * @return object The Envira_WooCommerce_Metaboxes object.
	 */
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Envira_WooCommerce_Metaboxes ) ) {
			self::$instance = new Envira_WooCommerce_Metaboxes();
		}

		return self::$instance;
	}
}

