<?php
/**
 * WooCommerce Admin Container Class
 *
 * @package Envira WooCommerce
 */

namespace Envira\WooCommerce\Admin;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Envira_WooCommerce_Settings class.
 *
 * @since 1.0.0
 *
 * @package Envira_WooCommerce
 * @author  Envira Team
 */
class Settings {

	/**
	 * Holds the class object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public static $instance;

	/**
	 * Path to the file.
	 *
	 * @since 1.0.0
	 *
	 * @var string
	 */
	public $file = __FILE__;

	/**
	 * Holds the base class object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public $base;

	/**
	 * Holds the common class object.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public $common;

	/**
	 * Holds settings.
	 *
	 * @since 1.0.0
	 *
	 * @var object
	 */
	public $settings;

	/**
	 * Primary class constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {

		// Actions.
		add_filter( 'envira_gallery_settings_tab_nav', [ $this, 'tabs' ] );
		add_action( 'envira_gallery_tab_settings_woocommerce', [ $this, 'settings' ] );
		add_action( 'init', [ $this, 'save' ] );
	}

	/**
	 * Add a tab to the Envira Gallery Settings screen
	 *
	 * @since 1.0.0
	 *
	 * @param array $tabs Existing tabs.
	 * @return array New tabs
	 */
	public function tabs( $tabs ) {

		$tabs['woocommerce'] = __( 'WooCommerce', 'envira-woocommerce' );

		return $tabs;
	}

	/**
	 * Outputs settings screen for the settings tab.
	 *
	 * @since 1.0.0
	 */
	public function settings() {

		$product_args               = [
			'numberposts' => -1,
			'post_status' => 'publish',
		];
		$products_args              = apply_filters( 'envira_woocommerce_admin_product_templates_args', $product_args );
		$products                   = function_exists( 'wc_get_products' ) ? apply_filters( 'envira_woocommerce_admin_product_templates', wc_get_products( $products_args ) ) : false;
		$current_global_template    = envira_get_setting( 'woocommerce_standalone_template' );
		$woocommerce_auto_create    = envira_get_setting( 'woocommerce_auto_create' );
		$woocommerce_product_delete = envira_get_setting( 'woocommerce_product_delete' );

		$woocommerce_add_product_url  = admin_url( 'post-new.php?post_type=product' );
		$woocommerce_link_plugin_repo = 'https://wordpress.org/plugins/woocommerce/';

		?>
		<div id="envira-settings-woocommerce">
			<?php
			// Output notices.
			do_action( 'envira_gallery_settings_woocommerce_tab_notice' );
			?>

			<table class="form-table">
				<tbody>
				<p class="envira-intro">
					<small>
						<?php esc_html_e( 'The settings below are global settings effecting all Envira galleries by default. See the \'WoCommerce\' tab in each gallery to override these settings.', 'envira-woocommerce' ); ?>
					<br/>
					</small>
				</p>
					<form action="edit.php?post_type=envira&amp;page=envira-gallery-settings#!envira-tab-woocommerce" method="post">
					<tr id="envira-config-woocommerce-template">
							<th scope="row">
								<label for="envira-config-standalone-template"><?php esc_html_e( 'Default Product Template', 'envira-standalone' ); ?></label>
							</th>
							<td>
								<?php if ( ! class_exists( 'WooCommerce' ) ) { ?>

									<?php /* translators: */ ?>
									<p class="description alert"><?php printf( esc_html__( 'You need to install and activate WooCommerce in order to see any available products. %s', 'envira-gallery' ), '<a target="_blank" href="' . esc_url( $woocommerce_link_plugin_repo ) . '">You can download the plugin</a> from the WordPress plugin repo.' ); ?></p>

								<?php } else { ?>

									<?php if ( ! empty( $products ) ) : ?>
								<select id="envira-config-standalone-template" name="envira_global_woocommerce_standalone_template">
									<option value="">(None)</option>
										<?php foreach ( (array) $products as $key => $product ) : ?>

									<option value="<?php echo esc_html( $product->get_id() ); ?>"<?php selected( $current_global_template, $product->get_id() ); ?>><?php echo esc_html( $product->get_name() ); ?></option>
									<?php endforeach; ?>
								</select>
										<?php wp_nonce_field( 'envira-woocommerce-nonce', 'envira-woocommerce-nonce' ); ?>
										<?php /* Translators: %s */ ?>
										<p><?php printf( esc_html__( 'Global default: any image you upload into a gallery will automatically generate a new WooCommerce product based on this selected product "template". %s', 'envira-gallery' ), '<a target="_blank" href="' . esc_url( $woocommerce_add_product_url ) . '">Create a new product</a> to get started.' ); ?></p>
								<?php else : ?>
									<?php /* translators: */ ?>
									<p class="description"><?php printf( esc_html__( 'You currently have no published WooCommerce products. %s', 'envira-gallery' ), '<a target="_blank" href="' . esc_url( $woocommerce_add_product_url ) . '">Create a new product</a> to get started.' ); ?></p>
								<?php endif; ?>

								<?php } ?>

							</td>
					</tr>
					<tr id="envira-config-woocommerce-product-delete">
							<th scope="row">
								<label for="envira-config-standalone-template"><?php esc_html_e( 'Delete WooCommerce Products', 'envira-standalone' ); ?></label>
							</th>
							<td>
								<input type="checkbox" name="envira_global_woocommerce_product_delete" id="envira-woocommerce-product-delete" value="1" <?php checked( $woocommerce_product_delete, 1 ); ?> />
								<p class="description"><?php esc_html_e( 'If enabled, Envira will delete a WooCommerce product associated with an image when the image is removed from a gallery.', 'envira-woocommerce' ); ?></p>
							</td>
					</tr>

						<tr>
							<th scope="row"><?php submit_button( __( 'Save', 'envira-woocommerce' ), 'primary', 'envira-woocommerce-save', false ); ?></th>
							<td>&nbsp;</td>
						</tr>
					</form>
				</tbody>
			</table>
		</div>
		<?php
	}

	/**
	 * Callback for saving the settings
	 *
	 * @since 1.0.0
	 */
	public function save() {

		// Check nonce is valid.
		if ( ! isset( $_POST['envira-woocommerce-nonce'] ) || ! wp_verify_nonce( sanitize_key( $_POST['envira-woocommerce-nonce'] ), 'envira-woocommerce-nonce' ) ) {
			return;
		}

		envira_update_setting( 'woocommerce_auto_create', empty( $_POST['envira_global_woocommerce_auto_create'] ) ? 0 : 1 );
		envira_update_setting( 'woocommerce_product_delete', empty( $_POST['envira_global_woocommerce_product_delete'] ) ? 0 : 1 );
		envira_update_setting( 'woocommerce_standalone_template', isset( $_POST['envira_global_woocommerce_standalone_template'] ) ? intval( $_POST['envira_global_woocommerce_standalone_template'] ) : false );

		if ( isset( $_POST['envira-woocommerce-nonce'] ) ) {
			add_action( 'envira_gallery_settings_woocommerce_tab_notice', [ $this, 'notice_update' ] );
		}
	}

	/**
	 * Outputs a message to tell the user that a new access token was generated.
	 *
	 * @since 1.0.0
	 */
	public function notice_generated_access_token() {

		?>
		<div class="notice updated below-h2">
			<p><?php esc_html_e( 'Access Token has been generated!', 'envira-woocommerce' ); ?></p>
		</div>
		<?php
	}
	/**
	 * Outputs a message to tell the user that a new access token was generated.
	 *
	 * @since 1.0.0
	 */
	public function notice_update() {

		?>
		<div class="notice updated below-h2">
			<p><?php esc_html_e( 'Settings have been updated!', 'envira-woocommerce' ); ?></p>
		</div>
		<?php
	}

	/**
	 * Outputs a message to tell the user that the user ID has been saved
	 *
	 * @since 1.0.0
	 */
	public function notice_updated_user_id() {

		?>
		<div class="notice updated below-h2">
			<p><?php esc_html_e( 'User ID updated successfully. Any publishing from WooCommerce will honor this user ID\'s roles and capabilities.', 'envira-woocommerce' ); ?></p>
		</div>
		<?php
	}

	/**
	 * Returns the singleton instance of the class.
	 *
	 * @since 1.0.0
	 *
	 * @return object The Envira_WooCommerce_Settings object.
	 */
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Envira_WooCommerce_Settings ) ) {
			self::$instance = new Envira_WooCommerce_Settings();
		}

		return self::$instance;
	}
}
